@@ -72,7 +72,7 @@ print "n_classes: %d" % n_classes
 
 # split into a training and testing set
 X_train, X_test, y_train, y_test = train_test_split(
-    X, y, test_fraction=0.25)
+    X, y, test_size=0.25)
 
 
 ###############################################################################
@@ -100,10 +100,8 @@ print "done in %0.3fs" % (time() - t0)
 
 print "Fitting the classifier to the training set"
 t0 = time()
-param_grid = {
- 'C': [1e3, 5e3, 1e4, 5e4, 1e5],
- 'gamma': [0.0001, 0.0005, 0.001, 0.005, 0.01, 0.1],
-}
+param_grid = {'C': [1e3, 5e3, 1e4, 5e4, 1e5],
+              'gamma': [0.0001, 0.0005, 0.001, 0.005, 0.01, 0.1], }
 clf = GridSearchCV(SVC(kernel='rbf', class_weight='auto'), param_grid)
 clf = clf.fit(X_train_pca, y_train)
 print "done in %0.3fs" % (time() - t0)
