@@ -84,68 +84,67 @@ d1 = datetime.datetime(2003, 01, 01)
 d2 = datetime.datetime(2008, 01, 01)
 
 symbol_dict = {
-        'TOT': 'Total',
-        'XOM': 'Exxon',
-        'CVX': 'Chevron',
-        'COP': 'ConocoPhillips',
-        'VLO': 'Valero Energy',
-        'MSFT': 'Microsoft',
-        'IBM': 'IBM',
-        'TWX': 'Time Warner',
-        'CMCSA': 'Comcast',
-        'CVC': 'Cablevision',
-        'YHOO': 'Yahoo',
-        'DELL': 'Dell',
-        'HPQ': 'HP',
-        'AMZN': 'Amazon',
-        'TM': 'Toyota',
-        'CAJ': 'Canon',
-        'MTU': 'Mitsubishi',
-        'SNE': 'Sony',
-        'F': 'Ford',
-        'HMC': 'Honda',
-        'NAV': 'Navistar',
-        'NOC': 'Northrop Grumman',
-        'BA': 'Boeing',
-        'KO': 'Coca Cola',
-        'MMM': '3M',
-        'MCD': 'Mc Donalds',
-        'PEP': 'Pepsi',
-        'KFT': 'Kraft Foods',
-        'K': 'Kellogg',
-        'UN': 'Unilever',
-        'MAR': 'Marriott',
-        'PG': 'Procter Gamble',
-        'CL': 'Colgate-Palmolive',
-        'NWS': 'News Corp',
-        'GE': 'General Electrics',
-        'WFC': 'Wells Fargo',
-        'JPM': 'JPMorgan Chase',
-        'AIG': 'AIG',
-        'AXP': 'American express',
-        'BAC': 'Bank of America',
-        'GS': 'Goldman Sachs',
-        'AAPL': 'Apple',
-        'SAP': 'SAP',
-        'CSCO': 'Cisco',
-        'TXN': 'Texas instruments',
-        'XRX': 'Xerox',
-        'LMT': 'Lookheed Martin',
-        'WMT': 'Wal-Mart',
-        'WAG': 'Walgreen',
-        'HD': 'Home Depot',
-        'GSK': 'GlaxoSmithKline',
-        'PFE': 'Pfizer',
-        'SNY': 'Sanofi-Aventis',
-        'NVS': 'Novartis',
-        'KMB': 'Kimberly-Clark',
-        'R': 'Ryder',
-        'GD': 'General Dynamics',
-        'RTN': 'Raytheon',
-        'CVS': 'CVS',
-        'CAT': 'Caterpillar',
-        'DD': 'DuPont de Nemours',
-    }
+    'TOT': 'Total',
+    'XOM': 'Exxon',
+    'CVX': 'Chevron',
+    'COP': 'ConocoPhillips',
+    'VLO': 'Valero Energy',
+    'MSFT': 'Microsoft',
+    'IBM': 'IBM',
+    'TWX': 'Time Warner',
+    'CMCSA': 'Comcast',
+    'CVC': 'Cablevision',
+    'YHOO': 'Yahoo',
+    'DELL': 'Dell',
+    'HPQ': 'HP',
+    'AMZN': 'Amazon',
+    'TM': 'Toyota',
+    'CAJ': 'Canon',
+    'MTU': 'Mitsubishi',
+    'SNE': 'Sony',
+    'F': 'Ford',
+    'HMC': 'Honda',
+    'NAV': 'Navistar',
+    'NOC': 'Northrop Grumman',
+    'BA': 'Boeing',
+    'KO': 'Coca Cola',
+    'MMM': '3M',
+    'MCD': 'Mc Donalds',
+    'PEP': 'Pepsi',
+    'KFT': 'Kraft Foods',
+    'K': 'Kellogg',
+    'UN': 'Unilever',
+    'MAR': 'Marriott',
+    'PG': 'Procter Gamble',
+    'CL': 'Colgate-Palmolive',
+    'NWS': 'News Corp',
+    'GE': 'General Electrics',
+    'WFC': 'Wells Fargo',
+    'JPM': 'JPMorgan Chase',
+    'AIG': 'AIG',
+    'AXP': 'American express',
+    'BAC': 'Bank of America',
+    'GS': 'Goldman Sachs',
+    'AAPL': 'Apple',
+    'SAP': 'SAP',
+    'CSCO': 'Cisco',
+    'TXN': 'Texas instruments',
+    'XRX': 'Xerox',
+    'LMT': 'Lookheed Martin',
+    'WMT': 'Wal-Mart',
+    'WAG': 'Walgreen',
+    'HD': 'Home Depot',
+    'GSK': 'GlaxoSmithKline',
+    'PFE': 'Pfizer',
+    'SNY': 'Sanofi-Aventis',
+    'NVS': 'Novartis',
+    'KMB': 'Kimberly-Clark',
+    'R': 'Ryder',
+    'GD': 'General Dynamics',
+    'RTN': 'Raytheon',
+    'CVS': 'CVS',
+    'CAT': 'Caterpillar',
+    'DD': 'DuPont de Nemours'}
 
 symbols, names = np.array(symbol_dict.items()).T
 
@@ -224,7 +223,7 @@ ax.add_collection(lc)
 # Add a label to each node. The challenge here is that we want to
 # position the labels to avoid overlap with other labels
 for index, (name, label, (x, y)) in enumerate(
-    zip(names, labels, embedding.T)):
+        zip(names, labels, embedding.T)):
 
     dx = x - embedding[0]
     dx[index] = 1
