@@ -93,7 +93,7 @@ class Controller(object):
         kernel_map = {0: "linear", 1: "rbf", 2: "poly"}
         if len(np.unique(y)) == 1:
             clf = svm.OneClassSVM(kernel=kernel_map[self.kernel.get()],
-                      gamma=gamma, coef0=coef0, degree=degree)
+                                  gamma=gamma, coef0=coef0, degree=degree)
             clf.fit(X)
         else:
             clf = svm.SVC(kernel=kernel_map[self.kernel.get()], C=C,
@@ -233,11 +233,9 @@ class View(object):
                                                  linestyles=linestyles))
         elif type == 1:
             self.contours.append(self.ax.contourf(X1, X2, Z, 10,
-                                             cmap=matplotlib.cm.bone,
-                                             origin='lower',
-                                             alpha=0.85))
-            self.contours.append(self.ax.contour(X1, X2, Z, [0.0],
-                                                 colors='k',
+                                                  cmap=matplotlib.cm.bone,
+                                                  origin='lower', alpha=0.85))
+            self.contours.append(self.ax.contour(X1, X2, Z, [0.0], colors='k',
                                                  linestyles=['solid']))
         else:
             raise ValueError("surface type unknown")
@@ -308,8 +306,8 @@ def get_parser():
     from optparse import OptionParser
     op = OptionParser()
     op.add_option("--output",
-              action="store", type="str", dest="output",
-              help="Path where to dump data.")
+                  action="store", type="str", dest="output",
+                  help="Path where to dump data.")
     return op
 
 
