@@ -48,40 +48,39 @@ labels = digits.target
 
 sample_size = 300
 
-print "n_digits: %d, \t n_samples %d, \t n_features %d" % (n_digits,
-                                                        n_samples, n_features)
+print("n_digits: %d, \t n_samples %d, \t n_features %d"
+      % (n_digits, n_samples, n_features))
 
 
-print 79 * '_'
-print ('% 9s' % 'init'
-      '    time  inertia    homo   compl  v-meas     ARI     AMI  silhouette')
+print(79 * '_')
+print('% 9s' % 'init'
+      '    time  inertia    homo   compl  v-meas     ARI AMI  silhouette')
 
 
 def bench_k_means(estimator, name, data):
     t0 = time()
     estimator.fit(data)
     print '% 9s   %.2fs    %i   %.3f   %.3f   %.3f   %.3f   %.3f    %.3f' % (
-         name, (time() - t0), estimator.inertia_,
-         metrics.homogeneity_score(labels, estimator.labels_),
-         metrics.completeness_score(labels, estimator.labels_),
-         metrics.v_measure_score(labels, estimator.labels_),
-         metrics.adjusted_rand_score(labels, estimator.labels_),
-         metrics.adjusted_mutual_info_score(labels,  estimator.labels_),
-         metrics.silhouette_score(data, estimator.labels_,
-                                  metric='euclidean',
-                                  sample_size=sample_size),
-         )
+        name, (time() - t0), estimator.inertia_,
+        metrics.homogeneity_score(labels, estimator.labels_),
+        metrics.completeness_score(labels, estimator.labels_),
+        metrics.v_measure_score(labels, estimator.labels_),
+        metrics.adjusted_rand_score(labels, estimator.labels_),
+        metrics.adjusted_mutual_info_score(labels,  estimator.labels_),
+        metrics.silhouette_score(data, estimator.labels_,
+                                 metric='euclidean',
+                                 sample_size=sample_size),)
 
-bench_k_means(KMeans(init='k-means++', k=n_digits, n_init=10),
+bench_k_means(KMeans(init='k-means++', n_clusters=n_digits, n_init=10),
               name="k-means++", data=data)
 
-bench_k_means(KMeans(init='random', k=n_digits, n_init=10),
+bench_k_means(KMeans(init='random', n_clusters=n_digits, n_init=10),
               name="random", data=data)
 
 # in this case the seeding of the centers is deterministic, hence we run the
 # kmeans algorithm only once with n_init=1
 pca = PCA(n_components=n_digits).fit(data)
-bench_k_means(KMeans(init=pca.components_, k=n_digits, n_init=1),
+bench_k_means(KMeans(init=pca.components_, n_clusters=n_digits, n_init=1),
               name="PCA-based",
               data=data)
 print 79 * '_'
@@ -90,7 +89,8 @@ print 79 * '_'
 # Visualize the results on PCA-reduced data
 
 reduced_data = PCA(n_components=2).fit_transform(data)
-kmeans = KMeans(init='k-means++', k=n_digits, n_init=10).fit(reduced_data)
+kmeans = KMeans(init='k-means++', n_clusters=n_digits, n_init=10)
+kmeans.fit(reduced_data)
 
 # Step size of the mesh. Decrease to increase the quality of the VQ.
 h = .02     # point in the mesh [x_min, m_max]x[y_min, y_max].
