@@ -4,12 +4,11 @@ Feature importances with forests of trees
 =========================================
 
 This examples shows the use of forests of trees to evaluate the importance of
-features on an artifical classification task. The red plots are the feature
-importances of each individual tree, and the blue plot is the feature
-importance of the whole forest.
+features on an artifical classification task. The red bars are the feature
+importances of the forest, along with their inter-trees variability.
 
-As expected, the knee in the blue plot suggests that 3 features are
-informative, while the remaining are not.
+As expected, the plot suggests that 3 features are informative, while the
+remaining are not.
 """
 print __doc__
 
@@ -35,6 +34,8 @@ forest = ExtraTreesClassifier(n_estimators=250,
 
 forest.fit(X, y)
 importances = forest.feature_importances_
+std = np.std([tree.feature_importances_ for tree in forest.estimators_],
+             axis=0)
 indices = np.argsort(importances)[::-1]
 
 # Print the feature ranking
@@ -43,13 +44,12 @@ print "Feature ranking:"
 for f in xrange(10):
     print "%d. feature %d (%f)" % (f + 1, indices[f], importances[indices[f]])
 
-# Plot the feature importances of the trees and of the forest
+# Plot the feature importances of the forest
 import pylab as pl
 pl.figure()
 pl.title("Feature importances")
-
-for tree in forest.estimators_:
-    pl.plot(xrange(10), tree.feature_importances_[indices], "r")
-
-pl.plot(xrange(10), importances[indices], "b")
+pl.bar(xrange(10), importances[indices],
+       color="r", yerr=std[indices], align="center")
+pl.xticks(xrange(10), indices)
+pl.xlim([-1, 10])
 pl.show()
