@@ -34,6 +34,10 @@ from sklearn import datasets
 X, y = datasets.make_hastie_10_2(n_samples=12000, random_state=1)
 X = X.astype(np.float32)
 
+# map labels from {-1, 1} to {0, 1}
+labels = np.unique(y)
+y = np.searchsorted(labels, y)
+
 X_train, X_test = X[:2000], X[2000:]
 y_train, y_test = y[:2000], y[2000:]
 
@@ -62,6 +66,7 @@ for label, color, setting in [('No shrinkage', 'orange',
     test_deviance = np.zeros((params['n_estimators'],), dtype=np.float64)
 
     for i, y_pred in enumerate(clf.staged_decision_function(X_test)):
+        # clf.loss_ assumes that y_test[i] in {0, 1}
         test_deviance[i] = clf.loss_(y_test, y_pred)
 
     pl.plot((np.arange(test_deviance.shape[0]) + 1)[::5], test_deviance[::5],
