@@ -15,8 +15,8 @@ from sklearn.pipeline import Pipeline
 
 # import some data to play with
 X, y = samples_generator.make_classification(
-        n_features=20, n_informative=3, n_redundant=0,
-        n_classes=4, n_clusters_per_class=2)
+    n_features=20, n_informative=3, n_redundant=0, n_classes=4,
+    n_clusters_per_class=2)
 
 # ANOVA SVM-C
 # 1) anova filter, take 3 best ranked features
