@@ -94,7 +94,7 @@ parameters = {
     # increase processing time in a combinatorial way
     'vect__max_df': (0.5, 0.75, 1.0),
     #'vect__max_features': (None, 5000, 10000, 50000),
-    'vect__max_n': (1, 2),  # words or bigrams
+    'vect__ngram_range': ((1, 1), (1, 2)),  # unigrams or bigrams
     #'tfidf__use_idf': (True, False),
     #'tfidf__norm': ('l1', 'l2'),
     'clf__alpha': (0.00001, 0.000001),
