@@ -3,6 +3,10 @@
 Lasso and Elastic Net for Sparse Signals
 ========================================
 
+Estimates Lasso and Elastic-Net regression models on a manually generated
+sparse signal corrupted with an additive noise. Estimated coefficients are
+compared with the ground-truth.
+
 """
 print __doc__
 
@@ -47,7 +51,7 @@ print "r^2 on test data : %f" % r2_score_lasso
 # ElasticNet
 from sklearn.linear_model import ElasticNet
 
-enet = ElasticNet(alpha=alpha, rho=0.7)
+enet = ElasticNet(alpha=alpha, l1_ratio=0.7)
 
 y_pred_enet = enet.fit(X_train, y_train).predict(X_test)
 r2_score_enet = r2_score(y_test, y_pred_enet)
@@ -58,6 +62,6 @@ pl.plot(enet.coef_, label='Elastic net coefficients')
 pl.plot(lasso.coef_, label='Lasso coefficients')
 pl.plot(coef, '--', label='original coefficients')
 pl.legend(loc='best')
-pl.title("Lasso R^2: %f, Elastic Net R^2: %f" % (r2_score_lasso,
-    r2_score_enet))
+pl.title("Lasso R^2: %f, Elastic Net R^2: %f"
+         % (r2_score_lasso, r2_score_enet))
 pl.show()
