@@ -23,7 +23,7 @@ diabetes = datasets.load_diabetes()
 X = diabetes.data
 y = diabetes.target
 
-X /= X.std(0)  # Standardize data (easier to set the rho parameter)
+X /= X.std(0)  # Standardize data (easier to set the l1_ratio parameter)
 
 ###############################################################################
 # Compute paths
@@ -41,12 +41,12 @@ alphas_positive_lasso = np.array([model.alpha for model in models])
 coefs_positive_lasso = np.array([model.coef_ for model in models])
 
 print "Computing regularization path using the elastic net..."
-models = enet_path(X, y, eps=eps, rho=0.8)
+models = enet_path(X, y, eps=eps, l1_ratio=0.8)
 alphas_enet = np.array([model.alpha for model in models])
 coefs_enet = np.array([model.coef_ for model in models])
 
 print "Computing regularization path using the positve elastic net..."
-models = enet_path(X, y, eps=eps, rho=0.8, positive=True)
+models = enet_path(X, y, eps=eps, l1_ratio=0.8, positive=True)
 alphas_positive_enet = np.array([model.alpha for model in models])
 coefs_positive_enet = np.array([model.coef_ for model in models])
 
@@ -89,6 +89,6 @@ pl.xlabel('-Log(lambda)')
 pl.ylabel('weights')
 pl.title('Elastic-Net and positive Elastic-Net')
 pl.legend((l1[-1], l2[-1]), ('Elastic-Net', 'positive Elastic-Net'),
-           loc='lower left')
+          loc='lower left')
 pl.axis('tight')
 pl.show()
