@@ -6,7 +6,11 @@ Plot Ridge coefficients as a function of the regularization
 .. currentmodule:: sklearn.linear_model
 
 Shows the effect of collinearity in the coefficients or the
-:class:`Ridge`. At the end of the path, as alpha tends toward zero
+:class:`Ridge`. Each color represents a different feature of the
+coefficient vector, and this is displayed as a function of the
+regularization parameter.
+
+At the end of the path, as alpha tends toward zero
 and the solution tends towards the ordinary least squares, coefficients
 exhibit big oscillations.
 """
