@@ -5,21 +5,27 @@ Explicit feature map approximation for RBF kernels
 
 .. currentmodule:: sklearn.kernel_approximation
 
-An example shows how to use :class:`RBFSampler` to appoximate the feature map
-of an RBF kernel for classification with an SVM on the digits dataset.  Results
-using a linear SVM in the original space, a linear SVM using the approximate
-mapping and using a kernelized SVM are compared.  Timings and accuracy for
-varying amounts of Monte Carlo samplings for the approximate mapping are shown.
+An example shows how to use :class:`RBFSampler` and :class:`Nystrom` to
+appoximate the feature map of an RBF kernel for classification with an SVM on
+the digits dataset. Results using a linear SVM in the original space, a linear
+SVM using the approximate mappings and using a kernelized SVM are compared.
+Timings and accuracy for varying amounts of Monte Carlo samplings (in the case
+of :class:`RBFSampler`, which uses random Fourier features) and different sized
+subsets of the training set (for :class:`Nystroem)` for the approximate mapping
+are shown.
+
+Please not that the dataset here is not large enough to show the benefits
+of kernel approximation, as the exact SVM is still reasonably fast.
 
 Sampling more dimensions clearly leads to better classification results, but
 comes at a greater cost. This means there is a tradeoff between runtime and
-accuracy, given by the parameter n_components.  Note that solving the Linear
+accuracy, given by the parameter n_components. Note that solving the Linear
 SVM and also the approximate kernel SVM could be greatly accelerated by using
 stochastic gradient descent via :class:`sklearn.linear_model.SGDClassifier`.
 This is not easily possible for the case of the kernelized SVM.
 
 The second plot visualized the decision surfaces of the RBF kernel SVM and
-the linear SVM with approximate kernel map.
+the linear SVM with approximate kernel maps.
 The plot shows decision surfaces of the classifiers projected onto
 the first two principal components of the data. This visualization should
 be taken with a grain of salt since it is just an interesting slice through
@@ -28,14 +34,14 @@ a datapoint (represented as a dot) does not necessarily be classified
 into the region it is lying in, since it will not lie on the plane
 that the first two principal components span.
 
-The usage of :class:`RBFSampler` is described in detail in
-:ref:`kernel_approximation`.
+The usage of :class:`RBFSampler` and :class:`Nystroem` is described in detail
+in :ref:`kernel_approximation`.
 
 """
 print __doc__
 
 # Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
-#         modified Andreas Mueller
+#         Andreas Mueller <amueller@ais.uni-bonn.de>
 # License: Simplified BSD
 
 # Standard scientific Python imports
@@ -45,7 +51,8 @@ from time import time
 
 # Import datasets, classifiers and performance metrics
 from sklearn import datasets, svm, pipeline
-from sklearn.kernel_approximation import RBFSampler
+from sklearn.kernel_approximation import (RBFSampler,
+                                          Nystroem)
 from sklearn.decomposition import PCA
 
 # The digits dataset
@@ -71,9 +78,13 @@ linear_svm = svm.LinearSVC()
 
 # create pipeline from kernel approximation
 # and linear svm
-feature_map = RBFSampler(gamma=.2, random_state=1)
-approx_kernel_svm = pipeline.Pipeline([("feature_map", feature_map),
-    ("svm", svm.LinearSVC())])
+feature_map_fourier = RBFSampler(gamma=.2, random_state=1)
+feature_map_nystroem = Nystroem(gamma=.2, random_state=1)
+fourier_approx_svm = pipeline.Pipeline([("feature_map", feature_map_fourier),
+                                        ("svm", svm.LinearSVC())])
+
+nystroem_approx_svm = pipeline.Pipeline([("feature_map", feature_map_nystroem),
+                                        ("svm", svm.LinearSVC())])
 
 # fit and predict using linear and kernel svm:
 
@@ -87,37 +98,52 @@ linear_svm.fit(data_train, targets_train)
 linear_svm_score = linear_svm.score(data_test, targets_test)
 linear_svm_time = time() - linear_svm_time
 
-sample_sizes = 50 * np.arange(1, 10)
-approx_kernel_scores = []
-approx_kernel_times = []
+sample_sizes = 30 * np.arange(1, 10)
+fourier_scores = []
+nystroem_scores = []
+fourier_times = []
+nystroem_times = []
 
 for D in sample_sizes:
-    approx_kernel_svm.set_params(feature_map__n_components=D)
-    approx_kernel_timing = time()
-    approx_kernel_svm.fit(data_train, targets_train)
-    approx_kernel_times.append(time() - approx_kernel_timing)
-    score = approx_kernel_svm.score(data_test, targets_test)
-    approx_kernel_scores.append(score)
+    fourier_approx_svm.set_params(feature_map__n_components=D)
+    nystroem_approx_svm.set_params(feature_map__n_components=D)
+    start = time()
+    nystroem_approx_svm.fit(data_train, targets_train)
+    nystroem_times.append(time() - start)
+
+    start = time()
+    fourier_approx_svm.fit(data_train, targets_train)
+    fourier_times.append(time() - start)
+
+    fourier_score = fourier_approx_svm.score(data_test, targets_test)
+    nystroem_score = nystroem_approx_svm.score(data_test, targets_test)
+    nystroem_scores.append(nystroem_score)
+    fourier_scores.append(fourier_score)
 
 # plot the results:
+pl.figure(figsize=(8, 8))
 accuracy = pl.subplot(211)
 # second y axis for timeings
 timescale = pl.subplot(212)
 
-accuracy.plot(sample_sizes, approx_kernel_scores, label="approx. kernel")
-timescale.plot(sample_sizes, approx_kernel_times, '--',
-        label='approx. kernel')
+accuracy.plot(sample_sizes, nystroem_scores, label="Nystroem approx. kernel")
+timescale.plot(sample_sizes, nystroem_times, '--',
+               label='Nystroem approx. kernel')
+
+accuracy.plot(sample_sizes, fourier_scores, label="Fourier approx. kernel")
+timescale.plot(sample_sizes, fourier_times, '--',
+               label='Fourier approx. kernel')
 
 # horizontal lines for exact rbf and linear kernels:
-accuracy.plot([sample_sizes[0], sample_sizes[-1]], [linear_svm_score,
-    linear_svm_score], label="linear svm")
-timescale.plot([sample_sizes[0], sample_sizes[-1]], [linear_svm_time,
-        linear_svm_time], '--', label='linear svm')
+accuracy.plot([sample_sizes[0], sample_sizes[-1]],
+              [linear_svm_score, linear_svm_score], label="linear svm")
+timescale.plot([sample_sizes[0], sample_sizes[-1]],
+               [linear_svm_time, linear_svm_time], '--', label='linear svm')
 
-accuracy.plot([sample_sizes[0], sample_sizes[-1]], [kernel_svm_score,
-    kernel_svm_score], label="rbf svm")
-timescale.plot([sample_sizes[0], sample_sizes[-1]], [kernel_svm_time,
-        kernel_svm_time], '--', label='rbf svm')
+accuracy.plot([sample_sizes[0], sample_sizes[-1]],
+              [kernel_svm_score, kernel_svm_score], label="rbf svm")
+timescale.plot([sample_sizes[0], sample_sizes[-1]],
+               [kernel_svm_time, kernel_svm_time], '--', label='rbf svm')
 
 # vertical line for dataset dimensionality = 64
 accuracy.plot([64, 64], [0.7, 1], label="n_features")
@@ -127,7 +153,7 @@ accuracy.set_title("Classification accuracy")
 timescale.set_title("Training times")
 accuracy.set_xlim(sample_sizes[0], sample_sizes[-1])
 accuracy.set_xticks(())
-accuracy.set_ylim(np.min(approx_kernel_scores), 1)
+accuracy.set_ylim(np.min(fourier_scores), 1)
 timescale.set_xlabel("Sampling steps = transformed feature dimension")
 accuracy.set_ylabel("Classification accuracy")
 timescale.set_ylabel("Training time in seconds")
@@ -152,15 +178,20 @@ flat_grid = grid.reshape(-1, data.shape[1])
 
 # title for the plots
 titles = ['SVC with rbf kernel',
-          'SVC (linear kernel) with rbf feature map\n n_components=100']
+          'SVC (linear kernel)\n with Fourier rbf feature map\n'
+          'n_components=100',
+          'SVC (linear kernel)\n with Nystroem rbf feature map\n'
+          'n_components=100']
 
+pl.tight_layout()
 pl.figure(figsize=(12, 5))
 
 # predict and plot
-for i, clf in enumerate((kernel_svm, approx_kernel_svm)):
+for i, clf in enumerate((kernel_svm, nystroem_approx_svm,
+                         fourier_approx_svm)):
     # Plot the decision boundary. For that, we will asign a color to each
     # point in the mesh [x_min, m_max]x[y_min, y_max].
-    pl.subplot(1, 2, i + 1)
+    pl.subplot(1, 3, i + 1)
     Z = clf.predict(flat_grid)
 
     # Put the result into a color plot
@@ -172,4 +203,5 @@ for i, clf in enumerate((kernel_svm, approx_kernel_svm)):
     pl.scatter(X[:, 0], X[:, 1], c=targets_train, cmap=pl.cm.Paired)
 
     pl.title(titles[i])
+pl.tight_layout()
 pl.show()
