@@ -17,11 +17,11 @@ import pylab as pl
 from sklearn import svm
 
 # we create 40 separable points
-np.random.seed(0)
+rng = np.random.RandomState(0)
 n_samples_1 = 1000
 n_samples_2 = 100
-X = np.r_[1.5 * np.random.randn(n_samples_1, 2),
-          0.5 * np.random.randn(n_samples_2, 2) + [2, 2]]
+X = np.r_[1.5 * rng.randn(n_samples_1, 2),
+          0.5 * rng.randn(n_samples_2, 2) + [2, 2]]
 y = [0] * (n_samples_1) + [1] * (n_samples_2)
 
 # fit the model and get the separating hyperplane
