@@ -30,8 +30,8 @@ Z = clf.decision_function(np.c_[xx.ravel(), yy.ravel()])
 Z = Z.reshape(xx.shape)
 
 pl.imshow(Z, interpolation='nearest',
-            extent=(xx.min(), xx.max(), yy.min(), yy.max()),
-            aspect='auto', origin='lower', cmap=pl.cm.PuOr_r)
+          extent=(xx.min(), xx.max(), yy.min(), yy.max()), aspect='auto',
+          origin='lower', cmap=pl.cm.PuOr_r)
 contours = pl.contour(xx, yy, Z, levels=[0], linewidths=2,
                       linetypes='--')
 pl.scatter(X[:, 0], X[:, 1], s=30, c=Y, cmap=pl.cm.Paired)
