@@ -5,18 +5,14 @@ from os.path import join
 
 import numpy
 
+from sklearn._build_utils import get_blas_info
+
 
 def configuration(parent_package='', top_path=None):
     from numpy.distutils.misc_util import Configuration
-    from numpy.distutils.system_info import get_info
 
-    blas_info = get_info('blas_opt', 0)
-    if (not blas_info) or (
-        ('NO_ATLAS_INFO', 1) in blas_info.get('define_macros', [])):
-        cblas_libs = ['cblas']
-        blas_info.pop('libraries', None)
-    else:
-        cblas_libs = blas_info.pop('libraries', [])
+    cblas_libs, blas_info = get_blas_info()
+
     libraries = []
     if os.name == 'posix':
         cblas_libs.append('m')
