@@ -17,7 +17,7 @@ from scipy import linalg
 
 from ..base import BaseEstimator
 from ..utils import array2d
-from ..utils.extmath import fast_logdet
+from ..utils.extmath import fast_logdet, pinvh
 
 
 def log_likelihood(emp_cov, precision):
@@ -57,8 +57,8 @@ def empirical_covariance(X, assume_centered=False):
     X = np.asarray(X)
     if X.ndim == 1:
         X = np.reshape(X, (1, -1))
-        warnings.warn("Only one sample available. " \
-                          "You may want to reshape your data array")
+        warnings.warn("Only one sample available. "
+                      "You may want to reshape your data array")
 
     if assume_centered:
         covariance = np.dot(X.T, X) / X.shape[0]
@@ -75,6 +75,11 @@ class EmpiricalCovariance(BaseEstimator):
     ----------
     store_precision : bool
         Specifies if the estimated precision is stored
+    assume_centered: bool
+        If True, data are not centered before computation.
+        Useful when working with data whose mean is almost, but not exactly
+        zero.
+        If False (default), data are centered before computation.
 
     Attributes
     ----------
@@ -87,23 +92,10 @@ class EmpiricalCovariance(BaseEstimator):
 
     """
     def __init__(self, store_precision=True, assume_centered=False):
-        """
-
-        Parameters
-        ----------
-        store_precision: bool
-          Specify if the estimated precision is stored
-        assume_centered: Boolean
-          If True, data are not centered before computation.
-          Useful when working with data whose mean is almost, but not exactly
-          zero.
-          If False, data are centered before computation.
-
-        """
         self.store_precision = store_precision
         self.assume_centered = assume_centered
 
-    def _set_estimates(self, covariance):
+    def _set_covariance(self, covariance):
         """Saves the covariance and precision estimates
 
         Storage is done accordingly to `self.store_precision`.
@@ -121,19 +113,35 @@ class EmpiricalCovariance(BaseEstimator):
         self.covariance_ = covariance
         # set precision
         if self.store_precision:
-            self.precision_ = linalg.pinv(covariance)
+            self.precision_ = pinvh(covariance)
         else:
             self.precision_ = None
 
-    def fit(self, X):
+    def get_precision(self):
+        """Getter for the precision matrix.
+
+        Returns
+        -------
+        precision_: array-like,
+          The precision matrix associated to the current covariance object.
+
+        """
+        if self.store_precision:
+            precision = self.precision_
+        else:
+            precision = pinvh(self.covariance_)
+        return precision
+
+    def fit(self, X, y=None):
         """Fits the Maximum Likelihood Estimator covariance model
         according to the given training data and parameters.
 
         Parameters
         ----------
-        X : array-like, shape = [n_samples, n_features]
-            Training data, where n_samples is the number of samples and
-            n_features is the number of features.
+        X: array-like, shape = [n_samples, n_features]
+          Training data, where n_samples is the number of samples and
+          n_features is the number of features.
+        y: not used, present for API consistence purpose.
 
         Returns
         -------
@@ -147,19 +155,22 @@ class EmpiricalCovariance(BaseEstimator):
             self.location_ = X.mean(0)
         covariance = empirical_covariance(
             X, assume_centered=self.assume_centered)
-        self._set_estimates(covariance)
+        self._set_covariance(covariance)
 
         return self
 
-    def score(self, X_test, assume_centered=False):
+    def score(self, X_test, y=None):
         """Computes the log-likelihood of a gaussian data set with
         `self.covariance_` as an estimator of its covariance matrix.
 
         Parameters
         ----------
-        X_test : array-like, shape = [n_samples, n_features]
-            Test data of which we compute the likelihood, where n_samples is
-            the number of samples and n_features is the number of features.
+        X_test: array-like, shape = [n_samples, n_features]
+          Test data of which we compute the likelihood, where n_samples is
+          the number of samples and n_features is the number of features.
+          X_test is assumed to be drawn from the same distribution than
+          tha data used in fit (including centering).
+        y: not used, present for API consistence purpose.
 
         Returns
         -------
@@ -169,13 +180,10 @@ class EmpiricalCovariance(BaseEstimator):
 
         """
         # compute empirical covariance of the test set
-        test_cov = empirical_covariance(X_test,
-                                        assume_centered=assume_centered)
+        test_cov = empirical_covariance(
+            X_test - self.location_, assume_centered=True)
         # compute log likelihood
-        if self.store_precision:
-            res = log_likelihood(test_cov, self.precision_)
-        else:
-            res = log_likelihood(test_cov, linalg.pinv(self.covariance_))
+        res = log_likelihood(test_cov, self.get_precision())
 
         return res
 
@@ -238,6 +246,8 @@ class EmpiricalCovariance(BaseEstimator):
         ----------
         observations: array-like, shape = [n_observations, n_features]
           The observations, the Mahalanobis distances of the which we compute.
+          Observations are assumed to be drawn from the same distribution than
+          tha data used in fit (including centering).
 
         Returns
         -------
@@ -245,14 +255,10 @@ class EmpiricalCovariance(BaseEstimator):
             Mahalanobis distances of the observations.
 
         """
-        # get precision
-        if self.store_precision:
-            precision = self.precision_
-        else:
-            precision = linalg.pinv(self.covariance_)
-
+        precision = self.get_precision()
         # compute mahalanobis distances
+        centered_obs = observations - self.location_
         mahalanobis_dist = np.sum(
-            np.dot(observations, precision) * observations, 1)
+            np.dot(centered_obs, precision) * centered_obs, 1)
 
         return mahalanobis_dist
