@@ -13,10 +13,11 @@ import time
 import numpy as np
 from scipy import linalg
 
-from .empirical_covariance_ import empirical_covariance, \
-                EmpiricalCovariance, log_likelihood
+from .empirical_covariance_ import (empirical_covariance, EmpiricalCovariance,
+                                    log_likelihood)
 
 from ..utils import ConvergenceWarning
+from ..utils.extmath import pinvh
 from ..linear_model import lars_path
 from ..linear_model import cd_fast
 from ..cross_validation import check_cv, cross_val_score
@@ -143,7 +144,7 @@ def graph_lasso(emp_cov, alpha, cov_init=None, mode='cd', tol=1e-4,
     covariance_ *= 0.95
     diagonal = emp_cov.flat[::n_features + 1]
     covariance_.flat[::n_features + 1] = diagonal
-    precision_ = linalg.pinv(covariance_)
+    precision_ = pinvh(covariance_)
 
     indices = np.arange(n_features)
     costs = list()
@@ -161,25 +162,25 @@ def graph_lasso(emp_cov, alpha, cov_init=None, mode='cd', tol=1e-4,
                     if mode == 'cd':
                         # Use coordinate descent
                         coefs = -(precision_[indices != idx, idx]
-                                    / (precision_[idx, idx] + 1000 * eps))
+                                  / (precision_[idx, idx] + 1000 * eps))
                         coefs, _, _ = cd_fast.enet_coordinate_descent_gram(
-                                            coefs, alpha, 0, sub_covariance,
-                                            row, row, max_iter, tol)
+                            coefs, alpha, 0, sub_covariance, row, row,
+                            max_iter, tol)
                     else:
                         # Use LARS
-                        _, _, coefs = lars_path(sub_covariance, row,
-                                            Xy=row, Gram=sub_covariance,
-                                            alpha_min=alpha / (n_features - 1),
-                                            copy_Gram=True,
-                                            method='lars')
+                        _, _, coefs = lars_path(
+                            sub_covariance, row, Xy=row, Gram=sub_covariance,
+                            alpha_min=alpha / (n_features - 1), copy_Gram=True,
+                            method='lars')
                         coefs = coefs[:, -1]
                 # Update the precision matrix
-                precision_[idx, idx] = 1. / (covariance_[idx, idx] -
-                            np.dot(covariance_[indices != idx, idx], coefs))
-                precision_[indices != idx, idx] = \
-                                            - precision_[idx, idx] * coefs
-                precision_[idx, indices != idx] = \
-                                            - precision_[idx, idx] * coefs
+                precision_[idx, idx] = (
+                    1. / (covariance_[idx, idx]
+                          - np.dot(covariance_[indices != idx, idx], coefs)))
+                precision_[indices != idx, idx] = (- precision_[idx, idx]
+                                                   * coefs)
+                precision_[idx, indices != idx] = (- precision_[idx, idx]
+                                                   * coefs)
                 coefs = np.dot(sub_covariance, coefs)
                 covariance_[idx, indices != idx] = coefs
                 covariance_[indices != idx, idx] = coefs
@@ -188,22 +189,21 @@ def graph_lasso(emp_cov, alpha, cov_init=None, mode='cd', tol=1e-4,
             if verbose:
                 print (
                     '[graph_lasso] Iteration % 3i, cost % 3.2e, dual gap %.3e'
-                                                % (i, cost, d_gap))
+                    % (i, cost, d_gap))
             if return_costs:
                 costs.append((cost, d_gap))
             if np.abs(d_gap) < tol:
                 break
             if not np.isfinite(cost) and i > 0:
                 raise FloatingPointError('Non SPD result: the system is '
-                                    'too ill-conditioned for this solver')
+                                         'too ill-conditioned for this solver')
         else:
             warnings.warn('graph_lasso: did not converge after %i iteration:'
-                            'dual gap: %.3e' % (max_iter, d_gap),
-                            ConvergenceWarning)
+                          'dual gap: %.3e' % (max_iter, d_gap),
+                          ConvergenceWarning)
     except FloatingPointError as e:
         e.args = (e.args[0]
-                  + '. The system is too ill-conditioned for this solver',
-                 )
+                  + '. The system is too ill-conditioned for this solver',)
         raise e
     if return_costs:
         return covariance_, precision_, costs
@@ -258,18 +258,16 @@ class GraphLasso(EmpiricalCovariance):
 
     def fit(self, X, y=None):
         emp_cov = empirical_covariance(X)
-        self.covariance_, self.precision_ = graph_lasso(emp_cov,
-                                        alpha=self.alpha, mode=self.mode,
-                                        tol=self.tol, max_iter=self.max_iter,
-                                        verbose=self.verbose,
-                                        )
+        self.covariance_, self.precision_ = graph_lasso(
+            emp_cov, alpha=self.alpha, mode=self.mode, tol=self.tol,
+            max_iter=self.max_iter, verbose=self.verbose,)
         return self
 
 
 ###############################################################################
 # Cross-validation with GraphLasso
 def graph_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
-                 tol=1e-4, max_iter=100, verbose=False):
+                     tol=1e-4, max_iter=100, verbose=False):
     """l1-penalized covariance estimator along a path of decreasing alphas
 
     Parameters
@@ -317,10 +315,9 @@ def graph_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
     for alpha in alphas:
         try:
             # Capture the errors, and move on
-            covariance_, precision_ = graph_lasso(emp_cov, alpha=alpha,
-                                    cov_init=covariance_, mode=mode, tol=tol,
-                                    max_iter=max_iter,
-                                    verbose=inner_verbose)
+            covariance_, precision_ = graph_lasso(
+                emp_cov, alpha=alpha, cov_init=covariance_, mode=mode, tol=tol,
+                max_iter=max_iter, verbose=inner_verbose)
             covariances_.append(covariance_)
             precisions_.append(precision_)
             if X_test is not None:
@@ -337,10 +334,10 @@ def graph_lasso_path(X, alphas, cov_init=None, X_test=None, mode='cd',
             sys.stderr.write('.')
         elif verbose:
             if X_test is not None:
-                print '[graph_lasso_path] alpha: %.2e, score: %.2e' % (alpha,
-                                                            this_score)
+                print('[graph_lasso_path] alpha: %.2e, score: %.2e'
+                      % (alpha, this_score))
             else:
-                print '[graph_lasso_path] alpha: %.2e' % alpha
+                print('[graph_lasso_path] alpha: %.2e' % alpha)
     if X_test is not None:
         return covariances_, precisions_, scores_
     return covariances_, precisions_
@@ -441,9 +438,8 @@ class GraphLassoCV(GraphLasso):
             n_refinements = self.n_refinements
             alpha_1 = alpha_max(emp_cov)
             alpha_0 = 1e-2 * alpha_1
-            alphas = np.logspace(np.log10(alpha_0),
-                                            np.log10(alpha_1),
-                                            n_alphas)[::-1]
+            alphas = np.logspace(np.log10(alpha_0), np.log10(alpha_1),
+                                 n_alphas)[::-1]
         covs_init = (None, None, None)
 
         t0 = time.time()
@@ -496,7 +492,7 @@ class GraphLassoCV(GraphLasso):
                 alpha_0 = path[1][0]
                 covs_init = path[0][-1]
             elif (best_index == last_finite_idx
-                        and not best_index == len(path) - 1):
+                    and not best_index == len(path) - 1):
                 # We have non-converged models on the upper bound of the
                 # grid, we need to refine the grid there
                 alpha_1 = path[best_index][0]
@@ -515,7 +511,7 @@ class GraphLassoCV(GraphLasso):
             alphas = alphas[1:-1]
             if self.verbose and n_refinements > 1:
                 print '[GraphLassoCV] Done refinement % 2i out of %i: % 3is'\
-                        % (i + 1, n_refinements, time.time() - t0)
+                      % (i + 1, n_refinements, time.time() - t0)
 
         path = zip(*path)
         cv_scores = list(path[1])
@@ -531,7 +527,7 @@ class GraphLassoCV(GraphLasso):
         self.cv_alphas_ = alphas
 
         # Finally fit the model with the selected alpha
-        self.covariance_, self.precision_ = graph_lasso(emp_cov,
-                        alpha=best_alpha, mode=self.mode, tol=self.tol,
-                        max_iter=self.max_iter, verbose=inner_verbose)
+        self.covariance_, self.precision_ = graph_lasso(
+            emp_cov, alpha=best_alpha, mode=self.mode, tol=self.tol,
+            max_iter=self.max_iter, verbose=inner_verbose)
         return self
