@@ -4,15 +4,20 @@
 #
 # License: BSD Style.
 
-from numpy.testing import assert_almost_equal, assert_array_almost_equal
-
 import numpy as np
+import warnings
+
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_raises
 
 from sklearn import datasets
 from sklearn.covariance import empirical_covariance, EmpiricalCovariance, \
-    ShrunkCovariance, shrunk_covariance, LedoitWolf, ledoit_wolf, OAS, oas
+    ShrunkCovariance, shrunk_covariance, \
+    LedoitWolf, ledoit_wolf, ledoit_wolf_shrinkage, OAS, oas
 
-X = datasets.load_iris().data
+X = datasets.load_diabetes().data
 X_1d = X[:, 0]
 n_samples, n_features = X.shape
 
@@ -24,20 +29,23 @@ def test_covariance():
     # test covariance fit from data
     cov = EmpiricalCovariance()
     cov.fit(X)
-    assert_array_almost_equal(empirical_covariance(X), cov.covariance_, 4)
-    assert_almost_equal(cov.error_norm(empirical_covariance(X)), 0)
+    emp_cov = empirical_covariance(X)
+    assert_array_almost_equal(emp_cov, cov.covariance_, 4)
+    assert_almost_equal(cov.error_norm(emp_cov), 0)
     assert_almost_equal(
-        cov.error_norm(empirical_covariance(X), norm='spectral'), 0)
+        cov.error_norm(emp_cov, norm='spectral'), 0)
     assert_almost_equal(
-        cov.error_norm(empirical_covariance(X), norm='frobenius'), 0)
+        cov.error_norm(emp_cov, norm='frobenius'), 0)
     assert_almost_equal(
-        cov.error_norm(empirical_covariance(X), scaling=False), 0)
+        cov.error_norm(emp_cov, scaling=False), 0)
     assert_almost_equal(
-        cov.error_norm(empirical_covariance(X), squared=False), 0)
+        cov.error_norm(emp_cov, squared=False), 0)
+    assert_raises(NotImplementedError,
+                  cov.error_norm, emp_cov, norm='foo')
     # Mahalanobis distances computation test
     mahal_dist = cov.mahalanobis(X)
-    assert(np.amax(mahal_dist) < 250)
-    assert(np.amin(mahal_dist) > 50)
+    print np.amin(mahal_dist), np.amax(mahal_dist)
+    assert(np.amin(mahal_dist) > 0)
 
     # test with n_features = 1
     X_1d = X[:, 0].reshape((-1, 1))
@@ -48,11 +56,22 @@ def test_covariance():
     assert_almost_equal(
         cov.error_norm(empirical_covariance(X_1d), norm='spectral'), 0)
 
+    # test with one sample
+    X_1sample = np.arange(5)
+    cov = EmpiricalCovariance()
+    with warnings.catch_warnings(record=True):
+        cov.fit(X_1sample)
+
     # test integer type
     X_integer = np.asarray([[0, 1], [1, 0]])
     result = np.asarray([[0.25, -0.25], [-0.25, 0.25]])
     assert_array_almost_equal(empirical_covariance(X_integer), result)
 
+    # test centered case
+    cov = EmpiricalCovariance(assume_centered=True)
+    cov.fit(X)
+    assert_array_equal(cov.location_, np.zeros(X.shape[1]))
+
 
 def test_shrunk_covariance():
     """Tests ShrunkCovariance module on a simple dataset.
@@ -93,24 +112,31 @@ def test_ledoit_wolf():
 
     """
     # test shrinkage coeff on a simple data set
-    lw = LedoitWolf()
-    lw.fit(X, assume_centered=True)
-    assert_almost_equal(lw.shrinkage_, 0.00192, 4)
-    assert_almost_equal(lw.score(X, assume_centered=True), -2.89795, 4)
+    X_centered = X - X.mean(axis=0)
+    lw = LedoitWolf(assume_centered=True)
+    lw.fit(X_centered)
+    shrinkage_ = lw.shrinkage_
+    score_ = lw.score(X_centered)
+    assert_almost_equal(ledoit_wolf_shrinkage(X_centered,
+                                              assume_centered=True),
+                        shrinkage_)
+    assert_almost_equal(ledoit_wolf_shrinkage(X_centered, assume_centered=True,
+                                              block_size=6),
+                        shrinkage_)
     # compare shrunk covariance obtained from data and from MLE estimate
-    lw_cov_from_mle, lw_shinkrage_from_mle = ledoit_wolf(X,
-                                                        assume_centered=True)
+    lw_cov_from_mle, lw_shinkrage_from_mle = ledoit_wolf(X_centered,
+                                                         assume_centered=True)
     assert_array_almost_equal(lw_cov_from_mle, lw.covariance_, 4)
     assert_almost_equal(lw_shinkrage_from_mle, lw.shrinkage_)
     # compare estimates given by LW and ShrunkCovariance
-    scov = ShrunkCovariance(shrinkage=lw.shrinkage_)
-    scov.fit(X, assume_centered=True)
+    scov = ShrunkCovariance(shrinkage=lw.shrinkage_, assume_centered=True)
+    scov.fit(X_centered)
     assert_array_almost_equal(scov.covariance_, lw.covariance_, 4)
 
     # test with n_features = 1
     X_1d = X[:, 0].reshape((-1, 1))
-    lw = LedoitWolf()
-    lw.fit(X_1d, assume_centered=True)
+    lw = LedoitWolf(assume_centered=True)
+    lw.fit(X_1d)
     lw_cov_from_mle, lw_shinkrage_from_mle = ledoit_wolf(X_1d,
                                                          assume_centered=True)
     assert_array_almost_equal(lw_cov_from_mle, lw.covariance_, 4)
@@ -118,17 +144,23 @@ def test_ledoit_wolf():
     assert_array_almost_equal((X_1d ** 2).sum() / n_samples, lw.covariance_, 4)
 
     # test shrinkage coeff on a simple data set (without saving precision)
-    lw = LedoitWolf(store_precision=False)
-    lw.fit(X, assume_centered=True)
-    assert_almost_equal(lw.score(X, assume_centered=True), -2.89795, 4)
+    lw = LedoitWolf(store_precision=False, assume_centered=True)
+    lw.fit(X_centered)
+    assert_almost_equal(lw.score(X_centered), score_, 4)
     assert(lw.precision_ is None)
 
+    # (too) large data set
+    X_large = np.ones((20, 200))
+    assert_raises(MemoryError, ledoit_wolf, X_large, block_size=100)
+
     # Same tests without assuming centered data
     # test shrinkage coeff on a simple data set
     lw = LedoitWolf()
     lw.fit(X)
-    assert_almost_equal(lw.shrinkage_, 0.007582, 4)
-    assert_almost_equal(lw.score(X), 2.243483, 4)
+    assert_almost_equal(lw.shrinkage_, shrinkage_, 4)
+    assert_almost_equal(lw.shrinkage_, ledoit_wolf_shrinkage(X))
+    assert_almost_equal(lw.shrinkage_, ledoit_wolf(X)[1])
+    assert_almost_equal(lw.score(X), score_, 4)
     # compare shrunk covariance obtained from data and from MLE estimate
     lw_cov_from_mle, lw_shinkrage_from_mle = ledoit_wolf(X)
     assert_array_almost_equal(lw_cov_from_mle, lw.covariance_, 4)
@@ -147,10 +179,16 @@ def test_ledoit_wolf():
     assert_almost_equal(lw_shinkrage_from_mle, lw.shrinkage_)
     assert_array_almost_equal(empirical_covariance(X_1d), lw.covariance_, 4)
 
+    # test with one sample
+    X_1sample = np.arange(5)
+    lw = LedoitWolf()
+    with warnings.catch_warnings(record=True):
+        lw.fit(X_1sample)
+
     # test shrinkage coeff on a simple data set (without saving precision)
     lw = LedoitWolf(store_precision=False)
     lw.fit(X)
-    assert_almost_equal(lw.score(X), 2.2434839, 4)
+    assert_almost_equal(lw.score(X), score_, 4)
     assert(lw.precision_ is None)
 
 
@@ -159,40 +197,42 @@ def test_oas():
 
     """
     # test shrinkage coeff on a simple data set
-    oa = OAS()
-    oa.fit(X, assume_centered=True)
-    assert_almost_equal(oa.shrinkage_, 0.018740, 4)
-    assert_almost_equal(oa.score(X, assume_centered=True), -5.03605, 4)
+    X_centered = X - X.mean(axis=0)
+    oa = OAS(assume_centered=True)
+    oa.fit(X_centered)
+    shrinkage_ = oa.shrinkage_
+    score_ = oa.score(X_centered)
     # compare shrunk covariance obtained from data and from MLE estimate
-    oa_cov_from_mle, oa_shinkrage_from_mle = oas(X, assume_centered=True)
+    oa_cov_from_mle, oa_shinkrage_from_mle = oas(X_centered,
+                                                 assume_centered=True)
     assert_array_almost_equal(oa_cov_from_mle, oa.covariance_, 4)
     assert_almost_equal(oa_shinkrage_from_mle, oa.shrinkage_)
     # compare estimates given by OAS and ShrunkCovariance
-    scov = ShrunkCovariance(shrinkage=oa.shrinkage_)
-    scov.fit(X, assume_centered=True)
+    scov = ShrunkCovariance(shrinkage=oa.shrinkage_, assume_centered=True)
+    scov.fit(X_centered)
     assert_array_almost_equal(scov.covariance_, oa.covariance_, 4)
 
     # test with n_features = 1
     X_1d = X[:, 0].reshape((-1, 1))
-    oa = OAS()
-    oa.fit(X_1d, assume_centered=True)
+    oa = OAS(assume_centered=True)
+    oa.fit(X_1d)
     oa_cov_from_mle, oa_shinkrage_from_mle = oas(X_1d, assume_centered=True)
     assert_array_almost_equal(oa_cov_from_mle, oa.covariance_, 4)
     assert_almost_equal(oa_shinkrage_from_mle, oa.shrinkage_)
     assert_array_almost_equal((X_1d ** 2).sum() / n_samples, oa.covariance_, 4)
 
     # test shrinkage coeff on a simple data set (without saving precision)
-    oa = OAS(store_precision=False)
-    oa.fit(X, assume_centered=True)
-    assert_almost_equal(oa.score(X, assume_centered=True), -5.03605, 4)
+    oa = OAS(store_precision=False, assume_centered=True)
+    oa.fit(X_centered)
+    assert_almost_equal(oa.score(X_centered), score_, 4)
     assert(oa.precision_ is None)
 
     ### Same tests without assuming centered data
     # test shrinkage coeff on a simple data set
     oa = OAS()
     oa.fit(X)
-    assert_almost_equal(oa.shrinkage_, 0.020236, 4)
-    assert_almost_equal(oa.score(X), 2.079025, 4)
+    assert_almost_equal(oa.shrinkage_, shrinkage_, 4)
+    assert_almost_equal(oa.score(X), score_, 4)
     # compare shrunk covariance obtained from data and from MLE estimate
     oa_cov_from_mle, oa_shinkrage_from_mle = oas(X)
     assert_array_almost_equal(oa_cov_from_mle, oa.covariance_, 4)
@@ -211,8 +251,14 @@ def test_oas():
     assert_almost_equal(oa_shinkrage_from_mle, oa.shrinkage_)
     assert_array_almost_equal(empirical_covariance(X_1d), oa.covariance_, 4)
 
+    # test with one sample
+    X_1sample = np.arange(5)
+    oa = OAS()
+    with warnings.catch_warnings(record=True):
+        oa.fit(X_1sample)
+
     # test shrinkage coeff on a simple data set (without saving precision)
     oa = OAS(store_precision=False)
     oa.fit(X)
-    assert_almost_equal(oa.score(X), 2.079025, 4)
+    assert_almost_equal(oa.score(X), score_, 4)
     assert(oa.precision_ is None)
