@@ -14,6 +14,7 @@ from .base import get_data_home
 from .base import clear_data_home
 from .base import load_sample_images
 from .base import load_sample_image
+from .covtype import fetch_covtype
 from .mlcomp import load_mlcomp
 from .lfw import load_lfw_pairs
 from .lfw import load_lfw_people
@@ -57,6 +58,7 @@ __all__ = ['clear_data_home',
            'fetch_olivetti_faces',
            'fetch_species_distributions',
            'fetch_california_housing',
+           'fetch_covtype',
            'get_data_home',
            'load_20newsgroups',
            'load_boston',
