@@ -7,6 +7,7 @@ Generate samples of synthetic data sets.
 # License: BSD 3 clause
 
 from itertools import product
+import numbers
 import numpy as np
 from scipy import linalg
 
@@ -111,13 +112,14 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
     # Count features, clusters and samples
     if n_informative + n_redundant + n_repeated > n_features:
         raise ValueError("Number of informative, redundant and repeated "
-            "features must sum to less than the number of total features")
+                         "features must sum to less than the number of total"
+                         " features")
     if 2 ** n_informative < n_classes * n_clusters_per_class:
         raise ValueError("n_classes * n_clusters_per_class must"
-            "be smaller or equal 2 ** n_informative")
+                         "be smaller or equal 2 ** n_informative")
     if weights and len(weights) not in [n_classes, n_classes - 1]:
         raise ValueError("Weights specified but incompatible with number "
-                "of classes.")
+                         "of classes.")
 
     n_useless = n_features - n_informative - n_redundant - n_repeated
     n_clusters = n_classes * n_clusters_per_class
@@ -185,7 +187,7 @@ def make_classification(n_samples=100, n_features=20, n_informative=2,
     if n_redundant > 0:
         B = 2 * generator.rand(n_informative, n_redundant) - 1
         X[:, n_informative:n_informative + n_redundant] = \
-                                            np.dot(X[:, :n_informative], B)
+            np.dot(X[:, :n_informative], B)
 
     # Repeat some features
     if n_repeated > 0:
@@ -363,9 +365,10 @@ def make_hastie_10_2(n_samples=12000, random_state=None):
     return X, y
 
 
-def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
-                    effective_rank=None, tail_strength=0.5, noise=0.0,
-                    shuffle=True, coef=False, random_state=None):
+def make_regression(n_samples=100, n_features=100, n_informative=10,
+                    n_targets=1, bias=0.0, effective_rank=None,
+                    tail_strength=0.5, noise=0.0, shuffle=True, coef=False,
+                    random_state=None):
     """Generate a random regression problem.
 
     The input set can either be well conditioned (by default) or have a low
@@ -389,6 +392,10 @@ def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
         The number of informative features, i.e., the number of features used
         to build the linear model used to generate the output.
 
+    n_targets : int, optional (default=1)
+        The number of regression targets, i.e., the dimension of the y output
+        vector associated with a sample. By default, the output is a scalar.
+
     bias : float, optional (default=0.0)
         The bias term in the underlying linear model.
 
@@ -426,10 +433,10 @@ def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
     X : array of shape [n_samples, n_features]
         The input samples.
 
-    y : array of shape [n_samples]
+    y : array of shape [n_samples] or [n_samples, n_targets]
         The output values.
 
-    coef : array of shape [n_features], optional
+    coef : array of shape [n_features] or [n_features, n_targets], optional
         The coefficient of the underlying linear model. It is returned only if
         coef is True.
     """
@@ -450,8 +457,9 @@ def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
     # Generate a ground truth model with only n_informative features being non
     # zeros (the other features are not correlated to y and should be ignored
     # by a sparsifying regularizers such as L1 or elastic net)
-    ground_truth = np.zeros(n_features)
-    ground_truth[:n_informative] = 100 * generator.rand(n_informative)
+    ground_truth = np.zeros((n_features, n_targets))
+    ground_truth[:n_informative, :] = 100 * generator.rand(n_informative,
+                                                           n_targets)
 
     y = np.dot(X, ground_truth) + bias
 
@@ -468,16 +476,18 @@ def make_regression(n_samples=100, n_features=100, n_informative=10, bias=0.0,
         X[:, :] = X[:, indices]
         ground_truth = ground_truth[indices]
 
+    y = np.squeeze(y)
+
     if coef:
-        return X, y, ground_truth
+        return X, y, np.squeeze(ground_truth)
 
     else:
         return X, y
 
 
 def make_circles(n_samples=100, shuffle=True, noise=None, random_state=None,
-        factor=.8):
-    """Make a large circle containing a smaller circle in 2di
+                 factor=.8):
+    """Make a large circle containing a smaller circle in 2d.
 
     A simple toy dataset to visualize clustering and classification
     algorithms.
@@ -495,27 +505,31 @@ def make_circles(n_samples=100, shuffle=True, noise=None, random_state=None,
 
     factor : double < 1 (default=.8)
         Scale factor between inner and outer circle.
+
+    Returns
+    -------
+    X : array of shape [n_samples, 2]
+        The generated samples.
+
+    y : array of shape [n_samples]
+        The integer labels (0 or 1) for class membership of each sample.
     """
 
     if factor > 1 or factor < 0:
         raise ValueError("'factor' has to be between 0 and 1.")
 
-    n_samples_out = int(n_samples / float(1 + factor))
-    n_samples_in = n_samples - n_samples_out
-
     generator = check_random_state(random_state)
-
     # so as not to have the first point = last point, we add one and then
     # remove it.
-    n_samples_out, n_samples_in = n_samples_out + 1, n_samples_in + 1
-    outer_circ_x = np.cos(np.linspace(0, 2 * np.pi, n_samples_out)[:-1])
-    outer_circ_y = np.sin(np.linspace(0, 2 * np.pi, n_samples_out)[:-1])
-    inner_circ_x = np.cos(np.linspace(0, 2 * np.pi, n_samples_in)[:-1]) * factor
-    inner_circ_y = np.sin(np.linspace(0, 2 * np.pi, n_samples_in)[:-1]) * factor
+    linspace = np.linspace(0, 2 * np.pi, n_samples / 2 + 1)[:-1]
+    outer_circ_x = np.cos(linspace)
+    outer_circ_y = np.sin(linspace)
+    inner_circ_x = outer_circ_x * factor
+    inner_circ_y = outer_circ_y * factor
 
-    X = np.vstack((np.append(outer_circ_x, inner_circ_x),\
-           np.append(outer_circ_y, inner_circ_y))).T
-    y = np.hstack([np.zeros(n_samples_out - 1), np.ones(n_samples_in - 1)])
+    X = np.vstack((np.append(outer_circ_x, inner_circ_x),
+                   np.append(outer_circ_y, inner_circ_y))).T
+    y = np.hstack([np.zeros(n_samples / 2), np.ones(n_samples / 2)])
     if shuffle:
         X, y = util_shuffle(X, y, random_state=generator)
 
@@ -542,6 +556,13 @@ def make_moons(n_samples=100, shuffle=True, noise=None, random_state=None):
     noise : double or None (default=None)
         Standard deviation of Gaussian noise added to the data.
 
+    Returns
+    -------
+    X : array of shape [n_samples, 2]
+        The generated samples.
+
+    y : array of shape [n_samples]
+        The integer labels (0 or 1) for class membership of each sample.
     """
 
     n_samples_out = n_samples / 2
@@ -554,8 +575,8 @@ def make_moons(n_samples=100, shuffle=True, noise=None, random_state=None):
     inner_circ_x = 1 - np.cos(np.linspace(0, np.pi, n_samples_in))
     inner_circ_y = 1 - np.sin(np.linspace(0, np.pi, n_samples_in)) - .5
 
-    X = np.vstack((np.append(outer_circ_x, inner_circ_x),\
-           np.append(outer_circ_y, inner_circ_y))).T
+    X = np.vstack((np.append(outer_circ_x, inner_circ_x),
+                   np.append(outer_circ_y, inner_circ_y))).T
     y = np.hstack([np.zeros(n_samples_in), np.ones(n_samples_out)])
 
     if shuffle:
@@ -612,14 +633,14 @@ def make_blobs(n_samples=100, n_features=2, centers=3, cluster_std=1.0,
     >>> from sklearn.datasets.samples_generator import make_blobs
     >>> X, y = make_blobs(n_samples=10, centers=3, n_features=2,
     ...                   random_state=0)
-    >>> X.shape
+    >>> print X.shape
     (10, 2)
     >>> y
     array([0, 0, 1, 0, 2, 2, 2, 1, 1, 0])
     """
     generator = check_random_state(random_state)
 
-    if isinstance(centers, int):
+    if isinstance(centers, numbers.Integral):
         centers = generator.uniform(center_box[0], center_box[1],
                                     size=(centers, n_features))
     else:
@@ -773,7 +794,7 @@ def make_friedman2(n_samples=100, noise=0.0, random_state=None):
     X[:, 3] += 1
 
     y = (X[:, 0] ** 2
-            + (X[:, 1] * X[:, 2] - 1 / (X[:, 1] * X[:, 3])) ** 2) ** 0.5 \
+         + (X[:, 1] * X[:, 2] - 1 / (X[:, 1] * X[:, 3])) ** 2) ** 0.5 \
         + noise * generator.randn(n_samples)
 
     return X, y
@@ -904,8 +925,8 @@ def make_low_rank_matrix(n_samples=100, n_features=100, effective_rank=10,
     singular_ind = np.arange(n, dtype=np.float64)
 
     # Build the singular profile by assembling signal and noise components
-    low_rank = (1 - tail_strength) * \
-               np.exp(-1.0 * (singular_ind / effective_rank) ** 2)
+    low_rank = ((1 - tail_strength) *
+                np.exp(-1.0 * (singular_ind / effective_rank) ** 2))
     tail = tail_strength * np.exp(-0.1 * singular_ind / effective_rank)
     s = np.identity(n) * (low_rank + tail)
 
@@ -1085,7 +1106,7 @@ def make_sparse_spd_matrix(dim=1, alpha=0.95, norm_diag=False,
     aux[aux < alpha] = 0
     aux[aux > alpha] = (smallest_coef
                         + (largest_coef - smallest_coef)
-                          * random_state.rand(np.sum(aux > alpha)))
+                        * random_state.rand(np.sum(aux > alpha)))
     aux = np.tril(aux, k=-1)
 
     # Permute the lines: we don't want to have assymetries in the final
