@@ -7,10 +7,13 @@
 # License: BSD Style.
 
 import numpy as np
+import warnings
 from scipy import linalg
+from math import log
 
 from ..base import BaseEstimator, TransformerMixin
 from ..utils import array2d, check_random_state, as_float_array
+from ..utils import atleast2d_or_csr
 from ..utils.extmath import fast_logdet
 from ..utils.extmath import safe_sparse_dot
 from ..utils.extmath import randomized_svd
@@ -44,12 +47,14 @@ def _assess_dimension_(spectrum, rank, n_samples, n_features):
     Automatic Choice of Dimensionality for PCA. NIPS 2000: 598-604`
     """
     if rank > len(spectrum):
-        raise ValueError("the tested rank cannot exceed the rank of the dataset")
+        raise ValueError("The tested rank cannot exceed the rank of the"
+                         " dataset")
     from scipy.special import gammaln
 
     pu = -rank * np.log(2)
     for i in range(rank):
-        pu += gammaln((n_features - i) / 2) - np.log(np.pi) * (n_features - i) / 2
+        pu += (gammaln((n_features - i) / 2)
+               - np.log(np.pi) * (n_features - i) / 2)
 
     pl = np.sum(np.log(spectrum[:rank]))
     pl = -pl * n_samples / 2
@@ -79,7 +84,7 @@ def _assess_dimension_(spectrum, rank, n_samples, n_features):
 
 
 def _infer_dimension_(spectrum, n_samples, n_features):
-    """This method infers the dimension of a dataset of shape (n_samples, n_features)
+    """Infers the dimension of a dataset of shape (n_samples, n_features)
 
     The dataset is described by its spectrum `spectrum`.
     """
@@ -160,7 +165,7 @@ class PCA(BaseEstimator, TransformerMixin):
     >>> pca = PCA(n_components=2)
     >>> pca.fit(X)
     PCA(copy=True, n_components=2, whiten=False)
-    >>> print pca.explained_variance_ratio_ # doctest: +ELLIPSIS
+    >>> print(pca.explained_variance_ratio_) # doctest: +ELLIPSIS
     [ 0.99244...  0.00755...]
 
     See also
@@ -227,8 +232,8 @@ class PCA(BaseEstimator, TransformerMixin):
         X -= self.mean_
         U, S, V = linalg.svd(X, full_matrices=False)
         self.explained_variance_ = (S ** 2) / n_samples
-        self.explained_variance_ratio_ = self.explained_variance_ / \
-                                        self.explained_variance_.sum()
+        self.explained_variance_ratio_ = (self.explained_variance_ /
+                                          self.explained_variance_.sum())
 
         if self.whiten:
             self.components_ = V / S[:, np.newaxis] * np.sqrt(n_samples)
@@ -238,9 +243,9 @@ class PCA(BaseEstimator, TransformerMixin):
         if self.n_components == 'mle':
             if n_samples < n_features:
                 raise ValueError("n_components='mle' is only supported "
-                "if n_samples >= n_features")
+                                 "if n_samples >= n_features")
             self.n_components = _infer_dimension_(self.explained_variance_,
-                                            n_samples, n_features)
+                                                  n_samples, n_features)
 
         elif (self.n_components is not None
               and 0 < self.n_components
@@ -253,9 +258,9 @@ class PCA(BaseEstimator, TransformerMixin):
         if self.n_components is not None:
             self.components_ = self.components_[:self.n_components, :]
             self.explained_variance_ = \
-                    self.explained_variance_[:self.n_components]
+                self.explained_variance_[:self.n_components]
             self.explained_variance_ratio_ = \
-                    self.explained_variance_ratio_[:self.n_components]
+                self.explained_variance_ratio_[:self.n_components]
 
         return (U, S, V)
 
@@ -273,6 +278,7 @@ class PCA(BaseEstimator, TransformerMixin):
         X_new : array-like, shape (n_samples, n_components)
 
         """
+        X = array2d(X)
         X_transformed = X - self.mean_
         X_transformed = np.dot(X_transformed, self.components_.T)
         return X_transformed
@@ -308,36 +314,48 @@ class ProbabilisticPCA(PCA):
 
         Parameters
         ----------
-        X : array of shape(n_samples, n_dim)
+        X : array of shape(n_samples, n_features)
             The data to fit
 
         homoscedastic : bool, optional,
             If True, average variance across remaining dimensions
         """
         PCA.fit(self, X)
-        self.dim = X.shape[1]
+        n_features = X.shape[1]
+        self._dim = n_features
         Xr = X - self.mean_
         Xr -= np.dot(np.dot(Xr, self.components_.T), self.components_)
         n_samples = X.shape[0]
-        if self.dim <= self.n_components:
-            delta = np.zeros(self.dim)
+        if n_features <= self.n_components:
+            delta = np.zeros(n_features)
         elif homoscedastic:
-            delta = (Xr ** 2).sum() * np.ones(self.dim) \
-                    / (n_samples * self.dim)
+            delta = ((Xr ** 2).sum() * np.ones(n_features)
+                     / (n_samples * n_features))
         else:
-            delta = (Xr ** 2).mean(0) / (self.dim - self.n_components)
+            delta = (Xr ** 2).mean(0) / (n_features - self.n_components)
         self.covariance_ = np.diag(delta)
-        for k in range(self.n_components):
+        n_components = self.n_components
+        if n_components is None:
+            n_components = n_features
+        for k in range(n_components):
             add_cov = np.outer(self.components_[k], self.components_[k])
             self.covariance_ += self.explained_variance_[k] * add_cov
         return self
 
+    @property
+    def dim(self):
+        warnings.warn("Using dim is deprecated "
+                      "since version 0.12, and backward compatibility "
+                      "won't be maintained from version 0.14 onward. ",
+                      DeprecationWarning, stacklevel=2)
+        return self._dim
+
     def score(self, X, y=None):
         """Return a score associated to new data
 
         Parameters
         ----------
-        X: array of shape(n_samples, n_dim)
+        X: array of shape(n_samples, n_features)
             The data to test
 
         Returns
@@ -346,12 +364,12 @@ class ProbabilisticPCA(PCA):
             log-likelihood of each row of X under the current model
         """
         Xr = X - self.mean_
+        n_features = X.shape[1]
         log_like = np.zeros(X.shape[0])
         self.precision_ = linalg.inv(self.covariance_)
-        for i in range(X.shape[0]):
-            log_like[i] = -.5 * np.dot(np.dot(self.precision_, Xr[i]), Xr[i])
-        log_like += fast_logdet(self.precision_) - \
-                                    self.dim / 2 * np.log(2 * np.pi)
+        log_like = -.5 * (Xr * (np.dot(Xr, self.precision_))).sum(axis=1)
+        log_like -= .5 * (fast_logdet(self.covariance_)
+                          + n_features * log(2 * np.pi))
         return log_like
 
 
@@ -384,7 +402,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         Whitening will remove some information from the transformed signal
         (the relative variance scales of the components) but can sometime
         improve the predictive accuracy of the downstream estimators by
-        making there data respect some hard-wired assumptions.
+        making their data respect some hard-wired assumptions.
 
     random_state : int or RandomState instance or None (default)
         Pseudo Random Number generator seed control. If None, use the
@@ -408,8 +426,8 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
     >>> pca = RandomizedPCA(n_components=2)
     >>> pca.fit(X)                 # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     RandomizedPCA(copy=True, iterated_power=3, n_components=2,
-           random_state=<mtrand.RandomState object at 0x...>, whiten=False)
-    >>> print pca.explained_variance_ratio_ # doctest: +ELLIPSIS
+           random_state=None, whiten=False)
+    >>> print(pca.explained_variance_ratio_) # doctest: +ELLIPSIS
     [ 0.99244...  0.00755...]
 
     See also
@@ -429,7 +447,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
 
     """
 
-    def __init__(self, n_components, copy=True, iterated_power=3,
+    def __init__(self, n_components=None, copy=True, iterated_power=3,
                  whiten=False, random_state=None):
         self.n_components = n_components
         self.copy = copy
@@ -452,27 +470,28 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         self : object
             Returns the instance itself.
         """
-        self.random_state = check_random_state(self.random_state)
+        random_state = check_random_state(self.random_state)
         if not hasattr(X, 'todense'):
             # not a sparse matrix, ensure this is a 2D array
-            X = array2d(X)
+            X = np.atleast_2d(as_float_array(X, copy=self.copy))
 
         n_samples = X.shape[0]
 
         if not hasattr(X, 'todense'):
-            X = as_float_array(X, copy=self.copy)
-
             # Center data
             self.mean_ = np.mean(X, axis=0)
             X -= self.mean_
+        if self.n_components is None:
+            n_components = X.shape[1]
+        else:
+            n_components = self.n_components
 
-        U, S, V = randomized_svd(X, self.n_components,
-                                 n_iterations=self.iterated_power,
-                                 random_state=self.random_state)
+        U, S, V = randomized_svd(X, n_components,
+                                 n_iter=self.iterated_power,
+                                 random_state=random_state)
 
-        self.explained_variance_ = (S ** 2) / n_samples
-        self.explained_variance_ratio_ = self.explained_variance_ / \
-                                        self.explained_variance_.sum()
+        self.explained_variance_ = exp_var = (S ** 2) / n_samples
+        self.explained_variance_ratio_ = exp_var / exp_var.sum()
 
         if self.whiten:
             n = X.shape[0]
@@ -483,7 +502,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         return self
 
     def transform(self, X):
-        """Apply the dimensionality reduction on X.
+        """Apply dimensionality reduction on X.
 
         Parameters
         ----------
@@ -496,6 +515,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         X_new : array-like, shape (n_samples, n_components)
 
         """
+        X = atleast2d_or_csr(X)
         if self.mean_ is not None:
             X = X - self.mean_
 
@@ -503,8 +523,9 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         return X
 
     def inverse_transform(self, X):
-        """Transform data back to its original space, i.e.,
-        return an input X_original whose transform would be X
+        """Transform data back to its original space.
+
+        Returns an array X_original whose transform would be X.
 
         Parameters
         ----------
@@ -519,7 +540,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         Notes
         -----
         If whitening is enabled, inverse_transform does not compute the
-        exact inverse operation as transform.
+        exact inverse operation of transform.
         """
         X_original = safe_sparse_dot(X, self.components_)
         if self.mean_ is not None:
