@@ -103,7 +103,7 @@ class SparsePCA(BaseEstimator, TransformerMixin):
         self : object
             Returns the instance itself.
         """
-        self.random_state = check_random_state(self.random_state)
+        random_state = check_random_state(self.random_state)
         X = array2d(X)
         if self.n_components is None:
             n_components = X.shape[1]
@@ -115,7 +115,7 @@ class SparsePCA(BaseEstimator, TransformerMixin):
                                  tol=self.tol, max_iter=self.max_iter,
                                  method=self.method, n_jobs=self.n_jobs,
                                  verbose=self.verbose,
-                                 random_state=self.random_state,
+                                 random_state=random_state,
                                  code_init=code_init,
                                  dict_init=dict_init)
         self.components_ = Vt.T
@@ -254,7 +254,7 @@ class MiniBatchSparsePCA(SparsePCA):
         self : object
             Returns the instance itself.
         """
-        self.random_state = check_random_state(self.random_state)
+        random_state = check_random_state(self.random_state)
         X = array2d(X)
         if self.n_components is None:
             n_components = X.shape[1]
@@ -267,6 +267,6 @@ class MiniBatchSparsePCA(SparsePCA):
                                      batch_size=self.batch_size,
                                      shuffle=self.shuffle,
                                      n_jobs=self.n_jobs, method=self.method,
-                                     random_state=self.random_state)
+                                     random_state=random_state)
         self.components_ = Vt.T
         return self
