@@ -1,18 +1,18 @@
 import numpy as np
-from nose.tools import assert_true
-from nose.tools import assert_equal
-
 from scipy.sparse import csr_matrix
-from numpy.testing import assert_almost_equal, assert_array_almost_equal
 
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_less, assert_greater
 
-from ... import datasets
-from .. import PCA
-from .. import ProbabilisticPCA
-from .. import RandomizedPCA
-from ..pca import _assess_dimension_
-from ..pca import _infer_dimension_
+from sklearn import datasets
+from sklearn.decomposition import PCA
+from sklearn.decomposition import ProbabilisticPCA
+from sklearn.decomposition import RandomizedPCA
+from sklearn.decomposition.pca import _assess_dimension_
+from sklearn.decomposition.pca import _infer_dimension_
 
 iris = datasets.load_iris()
 
@@ -135,8 +135,8 @@ def test_randomized_pca_check_projection():
 def test_randomized_pca_check_list():
     """Test that the projection by RandomizedPCA on list data is correct"""
     X = [[1.0, 0.0], [0.0, 1.0]]
-    X_transformed = RandomizedPCA(n_components=1, random_state=0
-                        ).fit(X).transform(X)
+    X_transformed = RandomizedPCA(n_components=1,
+                                  random_state=0).fit(X).transform(X)
     assert_equal(X_transformed.shape, (2, 1))
     assert_almost_equal(X_transformed.mean(), 0.00, 2)
     assert_almost_equal(X_transformed.std(), 0.71, 2)
@@ -227,8 +227,8 @@ def test_infer_dim_1():
     """
     n, p = 1000, 5
     rng = np.random.RandomState(0)
-    X = rng.randn(n, p) * .1 + rng.randn(n, 1) * np.array([3, 4, 5, 1, 2]) \
-            + np.array([1, 0, 7, 4, 6])
+    X = (rng.randn(n, p) * .1 + rng.randn(n, 1) * np.array([3, 4, 5, 1, 2])
+         + np.array([1, 0, 7, 4, 6]))
     pca = PCA(n_components=p)
     pca.fit(X)
     spect = pca.explained_variance_
@@ -295,7 +295,7 @@ def test_probabilistic_pca_1():
     ppca = ProbabilisticPCA(n_components=2)
     ppca.fit(X)
     ll1 = ppca.score(X)
-    h = 0.5 * np.log(2 * np.pi * np.exp(1) / 0.1 ** 2) * p
+    h = -0.5 * np.log(2 * np.pi * np.exp(1) * 0.1 ** 2) * p
     np.testing.assert_almost_equal(ll1.mean() / h, 1, 0)
 
 
