@@ -4,25 +4,28 @@
 import sys
 
 import numpy as np
-from numpy.testing import assert_array_almost_equal, assert_equal
 
-from nose import SkipTest
-from nose.tools import assert_true, assert_false
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import SkipTest
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_false
 
-from .. import SparsePCA, MiniBatchSparsePCA
-from ...utils import check_random_state
+from sklearn.decomposition import SparsePCA, MiniBatchSparsePCA
+from sklearn.utils import check_random_state
 
 
-def generate_toy_data(n_atoms, n_samples, image_size, random_state=None):
+def generate_toy_data(n_components, n_samples, image_size, random_state=None):
     n_features = image_size[0] * image_size[1]
 
     rng = check_random_state(random_state)
-    U = rng.randn(n_samples, n_atoms)
-    V = rng.randn(n_atoms, n_features)
+    U = rng.randn(n_samples, n_components)
+    V = rng.randn(n_components, n_features)
 
     centers = [(3, 3), (6, 7), (8, 1)]
     sz = [1, 2, 1]
-    for k in range(n_atoms):
+    for k in range(n_components):
         img = np.zeros(image_size)
         xmin, xmax = centers[k][0] - sz[k], centers[k][0] + sz[k]
         ymin, ymax = centers[k][1] - sz[k], centers[k][1] + sz[k]
@@ -114,7 +117,7 @@ def test_initialization():
     model = SparsePCA(n_components=3, U_init=U_init, V_init=V_init, max_iter=0,
                       random_state=rng)
     model.fit(rng.randn(5, 4))
-    assert_equal(model.components_, V_init)
+    assert_array_equal(model.components_, V_init)
 
 
 def test_mini_batch_correct_shapes():
