@@ -77,7 +77,10 @@ class LogOddsEstimator(BaseEstimator):
     """An estimator predicting the log odds ratio."""
     def fit(self, X, y):
         n_pos = np.sum(y)
-        self.prior = np.log(n_pos / (y.shape[0] - n_pos))
+        n_neg = y.shape[0] - n_pos
+        if n_neg == 0 or n_pos == 0:
+            raise ValueError('y contains non binary labels.')
+        self.prior = np.log(n_pos / n_neg)
 
     def predict(self, X):
         y = np.empty((X.shape[0], 1), dtype=np.float64)
@@ -340,14 +343,22 @@ class BinomialDeviance(LossFunction):
         """Compute the deviance (= negative log-likelihood). """
         # logaddexp(0, v) == log(1.0 + exp(v))
         pred = pred.ravel()
-        return np.sum(np.logaddexp(0.0, -2 * y * pred)) / y.shape[0]
+        return -2.0 * np.mean((y * pred) - np.logaddexp(0.0, pred))
 
     def negative_gradient(self, y, pred, **kargs):
+        """Compute the residual (= negative gradient). """
         return y - 1.0 / (1.0 + np.exp(-pred.ravel()))
 
     def _update_terminal_region(self, tree, terminal_regions, leaf, X, y,
                                 residual, pred):
-        """Make a single Newton-Raphson step. """
+        """Make a single Newton-Raphson step.
+
+        our node estimate is given by:
+
+            sum(y - prob) / sum(prob * (1 - prob))
+
+        we take advantage that: y - prob = residual
+        """
         terminal_region = np.where(terminal_regions == leaf)[0]
         residual = residual.take(terminal_region, axis=0)
         y = y.take(terminal_region, axis=0)
