@@ -242,9 +242,9 @@ def f_regression(X, y, center=True):
 
 
 ######################################################################
-# General class for filter univariate selection
+# Base classes
 
-class _AbstractUnivariateFilter(BaseEstimator, TransformerMixin):
+class _BaseFilter(BaseEstimator, TransformerMixin):
     __metaclass__ = ABCMeta
 
     def __init__(self, score_func):
@@ -262,16 +262,9 @@ class _AbstractUnivariateFilter(BaseEstimator, TransformerMixin):
                 "was passed." % (score_func, type(score_func)))
         self.score_func = score_func
 
+    @abstractmethod
     def fit(self, X, y):
-        """
-        Evaluate the function
-        """
-        self.scores_, self.pvalues_ = self.score_func(X, y)
-        if len(np.unique(self.pvalues_)) < len(self.pvalues_):
-            warn("Duplicate p-values. Result may depend on feature ordering."
-                 "There are probably duplicate features, or you used a "
-                 "classification score for a regression task.")
-        return self
+        """Run score function on (X, y) and get the appropriate features."""
 
     def get_support(self, indices=False):
         """
@@ -308,11 +301,40 @@ class _AbstractUnivariateFilter(BaseEstimator, TransformerMixin):
         return Xt
 
 
+class _PvalueFilter(_BaseFilter):
+    def fit(self, X, y):
+        """Evaluate the score function on samples X with outputs y.
+
+        Records and selects features according to the p-values output by the
+        score function.
+        """
+        self.scores_, self.pvalues_ = self.score_func(X, y)
+        if len(np.unique(self.pvalues_)) < len(self.pvalues_):
+            warn("Duplicate p-values. Result may depend on feature ordering."
+                 "There are probably duplicate features, or you used a "
+                 "classification score for a regression task.")
+        return self
+
+
+class _ScoreFilter(_BaseFilter):
+    def fit(self, X, y):
+        """Evaluate the score function on samples X with outputs y.
+
+        Records and selects features according to their scores.
+        """
+        self.scores_, self.pvalues_ = self.score_func(X, y)
+        if len(np.unique(self.scores_)) < len(self.scores_):
+            warn("Duplicate scores. Result may depend on feature ordering."
+                 "There are probably duplicate features, or you used a "
+                 "classification score for a regression task.")
+        return self
+
+
 ######################################################################
 # Specific filters
 ######################################################################
 
-class SelectPercentile(_AbstractUnivariateFilter):
+class SelectPercentile(_ScoreFilter):
     """Select features according to a percentile of the highest scores.
 
     Parameters
@@ -334,7 +356,7 @@ class SelectPercentile(_AbstractUnivariateFilter):
 
     Notes
     -----
-    Ties between features with equal p-values will be broken in an unspecified
+    Ties between features with equal scores will be broken in an unspecified
     way.
 
     """
@@ -369,7 +391,7 @@ class SelectPercentile(_AbstractUnivariateFilter):
         return mask
 
 
-class SelectKBest(_AbstractUnivariateFilter):
+class SelectKBest(_ScoreFilter):
     """Select features according to the k highest scores.
 
     Parameters
@@ -415,7 +437,7 @@ class SelectKBest(_AbstractUnivariateFilter):
         return mask
 
 
-class SelectFpr(_AbstractUnivariateFilter):
+class SelectFpr(_PvalueFilter):
     """Filter: Select the pvalues below alpha based on a FPR test.
 
     FPR test stands for False Positive Rate test. It controls the total
@@ -448,7 +470,7 @@ class SelectFpr(_AbstractUnivariateFilter):
         return self.pvalues_ < alpha
 
 
-class SelectFdr(_AbstractUnivariateFilter):
+class SelectFdr(_PvalueFilter):
     """Filter: Select the p-values for an estimated false discovery rate
 
     This uses the Benjamini-Hochberg procedure. ``alpha`` is the target false
@@ -484,7 +506,7 @@ class SelectFdr(_AbstractUnivariateFilter):
         return self.pvalues_ <= threshold
 
 
-class SelectFwe(_AbstractUnivariateFilter):
+class SelectFwe(_PvalueFilter):
     """Filter: Select the p-values corresponding to Family-wise error rate
 
     Parameters
@@ -518,7 +540,9 @@ class SelectFwe(_AbstractUnivariateFilter):
 # Generic filter
 ######################################################################
 
-class GenericUnivariateSelect(_AbstractUnivariateFilter):
+# TODO this class should fit on either p-values or scores,
+# depending on the mode.
+class GenericUnivariateSelect(_PvalueFilter):
     """Univariate feature selector with configurable strategy.
 
     Parameters
