@@ -2,6 +2,7 @@
 #
 # Author: Ron Weiss <ronweiss@gmail.com>
 # and Shiqiao Du <lucidfrontier.45@gmail.com>
+# API changes: Jaques Grobler <jaquesgrobler@gmail.com>
 
 """
 The :mod:`sklearn.hmm` module implements hidden Markov models.
@@ -12,6 +13,7 @@ make it more stable, this module will be removed in version 0.11.
 """
 
 import string
+import warnings
 
 import numpy as np
 
@@ -22,9 +24,13 @@ from .mixture import (
     GMM, log_multivariate_normal_density, sample_gaussian,
     distribute_covar_matrix_to_match_covariance_type, _validate_covars)
 from . import cluster
-from .utils import deprecated
 from . import _hmmc
 
+__all__ = ['GMMHMM',
+           'GaussianHMM',
+           'MultinomialHMM',
+           'decoder_algorithms',
+           'normalize']
 
 ZEROLOGPROB = -1e200
 EPS = np.finfo(float).eps
@@ -93,6 +99,24 @@ class _BaseHMM(BaseEstimator):
     random_state: RandomState or an int seed (0 by default)
         A random number generator instance
 
+    n_iter : int, optional
+        Number of iterations to perform.
+
+    thresh : float, optional
+        Convergence threshold.
+
+    params : string, optional
+        Controls which parameters are updated in the training
+        process.  Can contain any combination of 's' for startprob,
+        't' for transmat, 'm' for means, and 'c' for covars, etc.
+        Defaults to all parameters.
+
+    init_params : string, optional
+        Controls which parameters are initialized prior to
+        training.  Can contain any combination of 's' for
+        startprob, 't' for transmat, 'm' for means, and 'c' for
+        covars, etc.  Defaults to all parameters.
+
     See Also
     --------
     GMM : Gaussian mixture model
@@ -110,31 +134,21 @@ class _BaseHMM(BaseEstimator):
     # the emission distribution parameters to expose them publically.
 
     def __init__(self, n_components=1, startprob=None, transmat=None,
-            startprob_prior=None, transmat_prior=None,
-            algorithm="viterbi", random_state=None):
+                 startprob_prior=None, transmat_prior=None,
+                 algorithm="viterbi", random_state=None,
+                 n_iter=10, thresh=1e-2, params=string.ascii_letters,
+                 init_params=string.ascii_letters):
+
         self.n_components = n_components
-
-        if startprob is None:
-            startprob = np.tile(1.0 / n_components, n_components)
+        self.n_iter = n_iter
+        self.thresh = thresh
+        self.params = params
+        self.init_params = init_params
         self.startprob_ = startprob
-
-        if startprob_prior is None:
-            startprob_prior = 1.0
         self.startprob_prior = startprob_prior
-
-        if transmat is None:
-            transmat = np.tile(1.0 / n_components,
-                    (n_components, n_components))
         self.transmat_ = transmat
-
-        if transmat_prior is None:
-            transmat_prior = 1.0
         self.transmat_prior = transmat_prior
-
-        if algorithm in decoder_algorithms:
-            self._algorithm = algorithm
-        else:
-            self._algorithm = "viterbi"
+        self._algorithm = algorithm
         self.random_state = random_state
 
     def eval(self, obs):
@@ -356,24 +370,18 @@ class _BaseHMM(BaseEstimator):
         currstate = (startprob_cdf > rand).argmax()
         hidden_states = [currstate]
         obs = [self._generate_sample_from_state(
-                                currstate, random_state=random_state)]
+            currstate, random_state=random_state)]
 
         for _ in xrange(n - 1):
             rand = random_state.rand()
             currstate = (transmat_cdf[currstate] > rand).argmax()
             hidden_states.append(currstate)
             obs.append(self._generate_sample_from_state(
-                                currstate, random_state=random_state))
+                currstate, random_state=random_state))
 
         return np.array(obs), np.array(hidden_states, dtype=int)
 
-    @deprecated("rvs is deprecated in 0.11 will be removed in 0.13:"
-            + " use sample instead")
-    def rvs(self, n=1, random_state=None):
-        return self.sample(n, random_state)
-
-    def fit(self, obs, n_iter=10, thresh=1e-2, params=string.ascii_letters,
-            init_params=string.ascii_letters, **kwargs):
+    def fit(self, obs, **kwargs):
         """Estimate model parameters.
 
         An initialization step is performed before entering the EM
@@ -387,24 +395,6 @@ class _BaseHMM(BaseEstimator):
         obs : list
             List of array-like observation sequences (shape (n_i, n_features)).
 
-        n_iter : int, optional
-            Number of iterations to perform.
-
-        thresh : float, optional
-            Convergence threshold.
-
-        params : string, optional
-            Controls which parameters are updated in the training
-            process.  Can contain any combination of 's' for startprob,
-            't' for transmat, 'm' for means, and 'c' for covars, etc.
-            Defaults to all parameters.
-
-        init_params : string, optional
-            Controls which parameters are initialized prior to
-            training.  Can contain any combination of 's' for
-            startprob, 't' for transmat, 'm' for means, and 'c' for
-            covars, etc.  Defaults to all parameters.
-
         Notes
         -----
         In general, `logprob` should be non-decreasing unless
@@ -412,11 +402,30 @@ class _BaseHMM(BaseEstimator):
         a sign of overfitting (e.g. a covariance parameter getting too
         small).  You can fix this by getting more training data, or
         decreasing `covars_prior`.
+
+        **Please note that setting parameters in the `fit` method is
+        deprecated and will be removed in the next release.
+        Set it on initialization instead.**
         """
-        self._init(obs, init_params)
+
+        if kwargs:
+            warnings.warn("Setting parameters in the 'fit' method is"
+                          "deprecated and will be removed in 0.14. Set it on "
+                          "initialization instead.", DeprecationWarning,
+                          stacklevel=2)
+            # initialisations for in case the user still adds parameters to fit
+            # so things don't break
+            for name in ('n_iter', 'thresh', 'params', 'init_params'):
+                if name in kwargs:
+                    setattr(self, name, kwargs[name])
+
+        if self.algorithm not in decoder_algorithms:
+            self._algorithm = "viterbi"
+
+        self._init(obs, self.init_params)
 
         logprob = []
-        for i in xrange(n_iter):
+        for i in xrange(self.n_iter):
             # Expectation step
             stats = self._initialize_sufficient_statistics()
             curr_logprob = 0
@@ -429,15 +438,15 @@ class _BaseHMM(BaseEstimator):
                 curr_logprob += lpr
                 self._accumulate_sufficient_statistics(
                     stats, seq, framelogprob, posteriors, fwdlattice,
-                    bwdlattice, params)
+                    bwdlattice, self.params)
             logprob.append(curr_logprob)
 
             # Check for convergence.
-            if i > 0 and abs(logprob[-1] - logprob[-2]) < thresh:
+            if i > 0 and abs(logprob[-1] - logprob[-2]) < self.thresh:
                 break
 
             # Maximization step
-            self._do_mstep(stats, params)
+            self._do_mstep(stats, self.params)
 
         return self
 
@@ -457,6 +466,16 @@ class _BaseHMM(BaseEstimator):
         return np.exp(self._log_startprob)
 
     def _set_startprob(self, startprob):
+        if startprob is None:
+            startprob = np.tile(1.0 / self.n_components, self.n_components)
+        else:
+            startprob = np.asarray(startprob, dtype=np.float)
+
+        # check if there exists a component whose value is exactly zero
+        # if so, add a small number and re-normalize
+        if not np.alltrue(startprob):
+            normalize(startprob)
+
         if len(startprob) != self.n_components:
             raise ValueError('startprob must have length n_components')
         if not np.allclose(np.sum(startprob), 1.0):
@@ -471,10 +490,19 @@ class _BaseHMM(BaseEstimator):
         return np.exp(self._log_transmat)
 
     def _set_transmat(self, transmat):
+        if transmat is None:
+            transmat = np.tile(1.0 / self.n_components,
+                               (self.n_components, self.n_components))
+
+        # check if there exists a component whose value is exactly zero
+        # if so, add a small number and re-normalize
+        if not np.alltrue(transmat):
+            normalize(transmat, axis=1)
+
         if (np.asarray(transmat).shape
                 != (self.n_components, self.n_components)):
-            raise ValueError('transmat must have shape ' +
-                    '(n_components, n_components)')
+            raise ValueError('transmat must have shape '
+                             '(n_components, n_components)')
         if not np.all(np.allclose(np.sum(transmat, axis=1), 1.0)):
             raise ValueError('Rows of transmat must sum to 1.0')
 
@@ -487,8 +515,8 @@ class _BaseHMM(BaseEstimator):
     def _do_viterbi_pass(self, framelogprob):
         n_observations, n_components = framelogprob.shape
         state_sequence, logprob = _hmmc._viterbi(
-                n_observations, n_components, self._log_startprob,
-                self._log_transmat, framelogprob)
+            n_observations, n_components, self._log_startprob,
+            self._log_transmat, framelogprob)
         return logprob, state_sequence
 
     def _do_forward_pass(self, framelogprob):
@@ -496,16 +524,15 @@ class _BaseHMM(BaseEstimator):
         n_observations, n_components = framelogprob.shape
         fwdlattice = np.zeros((n_observations, n_components))
         _hmmc._forward(n_observations, n_components, self._log_startprob,
-                self._log_transmat, framelogprob, fwdlattice)
+                       self._log_transmat, framelogprob, fwdlattice)
         fwdlattice[fwdlattice <= ZEROLOGPROB] = NEGINF
         return logsumexp(fwdlattice[-1]), fwdlattice
 
     def _do_backward_pass(self, framelogprob):
         n_observations, n_components = framelogprob.shape
         bwdlattice = np.zeros((n_observations, n_components))
-        _hmmc._backward(n_observations, n_components,
-                self._log_startprob, self._log_transmat,
-                framelogprob, bwdlattice)
+        _hmmc._backward(n_observations, n_components, self._log_startprob,
+                        self._log_transmat, framelogprob, bwdlattice)
 
         bwdlattice[bwdlattice <= ZEROLOGPROB] = NEGINF
 
@@ -519,9 +546,9 @@ class _BaseHMM(BaseEstimator):
 
     def _init(self, obs, params):
         if 's' in params:
-            self.startprob_[:] = 1.0 / self.n_components
+            self.startprob_.fill(1.0 / self.n_components)
         if 't' in params:
-            self.transmat_[:] = 1.0 / self.n_components
+            self.transmat_.fill(1.0 / self.n_components)
 
     # Methods used by self.fit()
 
@@ -538,32 +565,30 @@ class _BaseHMM(BaseEstimator):
         if 's' in params:
             stats['start'] += posteriors[0]
         if 't' in params:
-            if _hmmc:
-                n_observations, n_components = framelogprob.shape
-                lneta = np.zeros((n_observations - 1,
-                            n_components, n_components))
-                lnP = logsumexp(fwdlattice[-1])
-                _hmmc._compute_lneta(n_observations, n_components,
-                        fwdlattice, self._log_transmat, bwdlattice,
-                        framelogprob, lnP, lneta)
-                stats["trans"] += np.exp(logsumexp(lneta, 0))
-            else:
-                for t in xrange(len(framelogprob)):
-                    zeta = (fwdlattice[t - 1][:, np.newaxis]
-                            + self._log_transmat + framelogprob[t]
-                            + bwdlattice[t])
-                    stats['trans'] += np.exp(zeta - logsumexp(zeta))
+            n_observations, n_components = framelogprob.shape
+            lneta = np.zeros((n_observations - 1, n_components, n_components))
+            lnP = logsumexp(fwdlattice[-1])
+            _hmmc._compute_lneta(n_observations, n_components, fwdlattice,
+                                 self._log_transmat, bwdlattice, framelogprob,
+                                 lnP, lneta)
+            stats["trans"] += np.exp(logsumexp(lneta, 0))
 
     def _do_mstep(self, stats, params):
         # Based on Huang, Acero, Hon, "Spoken Language Processing",
         # p. 443 - 445
+        if self.startprob_prior is None:
+            self.startprob_prior = 1.0
+        if self.transmat_prior is None:
+            self.transmat_prior = 1.0
+
         if 's' in params:
             self.startprob_ = normalize(
                 np.maximum(self.startprob_prior - 1.0 + stats['start'], 1e-20))
         if 't' in params:
-            self.transmat_ = normalize(
+            transmat_ = normalize(
                 np.maximum(self.transmat_prior - 1.0 + stats['trans'], 1e-20),
                 axis=1)
+            self.transmat_ = transmat_
 
 
 class GaussianHMM(_BaseHMM):
@@ -578,14 +603,14 @@ class GaussianHMM(_BaseHMM):
     n_components : int
         Number of states.
 
-    _covariance_type : string
+    ``_covariance_type`` : string
         String describing the type of covariance parameters to
         use.  Must be one of 'spherical', 'tied', 'diag', 'full'.
         Defaults to 'diag'.
 
     Attributes
     ----------
-    covariance_type : string
+    ``_covariance_type`` : string
         String describing the type of covariance parameters used by
         the model.  Must be one of 'spherical', 'tied', 'diag', 'full'.
 
@@ -606,7 +631,7 @@ class GaussianHMM(_BaseHMM):
 
     covars : array
         Covariance parameters for each state.  The shape depends on
-        `_covariance_type`::
+        ``_covariance_type``::
 
             (`n_components`,)                   if 'spherical',
             (`n_features`, `n_features`)              if 'tied',
@@ -616,15 +641,32 @@ class GaussianHMM(_BaseHMM):
     random_state: RandomState or an int seed (0 by default)
         A random number generator instance
 
+    n_iter : int, optional
+        Number of iterations to perform.
+
+    thresh : float, optional
+        Convergence threshold.
+
+    params : string, optional
+        Controls which parameters are updated in the training
+        process.  Can contain any combination of 's' for startprob,
+        't' for transmat, 'm' for means, and 'c' for covars, etc.
+        Defaults to all parameters.
+
+    init_params : string, optional
+        Controls which parameters are initialized prior to
+        training.  Can contain any combination of 's' for
+        startprob, 't' for transmat, 'm' for means, and 'c' for
+        covars, etc.  Defaults to all parameters.
+
+
     Examples
     --------
     >>> from sklearn.hmm import GaussianHMM
     >>> GaussianHMM(n_components=2)
     ...                             #doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
-    GaussianHMM(algorithm='viterbi', covariance_type='diag', covars_prior=0.01,
-        covars_weight=1, means_prior=None, means_weight=0, n_components=2,
-        random_state=None, startprob=None, startprob_prior=1.0, transmat=None,
-        transmat_prior=1.0)
+    GaussianHMM(algorithm='viterbi',...
+
 
     See Also
     --------
@@ -635,12 +677,15 @@ class GaussianHMM(_BaseHMM):
                  transmat=None, startprob_prior=None, transmat_prior=None,
                  algorithm="viterbi", means_prior=None, means_weight=0,
                  covars_prior=1e-2, covars_weight=1,
-                 random_state=None):
+                 random_state=None, n_iter=10, thresh=1e-2,
+                 params=string.ascii_letters,
+                 init_params=string.ascii_letters):
         _BaseHMM.__init__(self, n_components, startprob, transmat,
-                                        startprob_prior=startprob_prior,
-                                        transmat_prior=transmat_prior,
-                                        algorithm=algorithm,
-                                        random_state=random_state)
+                          startprob_prior=startprob_prior,
+                          transmat_prior=transmat_prior, algorithm=algorithm,
+                          random_state=random_state, n_iter=n_iter,
+                          thresh=thresh, params=params,
+                          init_params=init_params)
 
         self._covariance_type = covariance_type
         if not covariance_type in ['spherical', 'tied', 'diag', 'full']:
@@ -652,7 +697,6 @@ class GaussianHMM(_BaseHMM):
         self.covars_prior = covars_prior
         self.covars_weight = covars_weight
 
-    # Read-only properties.
     @property
     def covariance_type(self):
         """Covariance type of the model.
@@ -667,10 +711,10 @@ class GaussianHMM(_BaseHMM):
 
     def _set_means(self, means):
         means = np.asarray(means)
-        if hasattr(self, 'n_features') and \
-               means.shape != (self.n_components, self.n_features):
-            raise ValueError('means must have shape' +
-                    '(n_components, n_features)')
+        if (hasattr(self, 'n_features')
+                and means.shape != (self.n_components, self.n_features)):
+            raise ValueError('means must have shape '
+                             '(n_components, n_features)')
         self._means_ = means.copy()
         self.n_features = self._means_.shape[1]
 
@@ -710,7 +754,7 @@ class GaussianHMM(_BaseHMM):
         super(GaussianHMM, self)._init(obs, params=params)
 
         if (hasattr(self, 'n_features')
-            and self.n_features != obs[0].shape[1]):
+                and self.n_features != obs[0].shape[1]):
             raise ValueError('Unexpected number of dimensions, got %s but '
                              'expected %s' % (obs[0].shape[1],
                                               self.n_features))
@@ -719,7 +763,7 @@ class GaussianHMM(_BaseHMM):
 
         if 'm' in params:
             self._means_ = cluster.KMeans(
-                k=self.n_components).fit(obs[0]).cluster_centers_
+                n_clusters=self.n_components).fit(obs[0]).cluster_centers_
         if 'c' in params:
             cv = np.cov(obs[0].T)
             if not cv.shape:
@@ -792,8 +836,9 @@ class GaussianHMM(_BaseHMM):
                 cv_den = max(covars_weight - 1, 0) + denom
                 self._covars_ = (covars_prior + cv_num) / cv_den
                 if self._covariance_type == 'spherical':
-                    self._covars_ = np.tile(self._covars_.mean(1)
-                            [:, np.newaxis], (1, self._covars_.shape[1]))
+                    self._covars_ = np.tile(
+                        self._covars_.mean(1)[:, np.newaxis],
+                        (1, self._covars_.shape[1]))
             elif self._covariance_type in ('tied', 'full'):
                 cvnum = np.empty((self.n_components, self.n_features,
                                   self.n_features))
@@ -808,11 +853,11 @@ class GaussianHMM(_BaseHMM):
                                 * stats['post'][c])
                 cvweight = max(covars_weight - self.n_features, 0)
                 if self._covariance_type == 'tied':
-                    self._covars_ = ((covars_prior + cvnum.sum(axis=0))
-                                    / (cvweight + stats['post'].sum()))
+                    self._covars_ = ((covars_prior + cvnum.sum(axis=0)) /
+                                     (cvweight + stats['post'].sum()))
                 elif self._covariance_type == 'full':
-                    self._covars_ = ((covars_prior + cvnum)
-                                   / (cvweight + stats['post'][:, None, None]))
+                    self._covars_ = ((covars_prior + cvnum) /
+                                     (cvweight + stats['post'][:, None, None]))
 
 
 class MultinomialHMM(_BaseHMM):
@@ -838,14 +883,30 @@ class MultinomialHMM(_BaseHMM):
     random_state: RandomState or an int seed (0 by default)
         A random number generator instance
 
+    n_iter : int, optional
+        Number of iterations to perform.
+
+    thresh : float, optional
+        Convergence threshold.
+
+    params : string, optional
+        Controls which parameters are updated in the training
+        process.  Can contain any combination of 's' for startprob,
+        't' for transmat, 'm' for means, and 'c' for covars, etc.
+        Defaults to all parameters.
+
+    init_params : string, optional
+        Controls which parameters are initialized prior to
+        training.  Can contain any combination of 's' for
+        startprob, 't' for transmat, 'm' for means, and 'c' for
+        covars, etc.  Defaults to all parameters.
+
     Examples
     --------
     >>> from sklearn.hmm import MultinomialHMM
     >>> MultinomialHMM(n_components=2)
     ...                             #doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
-    MultinomialHMM(algorithm='viterbi', n_components=2, random_state=None,
-                   startprob=None, startprob_prior=1.0, transmat=None,
-                   transmat_prior=1.0)
+    MultinomialHMM(algorithm='viterbi',...
 
     See Also
     --------
@@ -853,8 +914,10 @@ class MultinomialHMM(_BaseHMM):
     """
 
     def __init__(self, n_components=1, startprob=None, transmat=None,
-            startprob_prior=None, transmat_prior=None,
-            algorithm="viterbi", random_state=None):
+                 startprob_prior=None, transmat_prior=None,
+                 algorithm="viterbi", random_state=None,
+                 n_iter=10, thresh=1e-2, params=string.ascii_letters,
+                 init_params=string.ascii_letters):
         """Create a hidden Markov model with multinomial emissions.
 
         Parameters
@@ -863,10 +926,14 @@ class MultinomialHMM(_BaseHMM):
             Number of states.
         """
         _BaseHMM.__init__(self, n_components, startprob, transmat,
-                                             startprob_prior=startprob_prior,
-                                             transmat_prior=transmat_prior,
-                                             algorithm=algorithm,
-                                             random_state=random_state)
+                          startprob_prior=startprob_prior,
+                          transmat_prior=transmat_prior,
+                          algorithm=algorithm,
+                          random_state=random_state,
+                          n_iter=n_iter,
+                          thresh=thresh,
+                          params=params,
+                          init_params=init_params)
 
     def _get_emissionprob(self):
         """Emission probability distribution for each state."""
@@ -875,10 +942,15 @@ class MultinomialHMM(_BaseHMM):
     def _set_emissionprob(self, emissionprob):
         emissionprob = np.asarray(emissionprob)
         if hasattr(self, 'n_symbols') and \
-               emissionprob.shape != (self.n_components, self.n_symbols):
+                emissionprob.shape != (self.n_components, self.n_symbols):
             raise ValueError('emissionprob must have shape '
                              '(n_components, n_symbols)')
 
+        # check if there exists a component whose value is exactly zero
+        # if so, add a small number and re-normalize
+        if not np.alltrue(emissionprob):
+            normalize(emissionprob)
+
         self._log_emissionprob = np.log(emissionprob)
         underflow_idx = np.isnan(self._log_emissionprob)
         self._log_emissionprob[underflow_idx] = NEGINF
@@ -901,8 +973,13 @@ class MultinomialHMM(_BaseHMM):
         self.random_state = check_random_state(self.random_state)
 
         if 'e' in params:
+            if not hasattr(self, 'n_symbols'):
+                symbols = set()
+                for o in obs:
+                    symbols = symbols.union(set(o))
+                self.n_symbols = len(symbols)
             emissionprob = normalize(self.random_state.rand(self.n_components,
-                self.n_symbols), 1)
+                                                            self.n_symbols), 1)
             self.emissionprob_ = emissionprob
 
     def _initialize_sufficient_statistics(self):
@@ -924,7 +1001,43 @@ class MultinomialHMM(_BaseHMM):
         super(MultinomialHMM, self)._do_mstep(stats, params)
         if 'e' in params:
             self.emissionprob_ = (stats['obs']
-                                 / stats['obs'].sum(1)[:, np.newaxis])
+                                  / stats['obs'].sum(1)[:, np.newaxis])
+
+    def _check_input_symbols(self, obs):
+        """check if input can be used for Multinomial.fit input must be both
+        positive integer array and every element must be continuous.
+        e.g. x = [0, 0, 2, 1, 3, 1, 1] is OK and y = [0, 0, 3, 5, 10] not
+        """
+
+        symbols = np.asanyarray(obs).flatten()
+
+        if symbols.dtype.kind != 'i':
+            # input symbols must be integer
+            return False
+
+        if len(symbols) == 1:
+            # input too short
+            return False
+
+        if np.any(symbols < 0):
+            # input containes negative intiger
+            return False
+
+        symbols.sort()
+        if np.any(np.diff(symbols) > 1):
+            # input is discontinous
+            return False
+
+        return True
+
+    def fit(self, obs, **kwargs):
+        err_msg = ("Input must be both positive integer array and "
+                   "every element must be continuous, but %s was given.")
+
+        if not self._check_input_symbols(obs):
+            raise ValueError(err_msg % obs)
+
+        return _BaseHMM.fit(self, obs, **kwargs)
 
 
 class GMMHMM(_BaseHMM):
@@ -932,6 +1045,16 @@ class GMMHMM(_BaseHMM):
 
     Attributes
     ----------
+    init_params : string, optional
+        Controls which parameters are initialized prior to training. Can \
+        contain any combination of 's' for startprob, 't' for transmat, 'm' \
+        for means, and 'c' for covars, etc.  Defaults to all parameters.
+
+    params : string, optional
+        Controls which parameters are updated in the training process.  Can
+        contain any combination of 's' for startprob, 't' for transmat,'m' for
+        means, and 'c' for covars, etc.  Defaults to all parameters.
+
     n_components : int
         Number of states in the model.
 
@@ -944,22 +1067,21 @@ class GMMHMM(_BaseHMM):
     gmms : array of GMM objects, length `n_components`
         GMM emission distributions for each state.
 
-    random_state: RandomState or an int seed (0 by default)
+    random_state : RandomState or an int seed (0 by default)
         A random number generator instance
 
+    n_iter : int, optional
+        Number of iterations to perform.
+
+    thresh : float, optional
+        Convergence threshold.
+
     Examples
     --------
     >>> from sklearn.hmm import GMMHMM
     >>> GMMHMM(n_components=2, n_mix=10, covariance_type='diag')
     ... # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-    GMMHMM(algorithm='viterbi', covariance_type='diag', covars_prior=0.01,
-        gmms=[GMM(covariance_type=None, init_params='wmc', min_covar=0.001,
-        n_components=10, n_init=1, n_iter=100, params='wmc', random_state=None,
-        thresh=0.01), GMM(covariance_type=None, init_params='wmc',
-        min_covar=0.001, n_components=10, n_init=1, n_iter=100, params='wmc',
-        random_state=None, thresh=0.01)], n_components=2, n_mix=10,
-        random_state=None, startprob=None, startprob_prior=1.0, transmat=None,
-        transmat_prior=1.0)
+    GMMHMM(algorithm='viterbi', covariance_type='diag',...
 
     See Also
     --------
@@ -967,9 +1089,11 @@ class GMMHMM(_BaseHMM):
     """
 
     def __init__(self, n_components=1, n_mix=1, startprob=None, transmat=None,
-            startprob_prior=None, transmat_prior=None, algorithm="viterbi",
-            gmms=None, covariance_type='diag', covars_prior=1e-2,
-            random_state=None):
+                 startprob_prior=None, transmat_prior=None,
+                 algorithm="viterbi", gmms=None, covariance_type='diag',
+                 covars_prior=1e-2, random_state=None, n_iter=10, thresh=1e-2,
+                 params=string.ascii_letters,
+                 init_params=string.ascii_letters):
         """Create a hidden Markov model with GMM emissions.
 
         Parameters
@@ -978,16 +1102,21 @@ class GMMHMM(_BaseHMM):
             Number of states.
         """
         _BaseHMM.__init__(self, n_components, startprob, transmat,
-                                     startprob_prior=startprob_prior,
-                                     transmat_prior=transmat_prior,
-                                     algorithm=algorithm,
-                                     random_state=random_state)
+                          startprob_prior=startprob_prior,
+                          transmat_prior=transmat_prior,
+                          algorithm=algorithm,
+                          random_state=random_state,
+                          n_iter=n_iter,
+                          thresh=thresh,
+                          params=params,
+                          init_params=init_params)
 
         # XXX: Hotfit for n_mix that is incompatible with the scikit's
         # BaseEstimator API
         self.n_mix = n_mix
         self._covariance_type = covariance_type
         self.covars_prior = covars_prior
+        self.gmms = gmms
         if gmms is None:
             gmms = []
             for x in xrange(self.n_components):
@@ -996,7 +1125,7 @@ class GMMHMM(_BaseHMM):
                 else:
                     g = GMM(n_mix, covariance_type=covariance_type)
                 gmms.append(g)
-        self.gmms = gmms
+        self.gmms_ = gmms
 
     # Read-only properties.
     @property
@@ -1008,24 +1137,24 @@ class GMMHMM(_BaseHMM):
         return self._covariance_type
 
     def _compute_log_likelihood(self, obs):
-        return np.array([g.score(obs) for g in self.gmms]).T
+        return np.array([g.score(obs) for g in self.gmms_]).T
 
     def _generate_sample_from_state(self, state, random_state=None):
-        return self.gmms[state].sample(1, random_state=random_state).flatten()
+        return self.gmms_[state].sample(1, random_state=random_state).flatten()
 
     def _init(self, obs, params='stwmc'):
         super(GMMHMM, self)._init(obs, params=params)
 
         allobs = np.concatenate(obs, 0)
-        for g in self.gmms:
+        for g in self.gmms_:
             g.set_params(init_params=params, n_iter=0)
             g.fit(allobs)
 
     def _initialize_sufficient_statistics(self):
         stats = super(GMMHMM, self)._initialize_sufficient_statistics()
-        stats['norm'] = [np.zeros(g.weights_.shape) for g in self.gmms]
-        stats['means'] = [np.zeros(np.shape(g.means_)) for g in self.gmms]
-        stats['covars'] = [np.zeros(np.shape(g.covars_)) for g in self.gmms]
+        stats['norm'] = [np.zeros(g.weights_.shape) for g in self.gmms_]
+        stats['means'] = [np.zeros(np.shape(g.means_)) for g in self.gmms_]
+        stats['covars'] = [np.zeros(np.shape(g.covars_)) for g in self.gmms_]
         return stats
 
     def _accumulate_sufficient_statistics(self, stats, obs, framelogprob,
@@ -1035,16 +1164,16 @@ class GMMHMM(_BaseHMM):
             stats, obs, framelogprob, posteriors, fwdlattice, bwdlattice,
             params)
 
-        for state, g in enumerate(self.gmms):
+        for state, g in enumerate(self.gmms_):
             _, lgmm_posteriors = g.eval(obs)
             lgmm_posteriors += np.log(posteriors[:, state][:, np.newaxis]
                                       + np.finfo(np.float).eps)
             gmm_posteriors = np.exp(lgmm_posteriors)
-            tmp_gmm = GMM(g.n_components, covariance_type=g._covariance_type)
+            tmp_gmm = GMM(g.n_components, covariance_type=g.covariance_type)
             n_features = g.means_.shape[1]
             tmp_gmm._set_covars(
                 distribute_covar_matrix_to_match_covariance_type(
-                    np.eye(n_features), g._covariance_type,
+                    np.eye(n_features), g.covariance_type,
                     g.n_components))
             norm = tmp_gmm._do_mstep(obs, gmm_posteriors, params)
 
@@ -1055,7 +1184,7 @@ class GMMHMM(_BaseHMM):
             if 'm' in params:
                 stats['means'][state] += tmp_gmm.means_ * norm[:, np.newaxis]
             if 'c' in params:
-                if tmp_gmm._covariance_type == 'tied':
+                if tmp_gmm.covariance_type == 'tied':
                     stats['covars'][state] += tmp_gmm.covars_ * norm.sum()
                 else:
                     cvnorm = np.copy(norm)
@@ -1068,7 +1197,7 @@ class GMMHMM(_BaseHMM):
         super(GMMHMM, self)._do_mstep(stats, params)
         # All that is left to do is to apply covars_prior to the
         # parameters updated in _accumulate_sufficient_statistics.
-        for state, g in enumerate(self.gmms):
+        for state, g in enumerate(self.gmms_):
             n_features = g.means_.shape[1]
             norm = stats['norm'][state]
             if 'w' in params:
@@ -1076,20 +1205,20 @@ class GMMHMM(_BaseHMM):
             if 'm' in params:
                 g.means_ = stats['means'][state] / norm[:, np.newaxis]
             if 'c' in params:
-                if g._covariance_type == 'tied':
+                if g.covariance_type == 'tied':
                     g.covars_ = ((stats['covars'][state]
                                  + self.covars_prior * np.eye(n_features))
-                                / norm.sum())
+                                 / norm.sum())
                 else:
                     cvnorm = np.copy(norm)
                     shape = np.ones(g.covars_.ndim)
                     shape[0] = np.shape(g.covars_)[0]
                     cvnorm.shape = shape
-                    if (g._covariance_type in ['spherical', 'diag']):
-                        g.covars_ = (stats['covars'][state]
-                                    + self.covars_prior) / cvnorm
-                    elif g._covariance_type == 'full':
+                    if (g.covariance_type in ['spherical', 'diag']):
+                        g.covars_ = (stats['covars'][state] +
+                                     self.covars_prior) / cvnorm
+                    elif g.covariance_type == 'full':
                         eye = np.eye(n_features)
                         g.covars_ = ((stats['covars'][state]
                                      + self.covars_prior * eye[np.newaxis])
-                                    / cvnorm)
+                                     / cvnorm)
