@@ -518,7 +518,9 @@ class OrthogonalMatchingPursuit(LinearModel, RegressorMixin):
         if self.n_nonzero_coefs is None and self.tol is None:
             # default for n_nonzero_coefs is 0.1 * n_features
             # but at least one.
-            self.n_nonzero_coefs = max(int(0.1 * n_features), 1)
+            self.n_nonzero_coefs_ = max(int(0.1 * n_features), 1)
+        else:
+            self.n_nonzero_coefs_ = self.n_nonzero_coefs
         if (Gram is not None or Xy is not None) and (self.fit_intercept
                                                      or self.normalize):
             warnings.warn('Mean subtraction (fit_intercept) and normalization '
@@ -546,14 +548,14 @@ class OrthogonalMatchingPursuit(LinearModel, RegressorMixin):
                 Gram /= X_std[:, np.newaxis]
 
             norms_sq = np.sum(y ** 2, axis=0) if self.tol is not None else None
-            self.coef_ = orthogonal_mp_gram(Gram, Xy, self.n_nonzero_coefs,
+            self.coef_ = orthogonal_mp_gram(Gram, Xy, self.n_nonzero_coefs_,
                                             self.tol, norms_sq,
                                             self.copy_Gram, True).T
         else:
             precompute_gram = self.precompute_gram
             if precompute_gram == 'auto':
                 precompute_gram = X.shape[0] > X.shape[1]
-            self.coef_ = orthogonal_mp(X, y, self.n_nonzero_coefs, self.tol,
+            self.coef_ = orthogonal_mp(X, y, self.n_nonzero_coefs_, self.tol,
                                        precompute_gram=self.precompute_gram,
                                        copy_X=self.copy_X).T
 
