@@ -384,14 +384,19 @@ def test_multitarget():
 
     for estimator in (linear_model.LassoLars(), linear_model.Lars()):
         estimator.fit(X, Y)
+        Y_pred = estimator.predict(X)
+        Y_dec = estimator.decision_function(X)
+        assert_array_almost_equal(Y_pred, Y_dec)
         alphas, active, coef, path = (estimator.alphas_, estimator.active_,
                                       estimator.coef_, estimator.coef_path_)
         for k in xrange(n_targets):
             estimator.fit(X, Y[:, k])
+            y_pred = estimator.predict(X)
             assert_array_almost_equal(alphas[k], estimator.alphas_)
             assert_array_almost_equal(active[k], estimator.active_)
             assert_array_almost_equal(coef[k], estimator.coef_)
             assert_array_almost_equal(path[k], estimator.coef_path_)
+            assert_array_almost_equal(Y_pred[:, k], y_pred)
 
 
 def test_lars_cv():
