@@ -1,13 +1,18 @@
 import numpy as np
 import scipy.sparse as sp
-from nose.tools import assert_true
-from numpy.testing import assert_almost_equal, assert_array_almost_equal, \
-                          assert_equal, assert_array_equal
-from sklearn import datasets
-from sklearn.metrics import mean_squared_error
+
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_equal
+from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_greater
 
+from sklearn import datasets
+from sklearn.metrics import mean_squared_error
+
 from sklearn.linear_model.base import LinearRegression
+from sklearn.linear_model.ridge import ridge_regression
 from sklearn.linear_model.ridge import Ridge
 from sklearn.linear_model.ridge import _RidgeGCV
 from sklearn.linear_model.ridge import RidgeCV
@@ -17,10 +22,10 @@ from sklearn.linear_model.ridge import RidgeClassifierCV
 
 from sklearn.cross_validation import KFold
 
-rng = np.random.RandomState(0)
 diabetes = datasets.load_diabetes()
 X_diabetes, y_diabetes = diabetes.data, diabetes.target
 ind = np.arange(X_diabetes.shape[0])
+rng = np.random.RandomState(0)
 rng.shuffle(ind)
 ind = ind[:200]
 X_diabetes, y_diabetes = X_diabetes[ind], y_diabetes[ind]
@@ -40,36 +45,60 @@ def test_ridge():
     TODO: for this test to be robust, we should use a dataset instead
     of np.random.
     """
+    rng = np.random.RandomState(0)
     alpha = 1.0
 
-    # With more samples than features
-    n_samples, n_features = 6, 5
-    y = rng.randn(n_samples)
-    X = rng.randn(n_samples, n_features)
+    for solver in ("sparse_cg", "dense_cholesky", "lsqr"):
+        # With more samples than features
+        n_samples, n_features = 6, 5
+        y = rng.randn(n_samples)
+        X = rng.randn(n_samples, n_features)
 
-    ridge = Ridge(alpha=alpha)
-    ridge.fit(X, y)
-    assert_equal(ridge.coef_.shape, (X.shape[1], ))
-    assert_greater(ridge.score(X, y), 0.5)
+        ridge = Ridge(alpha=alpha, solver=solver)
+        ridge.fit(X, y)
+        assert_equal(ridge.coef_.shape, (X.shape[1], ))
+        assert_greater(ridge.score(X, y), 0.47)
 
-    ridge.fit(X, y, sample_weight=np.ones(n_samples))
-    assert_greater(ridge.score(X, y), 0.5)
+        ridge.fit(X, y, sample_weight=np.ones(n_samples))
+        assert_greater(ridge.score(X, y), 0.47)
 
-    # With more features than samples
-    n_samples, n_features = 5, 10
-    y = rng.randn(n_samples)
-    X = rng.randn(n_samples, n_features)
-    ridge = Ridge(alpha=alpha)
-    ridge.fit(X, y)
-    assert_greater(ridge.score(X, y), .9)
+        # With more features than samples
+        n_samples, n_features = 5, 10
+        y = rng.randn(n_samples)
+        X = rng.randn(n_samples, n_features)
+        ridge = Ridge(alpha=alpha, solver=solver)
+        ridge.fit(X, y)
+        assert_greater(ridge.score(X, y), .9)
 
-    ridge.fit(X, y, sample_weight=np.ones(n_samples))
-    assert_greater(ridge.score(X, y), 0.9)
+        ridge.fit(X, y, sample_weight=np.ones(n_samples))
+        assert_greater(ridge.score(X, y), 0.9)
+
+
+def test_ridge_sample_weights():
+    rng = np.random.RandomState(0)
+    alpha = 1.0
+
+    for solver in ("sparse_cg", "dense_cholesky", "lsqr"):
+        for n_samples, n_features in ((6, 5), (5, 10)):
+            y = rng.randn(n_samples)
+            X = rng.randn(n_samples, n_features)
+            sample_weight = 1 + rng.rand(n_samples)
+
+            coefs = ridge_regression(X, y, alpha, sample_weight,
+                                     solver=solver)
+            # Sample weight can be implemented via a simple rescaling
+            # for the square loss
+            coefs2 = ridge_regression(
+                X * np.sqrt(sample_weight)[:, np.newaxis],
+                y * np.sqrt(sample_weight),
+                alpha, solver=solver)
+            assert_array_almost_equal(coefs, coefs2)
 
 
 def test_ridge_shapes():
     """Test shape of coef_ and intercept_
     """
+    rng = np.random.RandomState(0)
     n_samples, n_features = 5, 10
     X = rng.randn(n_samples, n_features)
     y = rng.randn(n_samples)
@@ -94,6 +123,7 @@ def test_ridge_shapes():
 def test_ridge_intercept():
     """Test intercept with multiple targets GH issue #708
     """
+    rng = np.random.RandomState(0)
     n_samples, n_features = 5, 10
     X = rng.randn(n_samples, n_features)
     y = rng.randn(n_samples)
@@ -136,6 +166,7 @@ def test_toy_ridge_object():
 def test_ridge_vs_lstsq():
     """On alpha=0., Ridge and OLS yield the same solution."""
 
+    rng = np.random.RandomState(0)
     # we need more samples than features
     n_samples, n_features = 5, 4
     y = rng.randn(n_samples)
@@ -160,7 +191,7 @@ def _test_ridge_loo(filter_):
     ret = []
 
     ridge_gcv = _RidgeGCV(fit_intercept=False)
-    ridge = Ridge(fit_intercept=False)
+    ridge = Ridge(alpha=1.0, fit_intercept=False)
 
     # generalized cross-validation (efficient leave-one-out)
     decomp = ridge_gcv._pre_compute(X_diabetes, y_diabetes)
@@ -187,8 +218,8 @@ def _test_ridge_loo(filter_):
     # generalized cross-validation (efficient leave-one-out,
     # SVD variation)
     decomp = ridge_gcv._pre_compute_svd(X_diabetes, y_diabetes)
-    errors3, c = ridge_gcv._errors_svd(1.0, y_diabetes, *decomp)
-    values3, c = ridge_gcv._values_svd(1.0, y_diabetes, *decomp)
+    errors3, c = ridge_gcv._errors_svd(ridge.alpha, y_diabetes, *decomp)
+    values3, c = ridge_gcv._values_svd(ridge.alpha, y_diabetes, *decomp)
 
     # check that efficient and SVD efficient LOO give same results
     assert_almost_equal(errors, errors3)
@@ -196,18 +227,24 @@ def _test_ridge_loo(filter_):
 
     # check best alpha
     ridge_gcv.fit(filter_(X_diabetes), y_diabetes)
-    best_alpha = ridge_gcv.best_alpha
-    ret.append(best_alpha)
+    alpha_ = ridge_gcv.alpha_
+    ret.append(alpha_)
 
     # check that we get same best alpha with custom loss_func
-    ridge_gcv2 = _RidgeGCV(fit_intercept=False, loss_func=mean_squared_error)
+    ridge_gcv2 = RidgeCV(fit_intercept=False, loss_func=mean_squared_error)
     ridge_gcv2.fit(filter_(X_diabetes), y_diabetes)
-    assert_equal(ridge_gcv2.best_alpha, best_alpha)
+    assert_equal(ridge_gcv2.alpha_, alpha_)
+
+    # check that we get same best alpha with custom score_func
+    func = lambda x, y: -mean_squared_error(x, y)
+    ridge_gcv3 = RidgeCV(fit_intercept=False, score_func=func)
+    ridge_gcv3.fit(filter_(X_diabetes), y_diabetes)
+    assert_equal(ridge_gcv3.alpha_, alpha_)
 
     # check that we get same best alpha with sample weights
     ridge_gcv.fit(filter_(X_diabetes), y_diabetes,
                   sample_weight=np.ones(n_samples))
-    assert_equal(ridge_gcv.best_alpha, best_alpha)
+    assert_equal(ridge_gcv.alpha_, alpha_)
 
     # simulate several responses
     Y = np.vstack((y_diabetes, y_diabetes)).T
@@ -270,7 +307,7 @@ def _test_ridge_classifiers(filter_):
         clf.fit(filter_(X_iris), y_iris)
         assert_equal(clf.coef_.shape, (n_classes, n_features))
         y_pred = clf.predict(filter_(X_iris))
-        assert_true(np.mean(y_iris == y_pred) >= 0.8)
+        assert_greater(np.mean(y_iris == y_pred), .79)
 
     n_samples = X_iris.shape[0]
     cv = KFold(n_samples, 5)
@@ -304,7 +341,7 @@ def test_dense_sparse():
         # test sparse matrix
         ret_sparse = test_func(SPARSE_FILTER)
         # test that the outputs are the same
-        if ret_dense != None and ret_sparse != None:
+        if ret_dense is not None and ret_sparse is not None:
             assert_array_almost_equal(ret_dense, ret_sparse, decimal=3)
 
 
@@ -345,3 +382,29 @@ def test_class_weights_cv():
     clf.fit(X, y)
 
     assert_array_equal(clf.predict([[-.2, 2]]), np.array([-1]))
+
+
+def test_ridgecv_store_cv_values():
+    """
+    Test _RidgeCV's store_cv_values attribute.
+    """
+    rng = rng = np.random.RandomState(42)
+
+    n_samples = 8
+    n_features = 5
+    x = rng.randn(n_samples, n_features)
+    alphas = [1e-1, 1e0, 1e1]
+    n_alphas = len(alphas)
+
+    r = RidgeCV(alphas=alphas, store_cv_values=True)
+
+    # with len(y.shape) == 1
+    y = rng.randn(n_samples)
+    r.fit(x, y)
+    assert_equal(r.cv_values_.shape, (n_samples, n_alphas))
+
+    # with len(y.shape) == 2
+    n_responses = 3
+    y = rng.randn(n_samples, n_responses)
+    r.fit(x, y)
+    assert_equal(r.cv_values_.shape, (n_samples, n_responses, n_alphas))
