@@ -1,17 +1,22 @@
+import unittest
+
 import numpy as np
 import scipy.sparse as sp
 
-from numpy.testing import assert_array_equal, assert_approx_equal
-from numpy.testing import assert_almost_equal, assert_array_almost_equal
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_almost_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_greater
+from sklearn.utils.testing import assert_less
+from sklearn.utils.testing import raises
+from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_true
+from sklearn.utils.testing import assert_equal
 
 from sklearn import linear_model, datasets, metrics
 from sklearn import preprocessing
 from sklearn.linear_model import SGDClassifier, SGDRegressor
-from sklearn.utils.testing import assert_greater, assert_less
-
-import unittest
-from nose.tools import raises
-from nose.tools import assert_raises, assert_true, assert_equal
+from sklearn.base import clone
 
 
 class SparseSGDClassifier(SGDClassifier):
@@ -28,6 +33,14 @@ class SparseSGDClassifier(SGDClassifier):
         X = sp.csr_matrix(X)
         return SGDClassifier.decision_function(self, X, *args, **kw)
 
+    def predict_proba(self, X, *args, **kw):
+        X = sp.csr_matrix(X)
+        return SGDClassifier.predict_proba(self, X, *args, **kw)
+
+    def predict_log_proba(self, X, *args, **kw):
+        X = sp.csr_matrix(X)
+        return SGDClassifier.predict_log_proba(self, X, *args, **kw)
+
 
 class SparseSGDRegressor(SGDRegressor):
 
@@ -133,7 +146,28 @@ class CommonTest(object):
         assert_true(hasattr(clf, "coef_"))
 
         clf.fit(X[:, :-1], Y)
-        assert_true(True)
+
+    def test_input_format(self):
+        """Input format tests. """
+        clf = self.factory(alpha=0.01, n_iter=5,
+                           shuffle=False)
+        Y_ = np.array(Y)[:, np.newaxis]
+        clf.fit(X, Y_)
+
+        Y_ = np.c_[Y_, Y_]
+        assert_raises(ValueError, clf.fit, X, Y_)
+
+    def test_clone(self):
+        """Test whether clone works ok. """
+        clf = self.factory(alpha=0.01, n_iter=5, penalty='l1')
+        clf = clone(clf)
+        clf.set_params(penalty='l2')
+        clf.fit(X, Y)
+
+        clf2 = self.factory(alpha=0.01, n_iter=5, penalty='l2')
+        clf2.fit(X, Y)
+
+        assert_array_equal(clf.coef_, clf2.coef_)
 
 
 class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
@@ -144,27 +178,37 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
     def test_sgd(self):
         """Check that SGD gives any results :-)"""
 
-        clf = self.factory(penalty='l2', alpha=0.01, fit_intercept=True,
-                      n_iter=10, shuffle=True)
-        clf.fit(X, Y)
-        #assert_almost_equal(clf.coef_[0], clf.coef_[1], decimal=7)
-        assert_array_equal(clf.predict(T), true_result)
+        for loss in ("hinge", "squared_hinge", "log", "modified_huber"):
+            clf = self.factory(penalty='l2', alpha=0.01, fit_intercept=True,
+                               loss=loss, n_iter=10, shuffle=True)
+            clf.fit(X, Y)
+            #assert_almost_equal(clf.coef_[0], clf.coef_[1], decimal=7)
+            assert_array_equal(clf.predict(T), true_result)
+
+    @raises(ValueError)
+    def test_sgd_bad_l1_ratio(self):
+        """Check whether expected ValueError on bad l1_ratio"""
+        self.factory(l1_ratio=1.1)
+
+    @raises(ValueError)
+    def test_sgd_bad_learning_rate_schedule(self):
+        """Check whether expected ValueError on bad learning_rate"""
+        self.factory(learning_rate="<unknown>")
+
+    @raises(ValueError)
+    def test_sgd_bad_eta0(self):
+        """Check whether expected ValueError on bad eta0"""
+        self.factory(eta0=0, learning_rate="constant")
+
+    @raises(ValueError)
+    def test_sgd_bad_alpha(self):
+        """Check whether expected ValueError on bad alpha"""
+        self.factory(alpha=-.1)
 
     @raises(ValueError)
     def test_sgd_bad_penalty(self):
         """Check whether expected ValueError on bad penalty"""
-        self.factory(penalty='foobar', rho=0.85)
-
-    def test_sgd_losses(self):
-        """Check whether losses and hyperparameters are set properly"""
-        clf = self.factory(loss='hinge')
-        assert_true(isinstance(clf.loss_function, linear_model.Hinge))
-
-        clf = self.factory(loss='log')
-        assert_true(isinstance(clf.loss_function, linear_model.Log))
-
-        clf = self.factory(loss='modified_huber')
-        assert_true(isinstance(clf.loss_function, linear_model.ModifiedHuber))
+        self.factory(penalty='foobar', l1_ratio=0.85)
 
     @raises(ValueError)
     def test_sgd_bad_loss(self):
@@ -263,18 +307,24 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf = self.factory().fit(X2, Y2, intercept_init=np.zeros((3,)))
 
     def test_sgd_proba(self):
-        """Check SGD.predict_proba for log loss only"""
+        """Check SGD.predict_proba"""
 
         # hinge loss does not allow for conditional prob estimate
         clf = self.factory(loss="hinge", alpha=0.01, n_iter=10).fit(X, Y)
         assert_raises(NotImplementedError, clf.predict_proba, [3, 2])
 
-        # log loss implements the logistic regression prob estimate
-        clf = self.factory(loss="log", alpha=0.01, n_iter=10).fit(X, Y)
-        p = clf.predict_proba([3, 2])
-        assert_greater(p, 0.5)
-        p = clf.predict_proba([-1, -1])
-        assert_less(p, 0.5)
+        # the log and modified_huber losses can output "probability" estimates
+        for loss in ("log", "modified_huber"):
+            clf = self.factory(loss=loss, alpha=0.01, n_iter=10).fit(X, Y)
+            p = clf.predict_proba([3, 2])
+            assert_true(p[0, 1] > 0.5)
+            p = clf.predict_proba([-1, -1])
+            assert_true(p[0, 1] < 0.5)
+
+            p = clf.predict_log_proba([3, 2])
+            assert_true(p[0, 1] > p[0, 0])
+            p = clf.predict_log_proba([-1, -1])
+            assert_true(p[0, 1] < p[0, 0])
 
     def test_sgd_l1(self):
         """Test L1 regularization"""
@@ -308,7 +358,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
 
         # we give a small weights to class 1
         clf = self.factory(alpha=0.1, n_iter=1000, fit_intercept=False,
-                class_weight={1: 0.001})
+                           class_weight={1: 0.001})
         clf.fit(X, y)
 
         # now the hyperplane should rotate clock-wise and
@@ -325,7 +375,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         X = [[1, 0], [0, 1]]
         y = [0, 1]
         clf_weighted = self.factory(alpha=0.1, n_iter=1000,
-                class_weight={0: 0.5, 1: 0.5})
+                                    class_weight={0: 0.5, 1: 0.5})
         clf_weighted.fit(X, y)
 
         # should be similar up to some epsilon due to learning rate schedule
@@ -356,12 +406,14 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         y = y[idx]
         clf = self.factory(alpha=0.0001, n_iter=1000,
                            class_weight=None).fit(X, y)
-        assert_approx_equal(metrics.f1_score(y, clf.predict(X)), 0.96, 2)
+        assert_almost_equal(metrics.f1_score(y, clf.predict(X)), 0.96,
+                            decimal=1)
 
         # make the same prediction using automated class_weight
         clf_auto = self.factory(alpha=0.0001, n_iter=1000,
                                 class_weight="auto").fit(X, y)
-        assert_approx_equal(metrics.f1_score(y, clf_auto.predict(X)), 0.96, 2)
+        assert_almost_equal(metrics.f1_score(y, clf_auto.predict(X)), 0.96,
+                            decimal=1)
 
         # Make sure that in the balanced case it does not change anything
         # to use "auto"
@@ -423,7 +475,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         clf.partial_fit(X3, Y3)
 
     def test_partial_fit_binary(self):
-        third = X.shape[0] / 3
+        third = X.shape[0] // 3
         clf = self.factory(alpha=0.01)
         classes = np.unique(Y)
 
@@ -442,7 +494,7 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
         assert_array_equal(y_pred, true_result)
 
     def test_partial_fit_multiclass(self):
-        third = X2.shape[0] / 3
+        third = X2.shape[0] // 3
         clf = self.factory(alpha=0.01)
         classes = np.unique(Y2)
 
@@ -484,6 +536,26 @@ class DenseSGDClassifierTestCase(unittest.TestCase, CommonTest):
     def test_partial_fit_equal_fit_invscaling(self):
         self._test_partial_fit_equal_fit("invscaling")
 
+    def test_regression_losses(self):
+        clf = self.factory(alpha=0.01, learning_rate="constant",
+                           eta0=0.1, loss="epsilon_insensitive")
+        clf.fit(X, Y)
+        assert_equal(1.0, np.mean(clf.predict(X) == Y))
+
+        clf = self.factory(alpha=0.01, learning_rate="constant",
+                           eta0=0.1, loss="squared_epsilon_insensitive")
+        clf.fit(X, Y)
+        assert_equal(1.0, np.mean(clf.predict(X) == Y))
+
+        clf = self.factory(alpha=0.01, loss="huber")
+        clf.fit(X, Y)
+        assert_equal(1.0, np.mean(clf.predict(X) == Y))
+
+        clf = self.factory(alpha=0.01, learning_rate="constant", eta0=0.01,
+                           loss="squared_loss")
+        clf.fit(X, Y)
+        assert_equal(1.0, np.mean(clf.predict(X) == Y))
+
 
 class SparseSGDClassifierTestCase(DenseSGDClassifierTestCase):
     """Run exactly the same tests using the sparse representation variant"""
@@ -509,16 +581,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
     @raises(ValueError)
     def test_sgd_bad_penalty(self):
         """Check whether expected ValueError on bad penalty"""
-        self.factory(penalty='foobar', rho=0.85)
-
-    def test_sgd_losses(self):
-        """Check whether losses and hyperparameters are set properly"""
-        clf = self.factory(loss='squared_loss')
-        assert_true(isinstance(clf.loss_function, linear_model.SquaredLoss))
-
-        clf = self.factory(loss='huber', p=0.5)
-        assert_true(isinstance(clf.loss_function, linear_model.Huber))
-        assert_equal(clf.p, 0.5)
+        self.factory(penalty='foobar', l1_ratio=0.85)
 
     @raises(ValueError)
     def test_sgd_bad_loss(self):
@@ -549,6 +612,32 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
         score = clf.score(X, y)
         assert_greater(score, 0.5)
 
+    def test_sgd_epsilon_insensitive(self):
+        xmin, xmax = -5, 5
+        n_samples = 100
+        X = np.linspace(xmin, xmax, n_samples).reshape(n_samples, 1)
+
+        # simple linear function without noise
+        y = 0.5 * X.ravel()
+
+        clf = self.factory(loss='epsilon_insensitive', epsilon=0.01,
+                           alpha=0.1, n_iter=20,
+                           fit_intercept=False)
+        clf.fit(X, y)
+        score = clf.score(X, y)
+        assert_true(score > 0.99)
+
+        # simple linear function with noise
+        y = 0.5 * X.ravel() \
+            + np.random.randn(n_samples, 1).ravel()
+
+        clf = self.factory(loss='epsilon_insensitive', epsilon=0.01,
+                           alpha=0.1, n_iter=20,
+                           fit_intercept=False)
+        clf.fit(X, y)
+        score = clf.score(X, y)
+        assert_true(score > 0.5)
+
     def test_sgd_huber_fit(self):
         xmin, xmax = -5, 5
         n_samples = 100
@@ -558,7 +647,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
         # simple linear function without noise
         y = 0.5 * X.ravel()
 
-        clf = self.factory(loss="huber", p=0.1, alpha=0.1, n_iter=20,
+        clf = self.factory(loss="huber", epsilon=0.1, alpha=0.1, n_iter=20,
                            fit_intercept=False)
         clf.fit(X, y)
         score = clf.score(X, y)
@@ -567,7 +656,7 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
         # simple linear function with noise
         y = 0.5 * X.ravel() + rng.randn(n_samples, 1).ravel()
 
-        clf = self.factory(loss="huber", p=0.1, alpha=0.1, n_iter=20,
+        clf = self.factory(loss="huber", epsilon=0.1, alpha=0.1, n_iter=20,
                            fit_intercept=False)
         clf.fit(X, y)
         score = clf.score(X, y)
@@ -586,20 +675,22 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
 
         # XXX: alpha = 0.1 seems to cause convergence problems
         for alpha in [0.01, 0.001]:
-            for rho in [0.5, 0.8, 1.0]:
-                cd = linear_model.ElasticNet(alpha=alpha, rho=rho,
+            for l1_ratio in [0.5, 0.8, 1.0]:
+                cd = linear_model.ElasticNet(alpha=alpha, l1_ratio=l1_ratio,
                                              fit_intercept=False)
                 cd.fit(X, y)
                 sgd = self.factory(penalty='elasticnet', n_iter=50,
-                                   alpha=alpha, rho=rho, fit_intercept=False)
+                                   alpha=alpha, l1_ratio=l1_ratio,
+                                   fit_intercept=False)
                 sgd.fit(X, y)
                 err_msg = ("cd and sgd did not converge to comparable "
-                           "results for alpha=%f and rho=%f" % (alpha, rho))
+                           "results for alpha=%f and l1_ratio=%f"
+                           % (alpha, l1_ratio))
                 assert_almost_equal(cd.coef_, sgd.coef_, decimal=2,
                                     err_msg=err_msg)
 
     def test_partial_fit(self):
-        third = X.shape[0] / 3
+        third = X.shape[0] // 3
         clf = self.factory(alpha=0.01)
 
         clf.partial_fit(X[:third], Y[:third])
@@ -638,6 +729,11 @@ class DenseSGDRegressorTestCase(unittest.TestCase):
     def test_partial_fit_equal_fit_invscaling(self):
         self._test_partial_fit_equal_fit("invscaling")
 
+    def test_loss_function_epsilon(self):
+        clf = self.factory(epsilon=0.9)
+        clf.set_params(epsilon=0.1)
+        assert clf.loss_functions['huber'][1] == 0.1
+
 
 class SparseSGDRegressorTestCase(DenseSGDRegressorTestCase):
     """Run exactly the same tests using the sparse representation variant"""
