@@ -16,7 +16,7 @@ from scipy import linalg
 from scipy.spatial.distance import cdist
 
 from ..utils import check_random_state
-from ..utils.extmath import norm, logsumexp
+from ..utils.extmath import norm, logsumexp, pinvh
 from .. import cluster
 from .gmm import GMM
 
@@ -190,8 +190,8 @@ class DPGMM(GMM):
     GMM : Finite Gaussian mixture model fit with EM
 
     VBGMM : Finite Gaussian mixture model fit with a variational
-    algorithm, better for situations where there might be too little
-    data to get a good estimate of the covariance matrix.
+        algorithm, better for situations where there might be too little
+        data to get a good estimate of the covariance matrix.
     """
 
     def __init__(self, n_components=1, covariance_type='diag', alpha=1.0,
@@ -207,15 +207,15 @@ class DPGMM(GMM):
 
     def _get_precisions(self):
         """Return precisions as a full matrix."""
-        if self._covariance_type == 'full':
+        if self.covariance_type == 'full':
             return self.precs_
-        elif self._covariance_type in ['diag', 'spherical']:
+        elif self.covariance_type in ['diag', 'spherical']:
             return [np.diag(cov) for cov in self.precs_]
-        elif self._covariance_type == 'tied':
+        elif self.covariance_type == 'tied':
             return [self.precs_] * self.n_components
 
     def _get_covars(self):
-        return [linalg.pinv(c) for c in self._get_precisions()]
+        return [pinvh(c) for c in self._get_precisions()]
 
     def _set_covars(self, covars):
         raise NotImplementedError("""The variational algorithm does
@@ -261,12 +261,12 @@ class DPGMM(GMM):
         # Free memory and developers cognitive load:
         del dgamma1, dgamma2, sd
 
-        if self._covariance_type not in ['full', 'tied', 'diag', 'spherical']:
+        if self.covariance_type not in ['full', 'tied', 'diag', 'spherical']:
             raise NotImplementedError("This ctype is not implemented: %s"
-                                      % self._covariance_type)
+                                      % self.covariance_type)
         p = _bound_state_log_lik(X, self._initial_bound + self.bound_prec_,
                                  self.precs_, self.means_,
-                                 self._covariance_type)
+                                 self.covariance_type)
         z = p + dgamma
         z = log_normalize(z, axis=-1)
         bound = np.sum(z * p, axis=-1)
@@ -285,13 +285,13 @@ class DPGMM(GMM):
         """Update the variational distributions for the means"""
         n_features = X.shape[1]
         for k in xrange(self.n_components):
-            if self._covariance_type in ['spherical', 'diag']:
+            if self.covariance_type in ['spherical', 'diag']:
                 num = np.sum(z.T[k].reshape((-1, 1)) * X, axis=0)
                 num *= self.precs_[k]
                 den = 1. + self.precs_[k] * np.sum(z.T[k])
                 self.means_[k] = num / den
-            elif self._covariance_type in ['tied', 'full']:
-                if self._covariance_type == 'tied':
+            elif self.covariance_type in ['tied', 'full']:
+                if self.covariance_type == 'tied':
                     cov = self.precs_
                 else:
                     cov = self.precs_[k]
@@ -303,7 +303,7 @@ class DPGMM(GMM):
     def _update_precisions(self, X, z):
         """Update the variational distributions for the precisions"""
         n_features = X.shape[1]
-        if self._covariance_type == 'spherical':
+        if self.covariance_type == 'spherical':
             self.dof_ = 0.5 * n_features * np.sum(z, axis=0)
             for k in xrange(self.n_components):
                 # could be more memory efficient ?
@@ -315,7 +315,7 @@ class DPGMM(GMM):
                         digamma(self.dof_[k]) - np.log(self.scale_[k])))
             self.precs_ = np.tile(self.dof_ / self.scale_, [n_features, 1]).T
 
-        elif self._covariance_type == 'diag':
+        elif self.covariance_type == 'diag':
             for k in xrange(self.n_components):
                 self.dof_[k].fill(1. + 0.5 * np.sum(z.T[k], axis=0))
                 sq_diff = (X - self.means_[k]) ** 2  # see comment above
@@ -323,36 +323,35 @@ class DPGMM(GMM):
                     z.T[k], (sq_diff + 1))
                 self.precs_[k] = self.dof_[k] / self.scale_[k]
                 self.bound_prec_[k] = 0.5 * np.sum(digamma(self.dof_[k])
-                                                    - np.log(self.scale_[k]))
+                                                   - np.log(self.scale_[k]))
                 self.bound_prec_[k] -= 0.5 * np.sum(self.precs_[k])
 
-        elif self._covariance_type == 'tied':
+        elif self.covariance_type == 'tied':
             self.dof_ = 2 + X.shape[0] + n_features
             self.scale_ = (X.shape[0] + 1) * np.identity(n_features)
             for k in xrange(self.n_components):
                     diff = X - self.means_[k]
                     self.scale_ += np.dot(diff.T, z[:, k:k + 1] * diff)
-            self.scale_ = linalg.pinv(self.scale_)
+            self.scale_ = pinvh(self.scale_)
             self.precs_ = self.dof_ * self.scale_
             self.det_scale_ = linalg.det(self.scale_)
             self.bound_prec_ = 0.5 * wishart_log_det(
                 self.dof_, self.scale_, self.det_scale_, n_features)
             self.bound_prec_ -= 0.5 * self.dof_ * np.trace(self.scale_)
 
-        elif self._covariance_type == 'full':
+        elif self.covariance_type == 'full':
             for k in xrange(self.n_components):
                 sum_resp = np.sum(z.T[k])
                 self.dof_[k] = 2 + sum_resp + n_features
                 self.scale_[k] = (sum_resp + 1) * np.identity(n_features)
                 diff = X - self.means_[k]
                 self.scale_[k] += np.dot(diff.T, z[:, k:k + 1] * diff)
-                self.scale_[k] = linalg.pinv(self.scale_[k])
+                self.scale_[k] = pinvh(self.scale_[k])
                 self.precs_[k] = self.dof_[k] * self.scale_[k]
                 self.det_scale_[k] = linalg.det(self.scale_[k])
-                self.bound_prec_[k] = 0.5 * wishart_log_det(self.dof_[k],
-                                                            self.scale_[k],
-                                                            self.det_scale_[k],
-                                                           n_features)
+                self.bound_prec_[k] = 0.5 * wishart_log_det(
+                    self.dof_[k], self.scale_[k], self.det_scale_[k],
+                    n_features)
                 self.bound_prec_[k] -= 0.5 * self.dof_[k] * np.trace(
                     self.scale_[k])
 
@@ -365,9 +364,9 @@ class DPGMM(GMM):
         Note: this is very expensive and should not be used by default."""
         if self.verbose:
             print "Bound after updating %8s: %f" % (n, self.lower_bound(X, z))
-            if end == True:
+            if end:
                 print "Cluster proportions:", self.gamma_.T[1]
-                print "covariance_type:", self._covariance_type
+                print "covariance_type:", self.covariance_type
 
     def _do_mstep(self, X, z, params):
         """Maximize the variational lower bound
@@ -391,17 +390,17 @@ class DPGMM(GMM):
         """The variational lower bound for the concentration parameter."""
         logprior = gammaln(self.alpha) * self.n_components
         logprior += np.sum((self.alpha - 1) * (
-                digamma(self.gamma_.T[2]) - digamma(self.gamma_.T[1] +
-                                                    self.gamma_.T[2])))
+            digamma(self.gamma_.T[2]) - digamma(self.gamma_.T[1] +
+                                                self.gamma_.T[2])))
         logprior += np.sum(- gammaln(self.gamma_.T[1] + self.gamma_.T[2]))
         logprior += np.sum(gammaln(self.gamma_.T[1]) +
                            gammaln(self.gamma_.T[2]))
         logprior -= np.sum((self.gamma_.T[1] - 1) * (
-                digamma(self.gamma_.T[1]) - digamma(self.gamma_.T[1] +
-                                                     self.gamma_.T[2])))
+            digamma(self.gamma_.T[1]) - digamma(self.gamma_.T[1] +
+                                                self.gamma_.T[2])))
         logprior -= np.sum((self.gamma_.T[2] - 1) * (
-                digamma(self.gamma_.T[2]) - digamma(self.gamma_.T[1] +
-                                                    self.gamma_.T[2])))
+            digamma(self.gamma_.T[2]) - digamma(self.gamma_.T[1] +
+                                                self.gamma_.T[2])))
         return logprior
 
     def _bound_means(self):
@@ -414,20 +413,20 @@ class DPGMM(GMM):
     def _bound_precisions(self):
         """Returns the bound term related to precisions"""
         logprior = 0.
-        if self._covariance_type == 'spherical':
+        if self.covariance_type == 'spherical':
             logprior += np.sum(gammaln(self.dof_))
             logprior -= np.sum(
                 (self.dof_ - 1) * digamma(np.maximum(0.5, self.dof_)))
-            logprior += np.sum(- np.log(self.scale_) + self.dof_ -\
-                                     self.precs_[:, 0])
-        elif self._covariance_type == 'diag':
+            logprior += np.sum(- np.log(self.scale_) + self.dof_
+                               - self.precs_[:, 0])
+        elif self.covariance_type == 'diag':
             logprior += np.sum(gammaln(self.dof_))
             logprior -= np.sum(
                 (self.dof_ - 1) * digamma(np.maximum(0.5, self.dof_)))
             logprior += np.sum(- np.log(self.scale_) + self.dof_ - self.precs_)
-        elif self._covariance_type == 'tied':
+        elif self.covariance_type == 'tied':
             logprior += _bound_wishart(self.dof_, self.scale_, self.det_scale_)
-        elif self._covariance_type == 'full':
+        elif self.covariance_type == 'full':
             for k in xrange(self.n_components):
                 logprior += _bound_wishart(self.dof_[k],
                                            self.scale_[k],
@@ -456,19 +455,25 @@ class DPGMM(GMM):
 
     def lower_bound(self, X, z):
         """returns a lower bound on model evidence based on X and membership"""
-        if self._covariance_type not in ['full', 'tied', 'diag', 'spherical']:
+        if self.covariance_type not in ['full', 'tied', 'diag', 'spherical']:
             raise NotImplementedError("This ctype is not implemented: %s"
-                                      % self._covariance_type)
+                                      % self.covariance_type)
 
         X = np.asarray(X)
         if X.ndim == 1:
             X = X[:, np.newaxis]
-        c = np.sum(z * _bound_state_log_lik(
-                X, self._initial_bound + self.bound_prec_,
-                self.precs_, self.means_, self._covariance_type))
+        c = np.sum(z * _bound_state_log_lik(X, self._initial_bound +
+                                            self.bound_prec_, self.precs_,
+                                            self.means_, self.covariance_type))
 
         return c + self._logprior(z)
 
+    def _set_weights(self):
+        for i in xrange(self.n_components):
+            self.weights_[i] = self.gamma_[i, 1] / (self.gamma_[i, 1]
+                                                    + self.gamma_[i, 2])
+        self.weights_ /= np.sum(self.weights_)
+
     def fit(self, X, **kwargs):
         """Estimate model parameters with the variational
         algorithm.
@@ -491,8 +496,9 @@ class DPGMM(GMM):
         self.random_state = check_random_state(self.random_state)
         if kwargs:
             warnings.warn("Setting parameters in the 'fit' method is"
-                    "deprecated. Set it on initialization instead.",
-                     DeprecationWarning)
+                          "deprecated and will be removed in 0.14. Set it on"
+                          "initialization instead.",
+                          DeprecationWarning)
             # initialisations for in case the user still adds parameters to fit
             # so things don't break
             if 'n_iter' in kwargs:
@@ -519,20 +525,20 @@ class DPGMM(GMM):
 
         if 'm' in self.init_params or not hasattr(self, 'means_'):
             self.means_ = cluster.KMeans(
-                k=self.n_components,
+                n_clusters=self.n_components,
                 random_state=self.random_state).fit(X).cluster_centers_[::-1]
 
         if 'w' in self.init_params or not hasattr(self, 'weights_'):
             self.weights_ = np.tile(1.0 / self.n_components, self.n_components)
 
         if 'c' in self.init_params or not hasattr(self, 'precs_'):
-            if self._covariance_type == 'spherical':
+            if self.covariance_type == 'spherical':
                 self.dof_ = np.ones(self.n_components)
                 self.scale_ = np.ones(self.n_components)
                 self.precs_ = np.ones((self.n_components, n_features))
                 self.bound_prec_ = 0.5 * n_features * (
                     digamma(self.dof_) - np.log(self.scale_))
-            elif self._covariance_type == 'diag':
+            elif self.covariance_type == 'diag':
                 self.dof_ = 1 + 0.5 * n_features
                 self.dof_ *= np.ones((self.n_components, n_features))
                 self.scale_ = np.ones((self.n_components, n_features))
@@ -540,7 +546,7 @@ class DPGMM(GMM):
                 self.bound_prec_ = 0.5 * (np.sum(digamma(self.dof_) -
                                                  np.log(self.scale_), 1))
                 self.bound_prec_ -= 0.5 * np.sum(self.precs_, 1)
-            elif self._covariance_type == 'tied':
+            elif self.covariance_type == 'tied':
                 self.dof_ = 1.
                 self.scale_ = np.identity(n_features)
                 self.precs_ = np.identity(n_features)
@@ -548,13 +554,13 @@ class DPGMM(GMM):
                 self.bound_prec_ = 0.5 * wishart_log_det(
                     self.dof_, self.scale_, self.det_scale_, n_features)
                 self.bound_prec_ -= 0.5 * self.dof_ * np.trace(self.scale_)
-            elif self._covariance_type == 'full':
+            elif self.covariance_type == 'full':
                 self.dof_ = (1 + self.n_components + X.shape[0])
                 self.dof_ *= np.ones(self.n_components)
                 self.scale_ = [2 * np.identity(n_features)
-                           for i in xrange(self.n_components)]
+                               for i in xrange(self.n_components)]
                 self.precs_ = [np.identity(n_features)
-                                for i in xrange(self.n_components)]
+                               for i in xrange(self.n_components)]
                 self.det_scale_ = np.ones(self.n_components)
                 self.bound_prec_ = np.zeros(self.n_components)
                 for k in xrange(self.n_components):
@@ -581,6 +587,8 @@ class DPGMM(GMM):
             # Maximization step
             self._do_mstep(X, z, self.params)
 
+        self._set_weights()
+
         return self
 
 
@@ -648,7 +656,7 @@ class VBGMM(DPGMM):
     --------
     GMM : Finite Gaussian mixture model fit with EM
     DPGMM : Ininite Gaussian mixture model, using the dirichlet
-    process, fit with a variational algorithm
+        process, fit with a variational algorithm
     """
 
     def __init__(self, n_components=1, covariance_type='diag', alpha=1.0,
@@ -692,12 +700,12 @@ class VBGMM(DPGMM):
         bound = np.zeros(X.shape[0])
         dg = digamma(self.gamma_) - digamma(np.sum(self.gamma_))
 
-        if self._covariance_type not in ['full', 'tied', 'diag', 'spherical']:
+        if self.covariance_type not in ['full', 'tied', 'diag', 'spherical']:
             raise NotImplementedError("This ctype is not implemented: %s"
-                                      % self._covariance_type)
-        p = _bound_state_log_lik(
-                X, self._initial_bound + self.bound_prec_,
-                self.precs_, self.means_, self._covariance_type)
+                                      % self.covariance_type)
+        p = _bound_state_log_lik(X, self._initial_bound + self.bound_prec_,
+                                 self.precs_, self.means_,
+                                 self.covariance_type)
 
         z = p + dg
         z = log_normalize(z, axis=-1)
@@ -739,6 +747,10 @@ class VBGMM(DPGMM):
         Note: this is very expensive and should not be used by default."""
         if self.verbose:
             print "Bound after updating %8s: %f" % (n, self.lower_bound(X, z))
-            if end == True:
+            if end:
                 print "Cluster proportions:", self.gamma_
-                print "covariance_type:", self._covariance_type
+                print "covariance_type:", self.covariance_type
+
+    def _set_weights(self):
+        self.weights_[:] = self.gamma_
+        self.weights_ /= np.sum(self.weights_)
