@@ -10,11 +10,10 @@ of Gaussian Mixture Models.
 #         Bertrand Thirion <bertrand.thirion@inria.fr>
 
 import numpy as np
-import warnings
 
 from ..base import BaseEstimator
-from ..utils import check_random_state, deprecated
-from ..utils.extmath import logsumexp
+from ..utils import check_random_state
+from ..utils.extmath import logsumexp, pinvh
 from .. import cluster
 
 EPS = np.finfo(float).eps
@@ -163,7 +162,7 @@ class GMM(BaseEstimator):
         Covariance parameters for each mixture component.  The shape
         depends on `covariance_type`::
 
-            (n_components,)                        if 'spherical',
+            (n_components, n_features)             if 'spherical',
             (n_features, n_features)               if 'tied',
             (n_components, n_features)             if 'diag',
             (n_components, n_features, n_features) if 'full'
@@ -196,7 +195,7 @@ class GMM(BaseEstimator):
     >>> obs = np.concatenate((np.random.randn(100, 1),
     ...                       10 + np.random.randn(300, 1)))
     >>> g.fit(obs) # doctest: +NORMALIZE_WHITESPACE
-    GMM(covariance_type=None, init_params='wmc', min_covar=0.001,
+    GMM(covariance_type='diag', init_params='wmc', min_covar=0.001,
             n_components=2, n_init=1, n_iter=100, params='wmc',
             random_state=None, thresh=0.01)
     >>> np.round(g.weights_, 2)
@@ -207,14 +206,14 @@ class GMM(BaseEstimator):
     >>> np.round(g.covars_, 2) #doctest: +SKIP
     array([[[ 1.02]],
            [[ 0.96]]])
-    >>> g.predict([[0], [2], [9], [10]])
-    array([1, 1, 0, 0])
+    >>> g.predict([[0], [2], [9], [10]]) #doctest: +ELLIPSIS
+    array([1, 1, 0, 0]...)
     >>> np.round(g.score([[0], [2], [9], [10]]), 2)
     array([-2.19, -4.58, -1.75, -1.21])
     >>> # Refit the model on new data (initial parameters remain the
     >>> # same), this time with an even split between the two modes.
     >>> g.fit(20 * [[0]] +  20 * [[10]]) # doctest: +NORMALIZE_WHITESPACE
-    GMM(covariance_type=None, init_params='wmc', min_covar=0.001,
+    GMM(covariance_type='diag', init_params='wmc', min_covar=0.001,
             n_components=2, n_init=1, n_iter=100, params='wmc',
             random_state=None, thresh=0.01)
     >>> np.round(g.weights_, 2)
@@ -226,7 +225,7 @@ class GMM(BaseEstimator):
                  random_state=None, thresh=1e-2, min_covar=1e-3,
                  n_iter=100, n_init=1, params='wmc', init_params='wmc'):
         self.n_components = n_components
-        self._covariance_type = covariance_type
+        self.covariance_type = covariance_type
         self.thresh = thresh
         self.min_covar = min_covar
         self.random_state = random_state
@@ -237,7 +236,7 @@ class GMM(BaseEstimator):
 
         if not covariance_type in ['spherical', 'tied', 'diag', 'full']:
             raise ValueError('Invalid value for covariance_type: %s' %
-                            covariance_type)
+                             covariance_type)
 
         if n_init < 1:
             raise ValueError('GMM estimation requires at least one run')
@@ -257,19 +256,19 @@ class GMM(BaseEstimator):
             (`n_states`, `n_features`)                if 'diag',
             (`n_states`, `n_features`, `n_features`)  if 'full'
             """
-        if self._covariance_type == 'full':
+        if self.covariance_type == 'full':
             return self.covars_
-        elif self._covariance_type == 'diag':
+        elif self.covariance_type == 'diag':
             return [np.diag(cov) for cov in self.covars_]
-        elif self._covariance_type == 'tied':
+        elif self.covariance_type == 'tied':
             return [self.covars_] * self.n_components
-        elif self._covariance_type == 'spherical':
+        elif self.covariance_type == 'spherical':
             return [np.diag(cov) for cov in self.covars_]
 
     def _set_covars(self, covars):
         """Provide values for covariance"""
         covars = np.asarray(covars)
-        _validate_covars(covars, self._covariance_type, self.n_components)
+        _validate_covars(covars, self.covariance_type, self.n_components)
         self.covars_ = covars
 
     def eval(self, X):
@@ -301,38 +300,13 @@ class GMM(BaseEstimator):
         if X.shape[1] != self.means_.shape[1]:
             raise ValueError('the shape of X  is not compatible with self')
 
-        lpr = (log_multivariate_normal_density(
-                X, self.means_, self.covars_, self._covariance_type)
+        lpr = (log_multivariate_normal_density(X, self.means_, self.covars_,
+                                               self.covariance_type)
                + np.log(self.weights_))
         logprob = logsumexp(lpr, axis=1)
         responsibilities = np.exp(lpr - logprob[:, np.newaxis])
         return logprob, responsibilities
 
-    @deprecated("""will be removed in v0.12;
-    use the score or predict method instead, depending on the question""")
-    def decode(self, X):
-        """Find most likely mixture components for each point in X.
-
-        DEPRECATED IN VERSION 0.10; WILL BE REMOVED IN VERSION 0.12
-        use the score or predict method instead, depending on the question.
-
-        Parameters
-        ----------
-        X : array_like, shape (n, n_features)
-            List of n_features-dimensional data points.  Each row
-            corresponds to a single data point.
-
-        Returns
-        -------
-        logprobs : array_like, shape (n_samples,)
-            Log probability of each point in `obs` under the model.
-
-        components : array_like, shape (n_samples,)
-            Index of the most likelihod mixture components for each observation
-        """
-        logprob, posteriors = self.eval(X)
-        return logprob, posteriors.argmax(axis=1)
-
     def score(self, X):
         """Compute the log probability under the model.
 
@@ -381,16 +355,6 @@ class GMM(BaseEstimator):
         logprob, responsibilities = self.eval(X)
         return responsibilities
 
-    @deprecated("""will be removed in v0.12;
-    use the score or predict method instead, depending on the question""")
-    def rvs(self, n_samples=1, random_state=None):
-        """Generate random samples from the model.
-
-        DEPRECATED IN VERSION 0.11; WILL BE REMOVED IN VERSION 0.12
-        use sample instead
-        """
-        return self.sample(n_samples, random_state)
-
     def sample(self, n_samples=1, random_state=None):
         """Generate random samples from the model.
 
@@ -420,18 +384,18 @@ class GMM(BaseEstimator):
             # number of those occurrences
             num_comp_in_X = comp_in_X.sum()
             if num_comp_in_X > 0:
-                if self._covariance_type == 'tied':
+                if self.covariance_type == 'tied':
                     cv = self.covars_
-                elif self._covariance_type == 'spherical':
+                elif self.covariance_type == 'spherical':
                     cv = self.covars_[comp][0]
                 else:
                     cv = self.covars_[comp]
                 X[comp_in_X] = sample_gaussian(
-                    self.means_[comp], cv, self._covariance_type,
+                    self.means_[comp], cv, self.covariance_type,
                     num_comp_in_X, random_state=random_state).T
         return X
 
-    def fit(self, X, **kwargs):
+    def fit(self, X):
         """Estimate model parameters with the expectation-maximization
         algorithm.
 
@@ -448,37 +412,21 @@ class GMM(BaseEstimator):
             corresponds to a single data point.
         """
         ## initialization step
-        X = np.asarray(X)
+        X = np.asarray(X, dtype=np.float)
         if X.ndim == 1:
             X = X[:, np.newaxis]
         if X.shape[0] < self.n_components:
             raise ValueError(
                 'GMM estimation with %s components, but got only %s samples' %
                 (self.n_components, X.shape[0]))
-        if kwargs:
-            warnings.warn("Setting parameters in the 'fit' method is"
-                    "deprecated. Set it on initialization instead.",
-                    DeprecationWarning)
-            # initialisations for in case the user still adds parameters to fit
-            # so things don't break
-            if 'n_iter' in kwargs:
-                self.n_iter = kwargs['n_iter']
-            if 'n_init' in kwargs:
-                if kwargs['n_init'] < 1:
-                    raise ValueError('GMM estimation requires at least one run')
-                else:
-                    self.n_init = kwargs['n_init']
-            if 'params' in kwargs:
-                self.params = kwargs['params']
-            if 'init_params' in kwargs:
-                self.init_params = kwargs['init_params']
 
-        max_log_prob = - np.infty
+        max_log_prob = -np.infty
 
         for _ in range(self.n_init):
             if 'm' in self.init_params or not hasattr(self, 'means_'):
                 self.means_ = cluster.KMeans(
-                    k=self.n_components).fit(X).cluster_centers_
+                    n_clusters=self.n_components,
+                    random_state=self.random_state).fit(X).cluster_centers_
 
             if 'w' in self.init_params or not hasattr(self, 'weights_'):
                 self.weights_ = np.tile(1.0 / self.n_components,
@@ -490,7 +438,7 @@ class GMM(BaseEstimator):
                     cv.shape = (1, 1)
                 self.covars_ = \
                     distribute_covar_matrix_to_match_covariance_type(
-                    cv, self._covariance_type, self.n_components)
+                        cv, self.covariance_type, self.n_components)
 
             # EM algorithms
             log_likelihood = []
@@ -508,7 +456,8 @@ class GMM(BaseEstimator):
                     break
 
                 # Maximization step
-                self._do_mstep(X, responsibilities, self.params, self.min_covar)
+                self._do_mstep(X, responsibilities, self.params,
+                               self.min_covar)
 
             # if the results are better, keep it
             if self.n_iter:
@@ -517,6 +466,14 @@ class GMM(BaseEstimator):
                     best_params = {'weights': self.weights_,
                                    'means': self.means_,
                                    'covars': self.covars_}
+        # check the existence of an init param that was not subject to
+        # likelihood computation issue.
+        if np.isneginf(max_log_prob) and self.n_iter:
+            raise RuntimeError(
+                "EM algorithm was never able to compute a valid likelihood " +
+                "given initial parameters. Try different init parameters " +
+                "(or increasing n_init) or check for degenerate data.")
+        # self.n_iter == 0 occurs when using GMM within HMM
         if self.n_iter:
             self.covars_ = best_params['covars']
             self.means_ = best_params['means']
@@ -535,7 +492,7 @@ class GMM(BaseEstimator):
         if 'm' in params:
             self.means_ = weighted_X_sum * inverse_weights
         if 'c' in params:
-            covar_mstep_func = _covar_mstep_funcs[self._covariance_type]
+            covar_mstep_func = _covar_mstep_funcs[self.covariance_type]
             self.covars_ = covar_mstep_func(
                 self, X, responsibilities, weighted_X_sum, inverse_weights,
                 min_covar)
@@ -544,16 +501,16 @@ class GMM(BaseEstimator):
     def _n_parameters(self):
         """Return the number of free parameters in the model."""
         ndim = self.means_.shape[1]
-        if self._covariance_type == 'full':
+        if self.covariance_type == 'full':
             cov_params = self.n_components * ndim * (ndim + 1) / 2.
-        elif self._covariance_type == 'diag':
+        elif self.covariance_type == 'diag':
             cov_params = self.n_components * ndim
-        elif self._covariance_type == 'tied':
+        elif self.covariance_type == 'tied':
             cov_params = ndim * (ndim + 1) / 2.
-        elif self._covariance_type == 'spherical':
+        elif self.covariance_type == 'spherical':
             cov_params = self.n_components
         mean_params = ndim * self.n_components
-        return  int(cov_params + mean_params + self.n_components - 1)
+        return int(cov_params + mean_params + self.n_components - 1)
 
     def bic(self, X):
         """Bayesian information criterion for the current model fit
@@ -567,8 +524,8 @@ class GMM(BaseEstimator):
         -------
         bic: float (the lower the better)
         """
-        return (- 2 * self.score(X).sum() +
-                 self._n_parameters() * np.log(X.shape[0]))
+        return (-2 * self.score(X).sum() +
+                self._n_parameters() * np.log(X.shape[0]))
 
     def aic(self, X):
         """Akaike information criterion for the current model fit
@@ -614,7 +571,7 @@ def _log_multivariate_normal_density_tied(X, means, covars):
     """Compute Gaussian log-density at X for a tied model"""
     from scipy import linalg
     n_samples, n_dim = X.shape
-    icv = linalg.pinv(covars)
+    icv = pinvh(covars)
     lpr = -0.5 * (n_dim * np.log(2 * np.pi) + np.log(linalg.det(covars) + 0.1)
                   + np.sum(X * np.dot(X, icv), 1)[:, np.newaxis]
                   - 2 * np.dot(np.dot(X, icv), means.T)
@@ -646,8 +603,8 @@ def _log_multivariate_normal_density_full(X, means, covars, min_covar=1.e-7):
                                       lower=True)
         cv_log_det = 2 * np.sum(np.log(np.diagonal(cv_chol)))
         cv_sol = solve_triangular(cv_chol, (X - mu).T, lower=True).T
-        log_prob[:, c] = - .5 * (np.sum(cv_sol ** 2, axis=1) + \
-                                     n_dim * np.log(2 * np.pi) + cv_log_det)
+        log_prob[:, c] = - .5 * (np.sum(cv_sol ** 2, axis=1) +
+                                 n_dim * np.log(2 * np.pi) + cv_log_det)
 
     return log_prob
 
@@ -683,7 +640,7 @@ def _validate_covars(covars, covariance_type, n_components):
                              "(n_components, n_dim, n_dim)")
         for n, cv in enumerate(covars):
             if (not np.allclose(cv, cv.T)
-                or np.any(linalg.eigvalsh(cv) <= 0)):
+                    or np.any(linalg.eigvalsh(cv) <= 0)):
                 raise ValueError("component %d of 'full' covars must be "
                                  "symmetric, positive-definite" % n)
     else:
@@ -692,7 +649,7 @@ def _validate_covars(covars, covariance_type, n_components):
 
 
 def distribute_covar_matrix_to_match_covariance_type(
-    tied_cv, covariance_type, n_components):
+        tied_cv, covariance_type, n_components):
     """Create all the covariance matrices from a given template
     """
     if covariance_type == 'spherical':
