@@ -1,9 +1,10 @@
-from ..base import ClassifierMixin, RegressorMixin
+from .base import BaseLibLinear, BaseSVC, BaseLibSVM
+from ..base import RegressorMixin
+from ..linear_model.base import LinearClassifierMixin
 from ..feature_selection.selector_mixin import SelectorMixin
-from .base import BaseLibLinear, BaseLibSVM
 
 
-class LinearSVC(BaseLibLinear, ClassifierMixin, SelectorMixin):
+class LinearSVC(BaseLibLinear, LinearClassifierMixin, SelectorMixin):
     """Linear Support Vector Classification.
 
     Similar to SVC with parameter kernel='linear', but implemented in terms of
@@ -16,9 +17,8 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, SelectorMixin):
 
     Parameters
     ----------
-    C : float or None, optional (default=None)
-        Penalty parameter C of the error term. If None then C is set
-        to n_samples.
+    C : float, optional (default=1.0)
+        Penalty parameter C of the error term.
 
     loss : string, 'l1' or 'l2' (default='l2')
         Specifies the loss function. 'l1' is the hinge loss (standard SVM)
@@ -33,7 +33,7 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, SelectorMixin):
         Select the algorithm to either solve the dual or primal
         optimization problem. Prefer dual=False when n_samples > n_features.
 
-    tol: float, optional (default=1e-4)
+    tol : float, optional (default=1e-4)
         Tolerance for stopping criteria
 
     multi_class: string, 'ovr' or 'crammer_singer' (default='ovr')
@@ -132,7 +132,7 @@ class LinearSVC(BaseLibLinear, ClassifierMixin, SelectorMixin):
     pass
 
 
-class SVC(BaseLibSVM, ClassifierMixin):
+class SVC(BaseSVC):
     """C-Support Vector Classification.
 
     The implementations is a based on libsvm. The fit time complexity
@@ -150,14 +150,15 @@ class SVC(BaseLibSVM, ClassifierMixin):
 
     Parameters
     ----------
-    C : float or None, optional (default=None)
-        Penalty parameter C of the error term. If None then C is set
-        to n_samples.
+    C : float, optional (default=1.0)
+        Penalty parameter C of the error term.
 
     kernel : string, optional (default='rbf')
          Specifies the kernel type to be used in the algorithm.
-         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed'.
-         If none is given, 'rbf' will be used.
+         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed' or
+         a callable.
+         If none is given, 'rbf' will be used. If a callable is given it is
+         used to precompute the kernel matrix.
 
     degree : int, optional (default=3)
         Degree of kernel function.
@@ -178,10 +179,10 @@ class SVC(BaseLibSVM, ClassifierMixin):
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
 
-    tol: float, optional (default=1e-3)
+    tol : float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
-    cache_size: float, optional
+    cache_size : float, optional
         Specify the size of the kernel cache (in MB)
 
     class_weight : {dict, 'auto'}, optional
@@ -196,6 +197,9 @@ class SVC(BaseLibSVM, ClassifierMixin):
         per-process runtime setting in libsvm that, if enabled, may not work
         properly in a multithreaded context.
 
+    max_iter : int, optional (default=-1)
+        Hard limit on iterations within solver, or -1 for no limit.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -233,10 +237,10 @@ class SVC(BaseLibSVM, ClassifierMixin):
     >>> clf = SVC()
     >>> clf.fit(X, y) #doctest: +NORMALIZE_WHITESPACE
     SVC(C=1.0, cache_size=200, class_weight=None, coef0=0.0, degree=3,
-            gamma=0.5, kernel='rbf', probability=False, shrinking=True,
-            tol=0.001, verbose=False)
-    >>> print clf.predict([[-0.8, -1]])
-    [ 1.]
+            gamma=0.0, kernel='rbf', max_iter=-1, probability=False,
+            shrinking=True, tol=0.001, verbose=False)
+    >>> print(clf.predict([[-0.8, -1]]))
+    [1]
 
     See also
     --------
@@ -253,14 +257,14 @@ class SVC(BaseLibSVM, ClassifierMixin):
     def __init__(self, C=1.0, kernel='rbf', degree=3, gamma=0.0,
                  coef0=0.0, shrinking=True, probability=False,
                  tol=1e-3, cache_size=200, class_weight=None,
-                 verbose=False):
+                 verbose=False, max_iter=-1):
 
-        super(SVC, self).__init__('c_svc', kernel, degree, gamma, coef0, tol,
-                C, 0., 0., shrinking, probability, cache_size, "auto",
-                class_weight, verbose)
+        super(SVC, self).__init__(
+            'c_svc', kernel, degree, gamma, coef0, tol, C, 0., 0., shrinking,
+            probability, cache_size, "auto", class_weight, verbose, max_iter)
 
 
-class NuSVC(BaseLibSVM, ClassifierMixin):
+class NuSVC(BaseSVC):
     """Nu-Support Vector Classification.
 
     Similar to SVC but uses a parameter to control the number of support
@@ -277,8 +281,10 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
 
     kernel : string, optional (default='rbf')
          Specifies the kernel type to be used in the algorithm.
-         one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed'.
-         If none is given 'rbf' will be used.
+         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed' or
+         a callable.
+         If none is given, 'rbf' will be used. If a callable is given it is
+         used to precompute the kernel matrix.
 
     degree : int, optional (default=3)
         degree of kernel function
@@ -299,24 +305,19 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
 
-    tol: float, optional (default=1e-3)
+    tol : float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
-    cache_size: float, optional
+    cache_size : float, optional
         Specify the size of the kernel cache (in MB)
 
-    class_weight : {dict, 'auto'}, optional
-        Set the parameter C of class i to class_weight[i]*C for
-        SVC. If not given, all classes are supposed to have
-        weight one. The 'auto' mode uses the values of y to
-        automatically adjust weights inversely proportional to
-        class frequencies.
-
     verbose : bool, default: False
         Enable verbose output. Note that this setting takes advantage of a
         per-process runtime setting in libsvm that, if enabled, may not work
         properly in a multithreaded context.
 
+    max_iter : int, optional (default=-1)
+        Hard limit on iterations within solver, or -1 for no limit.
 
     Attributes
     ----------
@@ -353,11 +354,12 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
     >>> y = np.array([1, 1, 2, 2])
     >>> from sklearn.svm import NuSVC
     >>> clf = NuSVC()
-    >>> clf.fit(X, y)
-    NuSVC(cache_size=200, coef0=0.0, degree=3, gamma=0.5, kernel='rbf', nu=0.5,
-       probability=False, shrinking=True, tol=0.001, verbose=False)
-    >>> print clf.predict([[-0.8, -1]])
-    [ 1.]
+    >>> clf.fit(X, y) #doctest: +NORMALIZE_WHITESPACE
+    NuSVC(cache_size=200, coef0=0.0, degree=3, gamma=0.0, kernel='rbf',
+            max_iter=-1, nu=0.5, probability=False, shrinking=True, tol=0.001,
+            verbose=False)
+    >>> print(clf.predict([[-0.8, -1]]))
+    [1]
 
     See also
     --------
@@ -371,11 +373,11 @@ class NuSVC(BaseLibSVM, ClassifierMixin):
 
     def __init__(self, nu=0.5, kernel='rbf', degree=3, gamma=0.0,
                  coef0=0.0, shrinking=True, probability=False,
-                 tol=1e-3, cache_size=200, verbose=False):
+                 tol=1e-3, cache_size=200, verbose=False, max_iter=-1):
 
-        super(NuSVC, self).__init__('nu_svc', kernel, degree, gamma, coef0,
-                tol, 0., nu, 0., shrinking, probability, cache_size,
-                "auto", None, verbose)
+        super(NuSVC, self).__init__(
+            'nu_svc', kernel, degree, gamma, coef0, tol, 0., nu, 0., shrinking,
+            probability, cache_size, "auto", None, verbose, max_iter)
 
 
 class SVR(BaseLibSVM, RegressorMixin):
@@ -387,9 +389,8 @@ class SVR(BaseLibSVM, RegressorMixin):
 
     Parameters
     ----------
-    C : float or None, optional (default=None)
-        penalty parameter C of the error term. If None then C is set
-        to n_samples.
+    C : float, optional (default=1.0)
+        penalty parameter C of the error term.
 
     epsilon : float, optional (default=0.1)
          epsilon in the epsilon-SVR model. It specifies the epsilon-tube
@@ -399,8 +400,10 @@ class SVR(BaseLibSVM, RegressorMixin):
 
     kernel : string, optional (default='rbf')
          Specifies the kernel type to be used in the algorithm.
-         one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed'.
-         If none is given 'rbf' will be used.
+         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed' or
+         a callable.
+         If none is given, 'rbf' will be used. If a callable is given it is
+         used to precompute the kernel matrix.
 
     degree : int, optional (default=3)
         degree of kernel function
@@ -421,10 +424,10 @@ class SVR(BaseLibSVM, RegressorMixin):
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
 
-    tol: float, optional (default=1e-3)
+    tol : float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
-    cache_size: float, optional
+    cache_size : float, optional
         Specify the size of the kernel cache (in MB)
 
     verbose : bool, default: False
@@ -432,6 +435,9 @@ class SVR(BaseLibSVM, RegressorMixin):
         per-process runtime setting in libsvm that, if enabled, may not work
         properly in a multithreaded context.
 
+    max_iter : int, optional (default=-1)
+        Hard limit on iterations within solver, or -1 for no limit.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -462,9 +468,9 @@ class SVR(BaseLibSVM, RegressorMixin):
     >>> y = np.random.randn(n_samples)
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = SVR(C=1.0, epsilon=0.2)
-    >>> clf.fit(X, y)
-    SVR(C=1.0, cache_size=200, coef0=0.0, degree=3, epsilon=0.2, gamma=0.2,
-      kernel='rbf', probability=False, shrinking=True, tol=0.001,
+    >>> clf.fit(X, y) #doctest: +NORMALIZE_WHITESPACE
+    SVR(C=1.0, cache_size=200, coef0=0.0, degree=3, epsilon=0.2, gamma=0.0,
+      kernel='rbf', max_iter=-1, probability=False, shrinking=True, tol=0.001,
       verbose=False)
 
     See also
@@ -475,12 +481,13 @@ class SVR(BaseLibSVM, RegressorMixin):
 
     """
     def __init__(self, kernel='rbf', degree=3, gamma=0.0, coef0=0.0, tol=1e-3,
-            C=1.0, epsilon=0.1, shrinking=True, probability=False,
-            cache_size=200, verbose=False):
+                 C=1.0, epsilon=0.1, shrinking=True, probability=False,
+                 cache_size=200, verbose=False, max_iter=-1):
 
-        super(SVR, self).__init__('epsilon_svr', kernel, degree, gamma, coef0,
-                tol, C, 0., epsilon, shrinking, probability, cache_size,
-                "auto", None, verbose)
+        super(SVR, self).__init__(
+            'epsilon_svr', kernel, degree, gamma, coef0, tol, C, 0., epsilon,
+            shrinking, probability, cache_size, "auto", None, verbose,
+            max_iter)
 
 
 class NuSVR(BaseLibSVM, RegressorMixin):
@@ -494,9 +501,8 @@ class NuSVR(BaseLibSVM, RegressorMixin):
 
     Parameters
     ----------
-    C : float or None, optional (default=None)
-        penalty parameter C of the error term. If None then C is set
-        to n_samples.
+    C : float, optional (default=1.0)
+        penalty parameter C of the error term.
 
     nu : float, optional
         An upper bound on the fraction of training errors and a lower bound of
@@ -505,8 +511,10 @@ class NuSVR(BaseLibSVM, RegressorMixin):
 
     kernel : string, optional (default='rbf')
          Specifies the kernel type to be used in the algorithm.
-         one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed'.
-         If none is given 'rbf' will be used.
+         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed' or
+         a callable.
+         If none is given, 'rbf' will be used. If a callable is given it is
+         used to precompute the kernel matrix.
 
     degree : int, optional (default=3)
         degree of kernel function
@@ -527,10 +535,10 @@ class NuSVR(BaseLibSVM, RegressorMixin):
     shrinking: boolean, optional (default=True)
         Whether to use the shrinking heuristic.
 
-    tol: float, optional (default=1e-3)
+    tol : float, optional (default=1e-3)
         Tolerance for stopping criterion.
 
-    cache_size: float, optional
+    cache_size : float, optional
         Specify the size of the kernel cache (in MB)
 
     verbose : bool, default: False
@@ -538,6 +546,9 @@ class NuSVR(BaseLibSVM, RegressorMixin):
         per-process runtime setting in libsvm that, if enabled, may not work
         properly in a multithreaded context.
 
+    max_iter : int, optional (default=-1)
+        Hard limit on iterations within solver, or -1 for no limit.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -568,9 +579,10 @@ class NuSVR(BaseLibSVM, RegressorMixin):
     >>> y = np.random.randn(n_samples)
     >>> X = np.random.randn(n_samples, n_features)
     >>> clf = NuSVR(C=1.0, nu=0.1)
-    >>> clf.fit(X, y)
-    NuSVR(C=1.0, cache_size=200, coef0=0.0, degree=3, gamma=0.2, kernel='rbf',
-       nu=0.1, probability=False, shrinking=True, tol=0.001, verbose=False)
+    >>> clf.fit(X, y)  #doctest: +NORMALIZE_WHITESPACE
+    NuSVR(C=1.0, cache_size=200, coef0=0.0, degree=3, gamma=0.0, kernel='rbf',
+       max_iter=-1, nu=0.1, probability=False, shrinking=True, tol=0.001,
+       verbose=False)
 
     See also
     --------
@@ -585,11 +597,11 @@ class NuSVR(BaseLibSVM, RegressorMixin):
     def __init__(self, nu=0.5, C=1.0, kernel='rbf', degree=3,
                  gamma=0.0, coef0=0.0, shrinking=True,
                  probability=False, tol=1e-3, cache_size=200,
-                 verbose=False):
+                 verbose=False, max_iter=-1):
 
-        super(NuSVR, self).__init__('nu_svr', kernel, degree, gamma, coef0,
-                tol, C, nu, 0., shrinking, probability, cache_size,
-                "auto", None, verbose)
+        super(NuSVR, self).__init__(
+            'nu_svr', kernel, degree, gamma, coef0, tol, C, nu, 0., shrinking,
+            probability, cache_size, "auto", None, verbose, max_iter)
 
 
 class OneClassSVM(BaseLibSVM):
@@ -601,10 +613,12 @@ class OneClassSVM(BaseLibSVM):
 
     Parameters
     ----------
-    kernel : string, optional
-        Specifies the kernel type to be used in
-        the algorithm. Can be one of 'linear', 'poly', 'rbf', 'sigmoid',
-        'precomputed'. If none is given 'rbf' will be used.
+    kernel : string, optional (default='rbf')
+         Specifies the kernel type to be used in the algorithm.
+         It must be one of 'linear', 'poly', 'rbf', 'sigmoid', 'precomputed' or
+         a callable.
+         If none is given, 'rbf' will be used. If a callable is given it is
+         used to precompute the kernel matrix.
 
     nu : float, optional
         An upper bound on the fraction of training
@@ -623,13 +637,13 @@ class OneClassSVM(BaseLibSVM):
         Independent term in kernel function. It is only significant in
         poly/sigmoid.
 
-    tol: float, optional
+    tol : float, optional
         Tolerance for stopping criterion.
 
     shrinking: boolean, optional
         Whether to use the shrinking heuristic.
 
-    cache_size: float, optional
+    cache_size : float, optional
         Specify the size of the kernel cache (in MB)
 
     verbose : bool, default: False
@@ -637,6 +651,9 @@ class OneClassSVM(BaseLibSVM):
         per-process runtime setting in libsvm that, if enabled, may not work
         properly in a multithreaded context.
 
+    max_iter : int, optional (default=-1)
+        Hard limit on iterations within solver, or -1 for no limit.
+
     Attributes
     ----------
     `support_` : array-like, shape = [n_SV]
@@ -660,11 +677,12 @@ class OneClassSVM(BaseLibSVM):
 
     """
     def __init__(self, kernel='rbf', degree=3, gamma=0.0, coef0=0.0, tol=1e-3,
-                 nu=0.5, shrinking=True, cache_size=200, verbose=False):
+                 nu=0.5, shrinking=True, cache_size=200, verbose=False,
+                 max_iter=-1):
 
-        super(OneClassSVM, self).__init__('one_class', kernel, degree, gamma,
-                coef0, tol, 0., nu, 0., shrinking, False, cache_size,
-                "auto", None, verbose)
+        super(OneClassSVM, self).__init__(
+            'one_class', kernel, degree, gamma, coef0, tol, 0., nu, 0.,
+            shrinking, False, cache_size, "auto", None, verbose, max_iter)
 
     def fit(self, X, sample_weight=None, **params):
         """
@@ -687,5 +705,5 @@ class OneClassSVM(BaseLibSVM):
 
         """
         super(OneClassSVM, self).fit(X, [], sample_weight=sample_weight,
-                **params)
+                                     **params)
         return self
