@@ -54,6 +54,18 @@ class TestBaseHMM(TestCase):
             for p in params:
                 assert_array_almost_equal(getattr(h, p), getattr(h2, p))
 
+    def test_set_startprob(self):
+        h, framelogprob = self.setup_example_hmm()
+        startprob = np.array([0.0, 1.0])
+        h.startprob_ = startprob
+        assert np.allclose(startprob, h.startprob_)
+
+    def test_set_transmat(self):
+        h, framelogprob = self.setup_example_hmm()
+        transmat = np.array([[0.8, 0.2], [0.0, 1.0]])
+        h.transmat_ = transmat
+        assert np.allclose(transmat, h.transmat_)
+
     def test_do_forward_pass(self):
         h, framelogprob = self.setup_example_hmm()
 
@@ -161,7 +173,7 @@ class TestBaseHMM(TestCase):
 
         h = self.StubHMM(n_components)
 
-        self.assertEquals(h.n_components, n_components)
+        self.assertEqual(h.n_components, n_components)
 
         h.startprob_ = startprob
         assert_array_almost_equal(h.startprob_, startprob)
@@ -182,10 +194,13 @@ class TestBaseHMM(TestCase):
 
 
 def train_hmm_and_keep_track_of_log_likelihood(hmm, obs, n_iter=1, **kwargs):
-    hmm.fit(obs, n_iter=1, **kwargs)
+    hmm.n_iter = 1
+    hmm.fit(obs)
     loglikelihoods = []
     for n in xrange(n_iter):
-        hmm.fit(obs, n_iter=1, init_params='', **kwargs)
+        hmm.n_iter = 1
+        hmm.init_params = ''
+        hmm.fit(obs)
         loglikelihoods.append(sum(hmm.score(x) for x in obs))
     return loglikelihoods
 
@@ -200,7 +215,7 @@ class GaussianHMMBaseTester(object):
         self.startprob = self.startprob / self.startprob.sum()
         self.transmat = prng.rand(n_components, n_components)
         self.transmat /= np.tile(self.transmat.sum(axis=1)[:, np.newaxis],
-                (1, n_components))
+                                 (1, n_components))
         self.means = prng.randint(-20, 20, (n_components, n_features))
         self.covars = {
             'spherical': (1.0 + 2 * np.dot(prng.rand(n_components, 1),
@@ -223,43 +238,7 @@ class GaussianHMMBaseTester(object):
     def test_bad_covariance_type(self):
         hmm.GaussianHMM(20, self.covariance_type)
         self.assertRaises(ValueError, hmm.GaussianHMM, 20,
-                'badcovariance_type')
-
-    def _test_attributes(self):
-        # XXX: This test is bugged and creates weird errors -- skipped
-        h = hmm.GaussianHMM(self.n_components, self.covariance_type)
-
-        self.assertEquals(h.n_components, self.n_components)
-        self.assertEquals(h.covariance_type, self.covariance_type)
-
-        h.startprob_ = self.startprob
-        assert_array_almost_equal(h.startprob_, self.startprob)
-        self.assertRaises(ValueError, h.__setattr__, 'startprob_',
-                          2 * self.startprob)
-        self.assertRaises(ValueError, h.__setattr__, 'startprob_', [])
-        self.assertRaises(ValueError, h.__setattr__, 'startprob_',
-                          np.zeros((self.n_components - 2, self.n_features)))
-
-        h.transmat_ = self.transmat
-        assert_array_almost_equal(h.transmat_, self.transmat)
-        self.assertRaises(ValueError, h.__setattr__, 'transmat_',
-                          2 * self.transmat)
-        self.assertRaises(ValueError, h.__setattr__, 'transmat_', [])
-        self.assertRaises(ValueError, h.__setattr__, 'transmat_',
-                          np.zeros((self.n_components - 2, self.n_components)))
-
-        h.means_ = self.means
-        self.assertEquals(h.n_features, self.n_features)
-        self.assertRaises(ValueError, h.__setattr__, 'means_', [])
-        self.assertRaises(ValueError, h.__setattr__, 'means_',
-                          np.zeros((self.n_components - 2, self.n_features)))
-
-        h.covars_ = self.covars[self.covariance_type]
-        assert_array_almost_equal(h.covars_,
-                self.expanded_covars[self.covariance_type])
-        #self.assertRaises(ValueError, h.__setattr__, 'covars', [])
-        #self.assertRaises(ValueError, h.__setattr__, 'covars',
-        #                  np.zeros((self.n_components - 2, self.n_features)))
+                          'badcovariance_type')
 
     def test_eval_and_decode(self):
         h = hmm.GaussianHMM(self.n_components, self.covariance_type)
@@ -291,13 +270,13 @@ class GaussianHMMBaseTester(object):
         h.startprob_ = self.startprob
 
         samples = h.sample(n)[0]
-        self.assertEquals(samples.shape, (n, self.n_features))
+        self.assertEqual(samples.shape, (n, self.n_features))
 
-    def test_fit(self, params='stmc', n_iter=25, verbose=False, **kwargs):
+    def test_fit(self, params='stmc', n_iter=5, verbose=False, **kwargs):
         h = hmm.GaussianHMM(self.n_components, self.covariance_type)
         h.startprob_ = self.startprob
-        h.transmat_ = hmm.normalize(self.transmat
-                + np.diag(self.prng.rand(self.n_components)), 1)
+        h.transmat_ = hmm.normalize(
+            self.transmat + np.diag(self.prng.rand(self.n_components)), 1)
         h.means_ = 20 * self.means
         h.covars_ = self.covars[self.covariance_type]
 
@@ -305,7 +284,8 @@ class GaussianHMMBaseTester(object):
         train_obs = [h.sample(n=10)[0] for x in xrange(10)]
 
         # Mess up the parameters and see if we can re-learn them.
-        h.fit(train_obs, n_iter=0)
+        h.n_iter = 0
+        h.fit(train_obs)
 
         trainll = train_hmm_and_keep_track_of_log_likelihood(
             h, train_obs, n_iter=n_iter, params=params, **kwargs)[1:]
@@ -333,7 +313,7 @@ class GaussianHMMBaseTester(object):
         # ValueError: setting an array element with a sequence.
         h.fit(obs)
 
-    def test_fit_with_priors(self, params='stmc', n_iter=10, verbose=False):
+    def test_fit_with_priors(self, params='stmc', n_iter=5, verbose=False):
         startprob_prior = 10 * self.startprob + 2.0
         transmat_prior = 10 * self.transmat + 2.0
         means_prior = self.means
@@ -346,8 +326,8 @@ class GaussianHMMBaseTester(object):
         h = hmm.GaussianHMM(self.n_components, self.covariance_type)
         h.startprob_ = self.startprob
         h.startprob_prior = startprob_prior
-        h.transmat_ = hmm.normalize(self.transmat
-                + np.diag(self.prng.rand(self.n_components)), 1)
+        h.transmat_ = hmm.normalize(
+            self.transmat + np.diag(self.prng.rand(self.n_components)), 1)
         h.transmat_prior = transmat_prior
         h.means_ = 20 * self.means
         h.means_prior = means_prior
@@ -360,7 +340,8 @@ class GaussianHMMBaseTester(object):
         train_obs = [h.sample(n=10)[0] for x in xrange(10)]
 
         # Mess up the parameters and see if we can re-learn them.
-        h.fit(train_obs[:1], n_iter=0)
+        h.n_iter = 0
+        h.fit(train_obs[:1])
 
         trainll = train_hmm_and_keep_track_of_log_likelihood(
             h, train_obs, n_iter=n_iter, params=params)[1:]
@@ -373,6 +354,25 @@ class GaussianHMMBaseTester(object):
         # XXX: Why such a large tolerance?
         self.assertTrue(np.all(np.diff(trainll) > -0.5))
 
+    def test_fit_non_ergodic_transmat(self):
+        startprob = np.array([1, 0, 0, 0, 0])
+        transmat = np.array([[0.9, 0.1, 0, 0, 0],
+                             [0, 0.9, 0.1, 0, 0],
+                             [0, 0, 0.9, 0.1, 0],
+                             [0, 0, 0, 0.9, 0.1],
+                             [0, 0, 0, 0, 1.0]])
+
+        h = hmm.GaussianHMM(n_components=5,
+                            covariance_type='full', startprob=startprob,
+                            transmat=transmat, n_iter=100, init_params='st')
+
+        h.means_ = np.zeros((5, 10))
+        h.covars_ = np.tile(np.identity(10), (5, 1, 1))
+
+        obs = [h.sample(10)[0] for _ in range(10)]
+
+        h.fit(obs=obs)
+
 
 class TestGaussianHMMWithSphericalCovars(GaussianHMMBaseTester, TestCase):
     covariance_type = 'spherical'
@@ -409,10 +409,16 @@ class MultinomialHMMTestCase(TestCase):
         self.transmat = [[0.7, 0.3], [0.4, 0.6]]
 
         self.h = hmm.MultinomialHMM(self.n_components,
-                               startprob=self.startprob,
-                               transmat=self.transmat)
+                                    startprob=self.startprob,
+                                    transmat=self.transmat)
         self.h.emissionprob_ = self.emissionprob
 
+    def test_set_emissionprob(self):
+        h = hmm.MultinomialHMM(self.n_components)
+        emissionprob = np.array([[0.8, 0.2, 0.0], [0.7, 0.2, 1.0]])
+        h.emissionprob = emissionprob
+        assert np.allclose(emissionprob, h.emissionprob)
+
     def test_wikipedia_viterbi_example(self):
         # From http://en.wikipedia.org/wiki/Viterbi_algorithm:
         # "This reveals that the observations ['walk', 'shop', 'clean']
@@ -423,10 +429,29 @@ class MultinomialHMMTestCase(TestCase):
         self.assertAlmostEqual(np.exp(logprob), 0.01344)
         assert_array_equal(state_sequence, [1, 0, 0])
 
+    def test_decode_map_algorithm(self):
+        observations = [0, 1, 2]
+        h = hmm.MultinomialHMM(self.n_components, startprob=self.startprob,
+                               transmat=self.transmat, algorithm="map",)
+        h.emissionprob_ = self.emissionprob
+        logprob, state_sequence = h.decode(observations)
+        assert_array_equal(state_sequence, [1, 0, 0])
+
+    def test_predict(self):
+        observations = [0, 1, 2]
+        state_sequence = self.h.predict(observations)
+        posteriors = self.h.predict_proba(observations)
+        assert_array_equal(state_sequence, [1, 0, 0])
+        assert_array_almost_equal(posteriors, [
+            [0.23170303, 0.76829697],
+            [0.62406281, 0.37593719],
+            [0.86397706, 0.13602294],
+        ])
+
     def test_attributes(self):
         h = hmm.MultinomialHMM(self.n_components)
 
-        self.assertEquals(h.n_components, self.n_components)
+        self.assertEqual(h.n_components, self.n_components)
 
         h.startprob_ = self.startprob
         assert_array_almost_equal(h.startprob_, self.startprob)
@@ -449,7 +474,7 @@ class MultinomialHMMTestCase(TestCase):
         self.assertRaises(ValueError, h.__setattr__, 'emissionprob_', [])
         self.assertRaises(ValueError, h.__setattr__, 'emissionprob_',
                           np.zeros((self.n_components - 2, self.n_symbols)))
-        self.assertEquals(h.n_symbols, self.n_symbols)
+        self.assertEqual(h.n_symbols, self.n_symbols)
 
     def test_eval(self):
         idx = np.repeat(range(self.n_components), 10)
@@ -463,10 +488,10 @@ class MultinomialHMMTestCase(TestCase):
 
     def test_sample(self, n=1000):
         samples = self.h.sample(n)[0]
-        self.assertEquals(len(samples), n)
-        self.assertEquals(len(np.unique(samples)), self.n_symbols)
+        self.assertEqual(len(samples), n)
+        self.assertEqual(len(np.unique(samples)), self.n_symbols)
 
-    def test_fit(self, params='ste', n_iter=15, verbose=False, **kwargs):
+    def test_fit(self, params='ste', n_iter=5, verbose=False, **kwargs):
         h = self.h
 
         # Create training data by sampling from the HMM.
@@ -475,7 +500,7 @@ class MultinomialHMMTestCase(TestCase):
         # Mess up the parameters and see if we can re-learn them.
         h.startprob_ = hmm.normalize(self.prng.rand(self.n_components))
         h.transmat_ = hmm.normalize(self.prng.rand(self.n_components,
-                                                  self.n_components), axis=1)
+                                                   self.n_components), axis=1)
         h.emissionprob_ = hmm.normalize(
             self.prng.rand(self.n_components, self.n_symbols), axis=1)
 
@@ -487,11 +512,32 @@ class MultinomialHMMTestCase(TestCase):
             print
             print 'Test train: (%s)\n  %s\n  %s' % (params, trainll,
                                                     np.diff(trainll))
-        self.assertTrue(np.all(np.diff(trainll) > - 1.e-3))
+        self.assertTrue(np.all(np.diff(trainll) > -1.e-3))
 
     def test_fit_emissionprob(self):
         self.test_fit('e')
 
+    def test_fit_with_init(self, params='ste', n_iter=5, verbose=False,
+                           **kwargs):
+        h = self.h
+        learner = hmm.MultinomialHMM(self.n_components)
+
+        # Create training data by sampling from the HMM.
+        train_obs = [h.sample(n=10)[0] for x in xrange(10)]
+
+        # use init_function to initialize paramerters
+        learner._init(train_obs, params)
+
+        trainll = train_hmm_and_keep_track_of_log_likelihood(
+            learner, train_obs, n_iter=n_iter, params=params, **kwargs)[1:]
+
+        # Check that the loglik is always increasing during training
+        if not np.all(np.diff(trainll) > 0) and verbose:
+            print
+            print 'Test train: (%s)\n  %s\n  %s' % (params, trainll,
+                                                    np.diff(trainll))
+        self.assertTrue(np.all(np.diff(trainll) > -1.e-3))
+
 
 def create_random_gmm(n_mix, n_features, covariance_type, prng=0):
     prng = check_random_state(prng)
@@ -526,16 +572,16 @@ class GMMHMMBaseTester(object):
         self.transmat /= np.tile(self.transmat.sum(axis=1)[:, np.newaxis],
                                  (1, self.n_components))
 
-        self.gmms = []
+        self.gmms_ = []
         for state in xrange(self.n_components):
-            self.gmms.append(create_random_gmm(
+            self.gmms_.append(create_random_gmm(
                 self.n_mix, self.n_features, self.covariance_type,
                 prng=self.prng))
 
     def test_attributes(self):
         h = hmm.GMMHMM(self.n_components, covariance_type=self.covariance_type)
 
-        self.assertEquals(h.n_components, self.n_components)
+        self.assertEqual(h.n_components, self.n_components)
 
         h.startprob_ = self.startprob
         assert_array_almost_equal(h.startprob_, self.startprob)
@@ -554,15 +600,15 @@ class GMMHMMBaseTester(object):
                           np.zeros((self.n_components - 2, self.n_components)))
 
     def test_eval_and_decode(self):
-        h = hmm.GMMHMM(self.n_components, gmms=self.gmms)
+        h = hmm.GMMHMM(self.n_components, gmms=self.gmms_)
         # Make sure the means are far apart so posteriors.argmax()
         # picks the actual component used to generate the observations.
-        for g in h.gmms:
+        for g in h.gmms_:
             g.means_ *= 20
 
         refstateseq = np.repeat(range(self.n_components), 5)
         nobs = len(refstateseq)
-        obs = [h.gmms[x].sample(1).flatten() for x in refstateseq]
+        obs = [h.gmms_[x].sample(1).flatten() for x in refstateseq]
 
         ll, posteriors = h.eval(obs)
 
@@ -575,23 +621,24 @@ class GMMHMMBaseTester(object):
     def test_sample(self, n=1000):
         h = hmm.GMMHMM(self.n_components, self.covariance_type,
                        startprob=self.startprob, transmat=self.transmat,
-                       gmms=self.gmms)
+                       gmms=self.gmms_)
         samples = h.sample(n)[0]
-        self.assertEquals(samples.shape, (n, self.n_features))
+        self.assertEqual(samples.shape, (n, self.n_features))
 
     def test_fit(self, params='stmwc', n_iter=5, verbose=False, **kwargs):
         h = hmm.GMMHMM(self.n_components, covars_prior=1.0)
         h.startprob_ = self.startprob
         h.transmat_ = hmm.normalize(
             self.transmat + np.diag(self.prng.rand(self.n_components)), 1)
-        h.gmms = self.gmms
+        h.gmms_ = self.gmms_
 
         # Create training data by sampling from the HMM.
-        train_obs = [h.sample(n=10,
-            random_state=self.prng)[0] for x in xrange(10)]
+        train_obs = [h.sample(n=10, random_state=self.prng)[0]
+                     for x in xrange(10)]
 
         # Mess up the parameters and see if we can re-learn them.
-        h.fit(train_obs, n_iter=0)
+        h.n_iter = 0
+        h.fit(train_obs)
         h.transmat_ = hmm.normalize(self.prng.rand(self.n_components,
                                                    self.n_components), axis=1)
         h.startprob_ = hmm.normalize(self.prng.rand(self.n_components))
