@@ -1,8 +1,10 @@
 import numpy as np
-from numpy.testing import assert_array_equal, assert_array_almost_equal
-from nose.tools import assert_true
 
-from .. import qda
+from sklearn.utils.testing import assert_array_equal
+from sklearn.utils.testing import assert_array_almost_equal
+from sklearn.utils.testing import assert_true
+
+from sklearn import qda
 
 # Data is just 6 separable points in the plane
 X = np.array([[0, 0], [-2, -2], [-2, -1], [-1, -1], [-1, -2],
@@ -39,3 +41,29 @@ def test_qda():
     y_pred3 = clf.fit(X, y3).predict(X)
     # QDA shouldn't be able to separate those
     assert_true(np.any(y_pred3 != y3))
+
+
+def test_qda_priors():
+    clf = qda.QDA(priors=np.array([0.0, 1.0]))
+    y_pred = clf.fit(X, y).predict(X)
+    assert (y_pred == 2).all()
+
+
+def test_qda_store_covariances():
+    # The default is to not set the covariances_ attribute
+    clf = qda.QDA().fit(X, y)
+    assert_true(not hasattr(clf, 'covariances_'))
+
+    # Test the actual attribute:
+    clf = qda.QDA().fit(X, y, store_covariances=True)
+    assert_true(hasattr(clf, 'covariances_'))
+
+    assert_array_almost_equal(
+        clf.covariances_[0],
+        np.array([[0.7, 0.45], [0.45, 0.7]])
+    )
+
+    assert_array_almost_equal(
+        clf.covariances_[1],
+        np.array([[0.33333333, -0.33333333], [-0.33333333, 0.66666667]])
+    )
