@@ -8,3 +8,6 @@ from .tree import DecisionTreeRegressor
 from .tree import ExtraTreeClassifier
 from .tree import ExtraTreeRegressor
 from .tree import export_graphviz
+
+__all__ = ["DecisionTreeClassifier", "DecisionTreeRegressor",
+           "ExtraTreeClassifier", "ExtraTreeRegressor", "export_graphviz"]
