@@ -10,15 +10,16 @@ at which the fixe is no longer needed.
 # License: BSD
 
 import collections
-import numpy as np
 from operator import itemgetter
+import inspect
 
+import numpy as np
 
 try:
     Counter = collections.Counter
 except AttributeError:
-# Partial replacement for Python 2.7 collections.Counter
     class Counter(collections.defaultdict):
+        """Partial replacement for Python 2.7 collections.Counter."""
         def __init__(self, iterable=(), **kwargs):
             super(Counter, self).__init__(int, **kwargs)
             self.update(iterable)
@@ -99,13 +100,36 @@ def _unique(ar, return_index=False, return_inverse=False):
         flag = np.concatenate(([True], ar[1:] != ar[:-1]))
         return ar[flag]
 
-np_version = np.__version__.split('.')
-if int(np_version[0]) < 2 and int(np_version[1]) < 5:
+np_version = []
+for x in np.__version__.split('.'):
+    try:
+        np_version.append(int(x))
+    except ValueError:
+        # x may be of the form dev-1ea1592
+        np_version.append(x)
+np_version = tuple(np_version)
+
+if np_version[:2] < (1, 5):
     unique = _unique
 else:
     unique = np.unique
 
 
+def _bincount(X, weights=None, minlength=None):
+    """Replacing np.bincount in numpy < 1.6 to provide minlength."""
+    result = np.bincount(X, weights)
+    if len(result) >= minlength:
+        return result
+    out = np.zeros(minlength, np.int)
+    out[:len(result)] = result
+    return out
+
+if np_version[:2] < (1, 6):
+    bincount = _bincount
+else:
+    bincount = np.bincount
+
+
 def _copysign(x1, x2):
     """Slow replacement for np.copysign, which was introduced in numpy 1.4"""
     return np.abs(x1) * np.sign(x2)
@@ -152,7 +176,10 @@ def qr_economic(A, **kwargs):
     if hasattr(scipy.linalg, 'solve_triangular'):
         return scipy.linalg.qr(A, mode='economic', **kwargs)
     else:
-        return scipy.linalg.qr(A, econ=True, **kwargs)
+        import warnings
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", DeprecationWarning)
+            return scipy.linalg.qr(A, econ=True, **kwargs)
 
 
 def savemat(file_name, mdict, oned_as="column", **kwargs):
@@ -174,19 +201,12 @@ except ImportError:
     def count_nonzero(X):
         return len(np.flatnonzero(X))
 
-try:
-    # check whether np.dot supports the out argument
-    np.dot(np.zeros(1), np.zeros(1), out=np.empty(1))
-
-    # this is ok, just use the existing implementation
-    dot_out = np.dot
-
-except (TypeError, ValueError):
-    # old version of np.dot that does not accept the third argument, define a
-    # pure python workaround:
-    def dot_out(a, b, out=None):
-        if out is not None:
-            out[:] = np.dot(a, b)
-            return out
-        else:
-            return np.dot(a, b)
+# little danse to see if np.copy has an 'order' keyword argument
+if 'order' in inspect.getargspec(np.copy)[0]:
+    def safe_copy(X):
+        # Copy, but keep the order
+        return np.copy(X, order='K')
+else:
+    # Before an 'order' argument was introduced, numpy wouldn't muck with
+    # the ordering
+    safe_copy = np.copy
