@@ -1,10 +1,21 @@
-"""
-Utilities for input validation
-"""
+"""Utilities for input validation"""
+# Authors: Olivier Grisel and Gael Varoquaux and others (please update me)
+# License: BSD 3
+
+import warnings
+import numbers
 
 import numpy as np
-import scipy.sparse as sp
-import warnings
+from scipy import sparse
+
+from .fixes import safe_copy
+
+
+def _assert_all_finite(X):
+    """Like assert_all_finite, but only for ndarray."""
+    if (X.dtype.char in np.typecodes['AllFloat'] and not np.isfinite(X.sum())
+            and not np.isfinite(X).all()):
+        raise ValueError("Array contains NaN or infinity.")
 
 
 def assert_all_finite(X):
@@ -15,18 +26,18 @@ def assert_all_finite(X):
     # First try an O(n) time, O(1) space solution for the common case that
     # there everything is finite; fall back to O(n) space np.isfinite to
     # prevent false positives from overflow in sum method.
-    if X.dtype.char in np.typecodes['AllFloat'] and not np.isfinite(X.sum()) \
-      and not np.isfinite(X).all():
-            raise ValueError("array contains NaN or infinity")
+    _assert_all_finite(X.data if sparse.issparse(X) else X)
 
 
 def safe_asarray(X, dtype=None, order=None):
     """Convert X to an array or sparse matrix.
 
     Prevents copying X when possible; sparse matrices are passed through."""
-    if not sp.issparse(X):
+    if sparse.issparse(X):
+        assert_all_finite(X.data)
+    else:
         X = np.asarray(X, dtype, order)
-    assert_all_finite(X)
+        assert_all_finite(X)
     return X
 
 
@@ -39,7 +50,7 @@ def as_float_array(X, copy=True):
 
     Parameters
     ----------
-    X : array
+    X : {array-like, sparse matrix}
 
     copy : bool, optional
         If True, a copy of X will be created. If False, a copy may still be
@@ -47,42 +58,62 @@ def as_float_array(X, copy=True):
 
     Returns
     -------
-    X : array
+    XT : {array, sparse matrix}
         An array of type np.float
     """
-    if isinstance(X, np.matrix):
-        X = X.A
-    elif not isinstance(X, np.ndarray) and not sp.issparse(X):
+    if isinstance(X, np.matrix) or (not isinstance(X, np.ndarray)
+                                    and not sparse.issparse(X)):
         return safe_asarray(X, dtype=np.float64)
-    if X.dtype in [np.float32, np.float64]:
+    elif sparse.issparse(X) and X.dtype in [np.float32, np.float64]:
         return X.copy() if copy else X
-    if X.dtype == np.int32:
-        X = X.astype(np.float32)
+    elif X.dtype in [np.float32, np.float64]:  # is numpy array
+        return X.copy('F' if X.flags['F_CONTIGUOUS'] else 'C') if copy else X
     else:
-        X = X.astype(np.float64)
+        return X.astype(np.float32 if X.dtype == np.int32 else np.float64)
+
+
+def array2d(X, dtype=None, order=None, copy=False):
+    """Returns at least 2-d array with data from X"""
+    if sparse.issparse(X):
+        raise TypeError('A sparse matrix was passed, but dense data '
+                        'is required. Use X.toarray() to convert to dense.')
+    X_2d = np.asarray(np.atleast_2d(X), dtype=dtype, order=order)
+    _assert_all_finite(X_2d)
+    if X is X_2d and copy:
+        X_2d = safe_copy(X_2d)
+    return X_2d
+
+
+def _atleast2d_or_sparse(X, dtype, order, copy, sparse_class, convmethod):
+    if sparse.issparse(X):
+        # Note: order is ignored because CSR matrices hold data in 1-d arrays
+        if dtype is None or X.dtype == dtype:
+            X = getattr(X, convmethod)()
+        else:
+            X = sparse_class(X, dtype=dtype)
+        _assert_all_finite(X.data)
+    else:
+        X = array2d(X, dtype=dtype, order=order, copy=copy)
+        _assert_all_finite(X)
     return X
 
 
-def array2d(X, dtype=None, order=None):
-    """Returns at least 2-d array with data from X"""
-    return np.asarray(np.atleast_2d(X), dtype=dtype, order=order)
+def atleast2d_or_csc(X, dtype=None, order=None, copy=False):
+    """Like numpy.atleast_2d, but converts sparse matrices to CSC format.
+
+    Also, converts np.matrix to np.ndarray.
+    """
+    return _atleast2d_or_sparse(X, dtype, order, copy, sparse.csc_matrix,
+                                "tocsc")
 
 
-def atleast2d_or_csr(X, dtype=None, order=None):
+def atleast2d_or_csr(X, dtype=None, order=None, copy=False):
     """Like numpy.atleast_2d, but converts sparse matrices to CSR format
 
     Also, converts np.matrix to np.ndarray.
     """
-    if sp.issparse(X):
-        # Note: order is ignored because CSR matrices hold data in 1-d arrays
-        if dtype is None or X.dtype == dtype:
-            X = X.tocsr()
-        else:
-            X = sp.csr_matrix(X, dtype=dtype)
-    else:
-        X = array2d(X, dtype=dtype, order=order)
-    assert_all_finite(X)
-    return X
+    return _atleast2d_or_sparse(X, dtype, order, copy, sparse.csr_matrix,
+                                "tocsr")
 
 
 def _num_samples(x):
@@ -93,17 +124,31 @@ def _num_samples(x):
 
 
 def check_arrays(*arrays, **options):
-    """Checked that all arrays have consistent first dimensions
+    """Check that all arrays have consistent first dimensions.
+
+    Checks whether all objects in arrays have the same shape or length.
+    By default lists and tuples are converted to numpy arrays.
+
+    It is possible to enforce certain properties, such as dtype, continguity
+    and sparse matrix format (if a sparse matrix is passed).
+
+    Converting lists to arrays can be disabled by setting ``allow_lists=True``.
+    Lists can then contain arbitrary objects and are not checked for dtype,
+    finiteness or anything else but length. Arrays are still checked
+    and possibly converted.
+
 
     Parameters
     ----------
     *arrays : sequence of arrays or scipy.sparse matrices with same shape[0]
         Python lists or tuples occurring in arrays are converted to 1D numpy
-        arrays.
+        arrays, unless allow_lists is specified.
 
-    sparse_format : 'csr' or 'csc', None by default
+    sparse_format : 'csr', 'csc' or 'dense', None by default
         If not None, any scipy.sparse matrix is converted to
         Compressed Sparse Rows or Compressed Sparse Columns representations.
+        If 'dense', an error is raised when a sparse array is
+        passed.
 
     copy : boolean, False by default
         If copy is True, ensure that returned arrays are copies of the original
@@ -114,13 +159,18 @@ def check_arrays(*arrays, **options):
 
     dtype : a numpy dtype instance, None by default
         Enforce a specific dtype.
+
+    allow_lists : bool
+        Allow lists of arbitrary objects as input, just check their length.
+        Disables
     """
     sparse_format = options.pop('sparse_format', None)
-    if sparse_format not in (None, 'csr', 'csc'):
+    if sparse_format not in (None, 'csr', 'csc', 'dense'):
         raise ValueError('Unexpected sparse format: %r' % sparse_format)
     copy = options.pop('copy', False)
     check_ccontiguous = options.pop('check_ccontiguous', False)
     dtype = options.pop('dtype', None)
+    allow_lists = options.pop('allow_lists', False)
     if options:
         raise TypeError("Unexpected keyword arguments: %r" % options.keys())
 
@@ -136,27 +186,33 @@ def check_arrays(*arrays, **options):
             # special case: ignore optional y=None kwarg pattern
             checked_arrays.append(array)
             continue
-
         size = _num_samples(array)
 
         if size != n_samples:
-            raise ValueError("Found array with dim %d. Expected %d" % (
-                size, n_samples))
+            raise ValueError("Found array with dim %d. Expected %d"
+                             % (size, n_samples))
 
-        if sp.issparse(array):
-            if sparse_format == 'csr':
-                array = array.tocsr()
-            elif sparse_format == 'csc':
-                array = array.tocsc()
-            if check_ccontiguous:
-                array.data = np.ascontiguousarray(array.data, dtype=dtype)
+        if not allow_lists or hasattr(array, "shape"):
+            if sparse.issparse(array):
+                if sparse_format == 'csr':
+                    array = array.tocsr()
+                elif sparse_format == 'csc':
+                    array = array.tocsc()
+                elif sparse_format == 'dense':
+                    raise TypeError('A sparse matrix was passed, but dense '
+                                    'data is required. Use X.toarray() to '
+                                    'convert to a dense numpy array.')
+                if check_ccontiguous:
+                    array.data = np.ascontiguousarray(array.data, dtype=dtype)
+                else:
+                    array.data = np.asarray(array.data, dtype=dtype)
+                _assert_all_finite(array.data)
             else:
-                array.data = np.asarray(array.data, dtype=dtype)
-        else:
-            if check_ccontiguous:
-                array = np.ascontiguousarray(array, dtype=dtype)
-            else:
-                array = np.asarray(array, dtype=dtype)
+                if check_ccontiguous:
+                    array = np.ascontiguousarray(array, dtype=dtype)
+                else:
+                    array = np.asarray(array, dtype=dtype)
+                _assert_all_finite(array)
 
         if copy and array is array_orig:
             array = array.copy()
@@ -184,7 +240,7 @@ def check_random_state(seed):
     """
     if seed is None or seed is np.random:
         return np.random.mtrand._rand
-    if isinstance(seed, int):
+    if isinstance(seed, numbers.Integral):
         return np.random.RandomState(seed)
     if isinstance(seed, np.random.RandomState):
         return seed
